// -*- C++ -*-

// ダイクストラ法のデモ

#include "X_Depot.hh"
#include "X_Station.hh"


X_VirtualCyclicDepot :: X_VirtualCyclicDepot
( string const & _n_in )
  : X_Depot_Base ( _n_in ) ,
    _tsetc ()
{
  _tsetc = new X_TrainSet_Container () ;
}



X_VirtualCyclicDepot :: X_VirtualCyclicDepot
( TiXmlNode const * const & curnode )
  : X_Depot_Base ( curnode ) ,
    _tsetc ()
{
  if ( ! curnode )
  {
    cerr << "Error: node not set in X_VirtualCyclicDepot constructor."
         << endl ;
    exit ( 1 ) ;
  }
  if ( curnode -> ValueStr () != "data" )
  {
    cerr << "Error: <" << curnode -> ValueStr () << "> tag not allowed in "
         << "X_VirtualCyclicDepot constructor." << endl ;
    exit ( 1 ) ;
  }

  // -----
  // (1) Get all attributes of this "data" tag.
  // -----
  map < string , string > _a_m ;
  getAttributeMap ( curnode , _a_m ) ;
  string _ein_msg = "in initialisation of an X_VirtualCyclicDepot instance" ;

  // -----
  // (2) Find attribute "class" in the map. Required (already checked ---
  // just discard it here).
  // -----
  map < string , string > :: iterator ii = findAttributeInMap
    ( _a_m , "class" , _ein_msg , true ) ;
  _a_m . erase ( ii ) ;

  // -----
  // (3) Find attribute "name" in the map. Required.
  // -----
  ii = findAttributeInMap ( _a_m , "name" , _ein_msg , true ) ;
  setName ( ii -> second ) ;
  _a_m . erase ( ii ) ;

  // -----
  // (4) Check if attribute map is exhausted.
  // -----
  if ( _a_m . size () )
  {
    ii = _a_m . begin () ;
    cerr << "Error: " << _ein_msg << ": incorrect attribute \""
         << ii -> first << "\" found" << endl ;
    exit ( 1 ) ;
  }

  // ----
  // Setting a train container.
  // ----
  _tsetc = new X_TrainSet_Container () ;
}



RS_ObjectContainer *
X_VirtualCyclicDepot :: getContainer
( string const & x )
{
  if ( x != "sujic_trainset" )
  {
    cerr << "Error: No class \"" << x << "\" container found in "
         << "X_VirtualCyclicDepot" << endl ;
    exit ( 1 ) ;
  }
  return _tsetc ;
}




RS_ObjectContainer const *
X_VirtualCyclicDepot :: getContainer
( string const & x )
  const
{
  if ( x != "sujic_trainset" )
  {
    cerr << "Error: No class \"" << x << "\" container found in "
         << "X_VirtualCyclicDepot" << endl ;
    exit ( 1 ) ;
  }
  return _tsetc ;
}




bool
X_Depot_Container :: create
( TiXmlNode const * & _n_in ,
  vector < string > const & _s_in ,
  vector < RS_Object * > const & _o_in )
{
  string xxx = getClassString ( _n_in ) ;
  string yyy = getNameString ( _n_in ) ;
  if ( find ( yyy ) != end () )
  {
    cerr << "Error: Depot container already has a Depot instance "
         << "of name \"" << xxx << "\"" << endl ;
    exit ( 1 ) ;
  }
  X_Depot_Base * x = Depot_Expander :: create ( xxx , _n_in ) ;
  ( * this ) [ yyy ] = x ;
  vector < string > :: const_iterator ii = _s_in . begin () ;
  vector < RS_Object * > :: const_iterator jj = _o_in . begin () ;
  while ( ii < _s_in . end () || jj < _o_in . end () )
  {
    if ( x -> acceptInitObject ( * ii , * jj ) )
    {
      ++ ii ;
      ++ jj ;
    }
    else
    {
      cerr << "Error: acceptInitObject failed, type = "
           << * ii << endl ;
      exit ( 1 ) ;
    }
  }
  return true ;
}




// -----
// Locate a depot (const).
// -----
X_Depot_Base const *
X_Depot_Container :: locateDepot
( string const & x )
  const
{
  RS_Object const * retv = locate ( x ) ;
  return dynamic_cast < X_Depot_Base const * > ( retv ) ;
}




// -----
// Locate a depot (non-const).
// -----
X_Depot_Base *
X_Depot_Container :: locateDepot
( string const & x )
{
  RS_Object * retv = locate ( x ) ;
  return dynamic_cast < X_Depot_Base * > ( retv ) ;
}




// -----
// Locate a trainset in a depot (non-const).
// -----
X_TrainSet_Base *
X_VirtualCyclicDepot :: locateTrainSet
( string const & x )
{
  return dynamic_cast < X_TrainSet_Container * > ( _tsetc )
    -> locateTrainSet ( x ) ;
}




// -----
// Locate a trainset in a depot (const).
// -----
X_TrainSet_Base const *
X_VirtualCyclicDepot :: locateTrainSet
( string const & x )
  const
{
  return dynamic_cast < X_TrainSet_Container const * > ( _tsetc )
    -> locateTrainSet ( x ) ;
}
