// -*- C++ -*-

// ダイクストラ法のデモ

#include "Expandable.hh"
#include "RS_XmlReader.hh"
#include "X_TrainSet.hh"

#ifndef X_Depot_HH
#define X_Depot_HH


#include <algorithm>
#include <iostream>
#include <vector>
#include <map>


using std :: cerr ;
using std :: endl ;
using std :: vector ;
using std :: map ;
using std :: sort ;




class X_Depot_Base
  : public virtual RS_Object
{
public :
  explicit X_Depot_Base ( TiXmlNode const * const & x ) : RS_Object () {}
  explicit X_Depot_Base ( string const & _n_in ) : RS_Object ( _n_in ) {}
  explicit X_Depot_Base ( X_Depot_Base const & x ) : RS_Object ( x ) {}
  X_Depot_Base & operator= ( X_Depot_Base const & x )
  { if ( & x == this ) return * this ;
    RS_Object :: operator= ( x ) ; return * this ; }
  virtual ~X_Depot_Base () {}

  virtual X_TrainSet_Container :: iterator firstTrainSet () = 0 ;
  virtual X_TrainSet_Container :: iterator lastTrainSet () = 0 ;
  virtual X_TrainSet_Container :: const_iterator firstTrainSet () const = 0 ;
  virtual X_TrainSet_Container :: const_iterator lastTrainSet () const = 0 ;

  virtual X_TrainSet_Base const * locateTrainSet ( string const & )
    const = 0 ;
  virtual X_TrainSet_Base * locateTrainSet ( string const & ) = 0 ;
} ;



typedef Expandable < X_Depot_Base , TiXmlNode const * > Depot_Expander ;



class X_VirtualCyclicDepot
  : public X_Depot_Base ,
    public Depot_Expander
{
  // Container of trains.
  RS_ObjectContainer * _tsetc ;

  // These must be prohibited...
  explicit X_VirtualCyclicDepot ( X_VirtualCyclicDepot const & x ) ;
  X_VirtualCyclicDepot & operator= ( X_VirtualCyclicDepot const & x ) ;

public :
  explicit X_VirtualCyclicDepot ( TiXmlNode const * const & ) ;
  explicit X_VirtualCyclicDepot ( string const & ) ;

  RS_ObjectContainer * getContainer ( string const & ) ;
  RS_ObjectContainer const * getContainer ( string const & ) const ;

  bool acceptInitObject ( string const & _sx , RS_Object * _ox )
  { return false ; }

  // Virtual functions returning data.
  X_TrainSet_Container :: iterator firstTrainSet ()
  { return _tsetc -> begin () ; }
  X_TrainSet_Container :: iterator lastTrainSet ()
  { return _tsetc -> end () ; }
  X_TrainSet_Container :: const_iterator firstTrainSet () const
  { return _tsetc -> begin () ; }
  X_TrainSet_Container :: const_iterator lastTrainSet () const
  { return _tsetc -> end () ; }

  X_TrainSet_Base const * locateTrainSet ( string const & ) const ;
  X_TrainSet_Base * locateTrainSet ( string const & ) ;
} ;


class X_VirtualCyclicDepot_Creator
  : public Depot_Expander :: ObjectCreator
{
public :
  X_VirtualCyclicDepot_Creator ()
  { add_to_creators_list ( "x_virtual_cyclic_depot" ) ; }
  X_VirtualCyclicDepot_Creator ( char const * x )
    : Depot_Expander :: ObjectCreator () { add_to_creators_list ( x ) ; }
  X_Depot_Base * create () const
  { cerr << "Error: X_VirtualCyclicDepot... default constructor prohibited"
         << endl ; exit ( 1 ) ; }
  X_Depot_Base * create ( TiXmlNode const * const & x ) const
  { return new X_VirtualCyclicDepot ( x ) ; }
} ;



class X_Depot_Container
  : public RS_ObjectContainer
{
public :
  bool create ( TiXmlNode const * & , vector < string > const & ,
                vector < RS_Object * > const & ) ;
  X_Depot_Base * locateDepot ( string const & ) ;
  X_Depot_Base const * locateDepot ( string const & ) const ;
  ~X_Depot_Container () {}
} ;




#endif // X_Depot_HH
