// -*- C++ -*-

// ダイクストラ法のデモ

#include "X_Station.hh"
#include <utility>


using std :: make_pair ;



X_IntraTime :: X_IntraTime
( string const & _n_in )
  : X_IntraTime_Base ( _n_in ) ,
    _from ( 0 ) ,
    _to ( 0 ) ,
    _walk_time ( 0 )
{
}



X_IntraTime :: X_IntraTime
( TiXmlNode const * const & curnode )
  : X_IntraTime_Base ( curnode ) ,
    _from ( 0 ) ,
    _to ( 0 ) ,
    _walk_time ( 0 )
{
  if ( ! curnode )
  {
    cerr << "Error: node not set in X_IntraTime constructor." << endl ;
    exit ( 1 ) ;
  }
  if ( curnode -> ValueStr () != "data" )
  {
    cerr << "Error: <" << curnode -> ValueStr ()
         << "> tag not allowed in X_IntraTime constructor." << endl ;
    exit ( 1 ) ;
  }

  // -----
  // (1) Get all attributes of this "data" or "platform" tag.
  // -----
  map < string , string > _a_m ;
  getAttributeMap ( curnode , _a_m ) ;
  string _ein_msg = "in constructor of X_IntraTime" ;

  // -----
  // (2) Find attribute "class" in the map. Required (already checked ---
  // just discard it here).
  // -----
  map < string , string > :: iterator ii = findAttributeInMap
    ( _a_m , "class" , _ein_msg , true ) ;
  _a_m . erase ( ii ) ;

  // -----
  // (3) Find attribute "name" in the map. Required.
  // -----
  ii = findAttributeInMap ( _a_m , "name" , _ein_msg , true ) ;
  setName ( ii -> second ) ;
  _a_m . erase ( ii ) ;

  // -----
  // (4) Find attribute "time" in the map. Required.
  // -----
  ii = findAttributeInMap ( _a_m , "time" , _ein_msg , true ) ;
  sscanf ( ii -> second . c_str () , "%lf" , & _walk_time ) ;
  _a_m . erase ( ii ) ;

  // -----
  // (5) Check if attribute map is exhausted.
  // -----
  if ( _a_m . size () )
  {
    ii = _a_m . begin () ;
    cerr << "Error: " << _ein_msg << ": incorrect attribute \""
         << ii -> first << "\" found when initialising an "
         << "intratime instance" << endl ;
    exit ( 1 ) ;
  }
}



// ----
// Accept init objects. Two types are to be accepted and required.
// ----
bool
X_IntraTime :: acceptInitObject
( string const & _t_in ,
  RS_Object * _o_in )
{
  if ( _t_in == "from" )
  {
    if ( _from )
    {
      cerr << "Error: _dest seems to have already been set. "
           << "Duplicate <using> tag?" << endl ;
      exit ( 1 ) ;
    }
    _from = dynamic_cast < X_Platform_Base * > ( _o_in ) ;
    if ( ! _from )
    {
      cerr << "Error: dynamic cast failed in acceptInitObject, type = \""
           << _t_in << "\"." << endl ;
      exit ( 1 ) ;
    }
  }
  else if ( _t_in == "to" )
  {
    if ( _to )
    {
      cerr << "Error: _dest seems to have already been set. "
           << "Duplicate <using> tag?" << endl ;
      exit ( 1 ) ;
    }
    _to = dynamic_cast < X_Platform_Base * > ( _o_in ) ;
    if ( ! _to )
    {
      cerr << "Error: dynamic cast failed in acceptInitObject, type = \""
           << _t_in << "\"." << endl ;
      exit ( 1 ) ;
    }
  }
  else
  {
    cerr << "Error: <using> tag with incorrect type \"" << _t_in
         << "\", not accepted by X_IntraTime object." << endl ;
    exit ( 1 ) ;
  }
  return true ;
}



RS_ObjectContainer *
X_IntraTime :: getContainer
( string const & x )
{
  cerr << "Error: No class \"" << x << "\" found in X_IntraTime" << endl ;
  exit ( 1 ) ;
  return 0 ;
}




RS_ObjectContainer const *
X_IntraTime :: getContainer
( string const & x )
  const
{
  cerr << "Error: No class \"" << x << "\" found in X_IntraTime" << endl ;
  exit ( 1 ) ;
  return 0 ;
}




// ----
// Creator.
// ----
bool
X_IntraTime_Container :: create
( TiXmlNode const * & _n_in ,
  vector < string > const & _s_in ,
  vector < RS_Object * > const & _o_in )
{
  string xxx = getClassString ( _n_in ) ;
  string yyy = getNameString ( _n_in ) ;
  X_IntraTime_Base * x = ITime_Expander :: create ( xxx , _n_in ) ;
  ( * this ) [ yyy ] = x ;
  vector < string > :: const_iterator ii = _s_in . begin () ;
  vector < RS_Object * > :: const_iterator jj = _o_in . begin () ;
  while ( ii < _s_in . end () || jj < _o_in . end () )
  {
    if ( x -> acceptInitObject ( * ii , * jj ) )
    {
      ++ ii ;
      ++ jj ;
    }
    else
    {
      cerr << "Error: acceptInitObject failed, type = "
           << * ii << endl ;
      exit ( 1 ) ;
    }
  }
  if ( ( ! x -> From () ) || ( ! x -> To () ) )
  {
    cerr << "Error: in initialising an intratime object: From or To "
         << "platform not set" << endl ;
    exit ( 1 ) ;
  }
  from_map . insert ( make_pair ( x -> From () , x ) ) ;
  to_map . insert ( make_pair ( x -> To () , x ) ) ;
  if ( p_map . find ( make_pair ( x -> From () , x -> To () ) )
       != p_map . end () )
  {
    cerr << "Error: in initialising an intratime object \"" << yyy
         << ": multiple data from " << x -> From () -> getName() << " to "
         << x -> To () -> getName() << endl ;
    exit ( 1 ) ;
  }
  p_map . insert
    ( make_pair ( make_pair ( x -> From () , x -> To () ) , x ) ) ;
  return true ;
}




IntraTimeRange
X_IntraTime_Container :: getIntraTimesTo
( X_Platform_Base const * _x )
{
  return to_map . equal_range ( _x ) ;
}




const_IntraTimeRange
X_IntraTime_Container :: getIntraTimesTo
( X_Platform_Base const * _x )
  const
{
  return to_map . equal_range ( _x ) ;
}




IntraTimeRange
X_IntraTime_Container :: getIntraTimesFrom
( X_Platform_Base const * _x )
{
  return from_map . equal_range ( _x ) ;
}




const_IntraTimeRange
X_IntraTime_Container :: getIntraTimesFrom
( X_Platform_Base const * _x )
  const
{
  return from_map . equal_range ( _x ) ;
}
