// -*- C++ -*-

// ダイクストラ法のデモ

#include "X_PassengerActivity.hh"
#include "X_TrainSet.hh"
#include "X_Station.hh"
#include "X_TheWorld.hh"
#include "X_Weight.hh"



// ----
// Get a "Sujic Weight" object from two space time nodes.
// Boolean arguments:
// (1) true if cycle delay is taken into account,
// (2) true if train node, and
// (3) true if transfer node.
// ----
Sujic_Weight
getSujicWeight
( X_SpaceTimeNode const & _f ,
  X_SpaceTimeNode const & _t ,
  bool _cs ,
  bool _tn ,
  bool _trf )
{
  // ----
  // Creates Sujic_Weight ( a, b, c, d ).
  // ----
  double xa = _t . getTime () ;
  if ( RS_Object :: IS_CYCLIC && _cs )
  {
    xa += _t . getCycleDelay () * RS_Object :: CYCLE_TIME ;
  }
  xa -= _f . getTime () ;
  if ( RS_Object :: IS_CYCLIC && _cs )
  {
    xa -= _f . getCycleDelay () * RS_Object :: CYCLE_TIME ;
  }
  if ( RS_Object :: IS_CYCLIC )
  {
    if ( xa < 0 )
    {
      xa += RS_Object :: CYCLE_TIME ;
    }
  }

  if ( _tn )
  {
    // ----
    // Train branch.
    //  a = xa; b = 0; c = 0; d = 0;
    // ----
    cerr << "W: " << xa << "(X000) = " << _f . getName () << " ["
         << _f . getTime () << "/" << _f . getCycleDelay () << "] --> "
         << _t . getName () << " [" << _t . getTime () << "/"
         << _t . getCycleDelay () << "]" << endl ;
    return Sujic_Weight ( xa , 0 , 0 , 0 ) ;
  }
  if ( _trf )
  {
    // ----
    // Transfer branch.
    //  a = 0; b = 1; c = 0; d = 0;
    // ----
    cerr << "W: " << 0 << "(0100) = " << _f . getName () << " ["
         << _f . getTime () << "/" << _f . getCycleDelay () << "] --> "
         << _t . getName () << " [" << _t . getTime () << "/"
         << _t . getCycleDelay () << "]" << endl ;
    return Sujic_Weight ( 0 , 1 , 0 , 0 ) ;
  }
  if ( _f . getPlatform () && _t . getPlatform ()
       && _f . getPlatform () -> MyStation ()
       && _t . getPlatform () -> MyStation ()
       && ( & _f == _f . getPlatform () -> MyStation () -> EntranceNode ()
            || & _t == _t . getPlatform () -> MyStation () -> ExitNode () ) )
  {
    // ----
    // Exit or entrance branch.
    //  a = 0; b = 0; c = 0; d = 0;
    // ----
    cerr << "W: " << 0 << "(0000) = " << _f . getName () << " ["
         << _f . getTime () << "/" << _f . getCycleDelay () << "] --> "
         << _t . getName () << " [" << _t . getTime () << "/"
         << _t . getCycleDelay () << "]" << endl ;
    return Sujic_Weight ( 0 , 0 , 0 , 0 ) ;
  }
  if ( _f . getPlatform () && _t . getPlatform ()
       && _f . getPlatform () != _t . getPlatform ()
       && _f . getPlatform () -> MyStation ()
       == _t . getPlatform () -> MyStation ()
       && _f . getPlatform () -> MyStation ()
       && _t . getPlatform () -> MyStation ()
       && ( & _f != _f . getPlatform () -> MyStation () -> EntranceNode () )
       && ( & _t != _t . getPlatform () -> MyStation () -> ExitNode () ) )
  {
    // ----
    // Intra-platform transfer branch.
    //  a = xa; b = 0; c = xa; d = 0;
    // ----
    cerr << "W: " << xa << "(X0X0) = " << _f . getName () << " ["
         << _f . getTime () << "/" << _f . getCycleDelay () << "] --> "
         << _t . getName () << " [" << _t . getTime () << "/"
         << _t . getCycleDelay () << "]" << endl ;
    return Sujic_Weight ( xa , 0 , xa , 0 ) ;
  }
  if ( _f . getPlatform () == _t . getPlatform () )
  {
    // ----
    // Platform branch.
    //  a = xa; b = 0; c = 0; d = xa;
    // ----
    cerr << "W: " << xa << "(plat) = " << _f . getName () << " ["
         << _f . getTime () << "/" << _f . getCycleDelay () << "] --> "
         << _t . getName () << " [" << _t . getTime () << "/"
         << _t . getCycleDelay () << "]" << endl ;
    return Sujic_Weight ( xa , 0 , 0 , xa ) ;
  }

  // ----
  // Error??
  // ----
  cerr << "Error: Weight could not be calculated: " << _f . getName ()
       << " --> " << _t . getName () << ": Did I do anything wrong??"
       << endl ;
  exit ( 1 ) ;
  return Sujic_Weight ( 0 , 0 , 0 , 0 ) ;
}




X_Dijkstra_Key :: X_Dijkstra_Key
( X_SpaceTimeNode const * x_n ,
  X_TrainSet_Base const * x_t )
  : RS_Object () ,
    _n ( x_n ) ,
    _t ( x_t )
{
  if ( ! _n )
  {
    cerr << "Error: null pointer used to initialise X_Dijkstra_Key" << endl ;
    exit ( 1 ) ;
  }
  setName ( _n -> getName () + "::" + ( _t ? _t -> getName () : "NOSET" ) ) ; 
}



ostream &
operator<<
( ostream & _x_out ,
  X_Dijkstra_Key const & x )
{
  _x_out << x . getName () ;
  return _x_out ;
}





X_PasAct_F1 :: X_PasAct_F1
( string const & _n_in )
  : X_PassengerActivity_Base ( _n_in ) ,
    _pappc ( 0 )
{
  _pappc = new X_PassengerAppearance_Container ;
}



X_PasAct_F1 :: X_PasAct_F1
( TiXmlNode const * const & curnode )
  : X_PassengerActivity_Base ( curnode ) ,
    _pappc ( 0 )
{
  _pappc = new X_PassengerAppearance_Container ;

  if ( ! curnode )
  {
    cerr << "Error: node not set in X_PasAct_F1 constructor."
         << endl ;
    exit ( 1 ) ;
  }
  if ( curnode -> ValueStr () != "data" )
  {
    cerr << "Error: <" << curnode -> ValueStr () << "> tag not allowed in "
         << "X_PasAct_F1 constructor." << endl ;
    exit ( 1 ) ;
  }

  // -----
  // (1) Get all attributes of this "data" tag.
  // -----
  map < string , string > _a_m ;
  getAttributeMap ( curnode , _a_m ) ;
  string _ein_msg = "in initialisation of an X_PasAct_F1 instance" ;

  // -----
  // (2) Find attribute "class" in the map. Required (already checked ---
  // just discard it here).
  // -----
  map < string , string > :: iterator ii = findAttributeInMap
    ( _a_m , "class" , _ein_msg , true ) ;
  _a_m . erase ( ii ) ;

  // -----
  // (3) Find attribute "name" in the map. Required.
  // -----
  ii = findAttributeInMap ( _a_m , "name" , _ein_msg , true ) ;
  setName ( ii -> second ) ;
  _a_m . erase ( ii ) ;

  // -----
  // (4) Check if attribute map is exhausted.
  // -----
  if ( _a_m . size () )
  {
    ii = _a_m . begin () ;
    cerr << "Error: " << _ein_msg << ": incorrect attribute \""
         << ii -> first << "\" found" << endl ;
    exit ( 1 ) ;
  }
}



RS_ObjectContainer *
X_PasAct_F1 :: getContainer
( string const & x )
{
  if ( x != "sujic_passenger_appearance_model" )
  {
    cerr << "Error: No class \"" << x << "\" container found in "
         << "X_PasAct_F1" << endl ;
    exit ( 1 ) ;
  }
  return _pappc ;
}




RS_ObjectContainer const *
X_PasAct_F1 :: getContainer
( string const & x )
  const
{
  if ( x != "sujic_passenger_appearance_model" )
  {
    cerr << "Error: No class \"" << x << "\" container found in "
         << "X_PasAct_F1" << endl ;
    exit ( 1 ) ;
  }
  return _pappc ;
}




bool
X_PassengerActivity_Container :: create
( TiXmlNode const * & _n_in ,
  vector < string > const & _s_in ,
  vector < RS_Object * > const & _o_in )
{
  string xxx = getClassString ( _n_in ) ;
  string yyy = getNameString ( _n_in ) ;
  if ( find ( yyy ) != end () )
  {
    cerr << "Error: PassengerActivity container already has a PassengerActivity instance "
         << "of name \"" << xxx << "\"" << endl ;
    exit ( 1 ) ;
  }
  X_PassengerActivity_Base * x = PAct_Expander :: create ( xxx , _n_in ) ;
  ( * this ) [ yyy ] = x ;
  vector < string > :: const_iterator ii = _s_in . begin () ;
  vector < RS_Object * > :: const_iterator jj = _o_in . begin () ;
  while ( ii < _s_in . end () || jj < _o_in . end () )
  {
    if ( x -> acceptInitObject ( * ii , * jj ) )
    {
      ++ ii ;
      ++ jj ;
    }
    else
    {
      cerr << "Error: acceptInitObject failed, type = "
           << * ii << endl ;
      exit ( 1 ) ;
    }
  }
  return true ;
}




// -----
// Locate a depot (const).
// -----
X_PassengerActivity_Base const *
X_PassengerActivity_Container :: locatePassengerActivity
( string const & x )
  const
{
  RS_Object const * retv = locate ( x ) ;
  return dynamic_cast < X_PassengerActivity_Base const * > ( retv ) ;
}




// -----
// Locate a depot (non-const).
// -----
X_PassengerActivity_Base *
X_PassengerActivity_Container :: locatePassengerActivity
( string const & x )
{
  RS_Object * retv = locate ( x ) ;
  return dynamic_cast < X_PassengerActivity_Base * > ( retv ) ;
}




// ----
// Solve schedule.
// ----
void
X_PasAct_F1 :: solveOriginal
( X_Station_Container * xs_c ,
  X_ScheduleSet_Container * xschs_c )
{

  X_ScheduleSet_Base * p_xs = xschs_c -> firstScheduleSet () ;
  for ( X_Station_Container :: iterator ii_sb = xs_c -> begin () ;
        ii_sb != xs_c -> end () ; ++ ii_sb )
  {
    X_Station_Base * p_sb
      = dynamic_cast < X_Station_Base * > ( ii_sb -> second ) ;
    if ( ! p_sb )
    {
      cerr << "Error: station could not be extracted in solve" << endl ;
      exit ( 1 ) ;
    }
    for ( X_PassengerAppearance_Container :: iterator ii_pa
            = _pappc -> begin () ; ii_pa != _pappc -> end () ; ++ ii_pa )
    {
      // ----
      // Get passenger appearance model.
      // ----
      X_PassengerAppearance_Base * p_pa = dynamic_cast
        < X_PassengerAppearance_Base * > ( ii_pa -> second ) ;
      if ( ! p_pa )
      {
        cerr << "Error: Passenger Appearance model could not be extracted "
             << "in solve" << endl ;
        exit ( 1 ) ;
      }
      if ( ! p_pa -> hasOD ( p_sb ) )
      {
        continue ;
      }

      // ----
      // For each platform branch at entrance of p_sb...
      // ----
      X_Platform_Base * p_sb_entr = p_sb -> Entrance () ;
      for ( vector < X_SpaceTimeNode * > :: iterator ii_n = p_sb_entr
              -> firstNode () ; ii_n != p_sb_entr -> lastNode () ; ++ ii_n )
      {
        vector < X_SpaceTimeNode * > :: iterator jj_n = ii_n + 1 ;
        if ( RS_Object :: IS_CYCLIC && jj_n == p_sb_entr -> lastNode () )
        {
          jj_n = p_sb_entr -> firstNode () ;
        }
        X_PlatformBranch * p_pb = p_xs -> getPlatformBranch
          ( * ii_n , * jj_n ) ;
        p_pa -> solveOriginal ( p_pb , xs_c , p_xs ) ;
      }
    }
  }
}
