// -*- C++ -*-

// ダイクストラ法のデモ

#include "Expandable.hh"
#include "RS_XmlReader.hh"
#include "X_SpaceTimeBranch.hh"
#include "RS_Dijkstra.hh"
#include "X_Weight.hh"


#ifndef X_PassengerActivity_HH
#define X_PassengerActivity_HH


#include <algorithm>
#include <iostream>
#include <vector>
#include <map>


using std :: cerr ;
using std :: endl ;
using std :: vector ;
using std :: map ;
using std :: sort ;




class X_Station_Base ;
class X_Station_Container ;
class X_ScheduleSet_Base ;
class X_ScheduleSet_Container ;
class X_TheWorld ;





// ----
// Functor comparing two vector < RS_Object * > instances.
// ----
class LessROV
{
public :
  bool operator() ( vector < RS_Object * > const & _l ,
                    vector < RS_Object * > const & _r ) const
  { for ( vector < RS_Object * > :: const_iterator ii = _l . begin () ,
            jj = _r . begin () ; ii != _l . end () && jj != _r . end () ;
          ++ ii , ++ jj ) 
    { if ( ( * ii ) < ( * jj ) ) return true ;
      else if ( ( * ii ) > ( * jj ) ) return false ; }
    if ( _l . size () < _r . size () ) return true ; return false ; }
} ;


class ROV_Same_Until
{
public :
  vector < RS_Object * > :: size_type
  operator() ( vector < RS_Object * > const & _l ,
               vector < RS_Object * > const & _r ,
               vector < RS_Object * > :: size_type _x ) const
  { for ( ; _x < _l . size () && _x < _r . size () ; ++ _x )
    { if ( _l [ _x ] != _r [ _x ] ) return _x ; }
    return _x ; }
} ;



// ----
// Get a "Sujic Weight" object from two space time nodes.
// Boolean arguments: (1) true if transfer node.
// ----
Sujic_Weight getSujicWeight
( X_SpaceTimeNode const & , X_SpaceTimeNode const & , bool , bool , bool ) ;




class X_Passenger_Token
  : public virtual RS_Object
{
  X_Station_Base const * _sta_orig ;
  X_Station_Base const * _sta_dest ;
  X_PlatformBranch * _plat_branch ;
  X_PassengerAppearance_Base const * _pasapp ;
  double _t_ ;
  int _cd_ ;
  double _num_passen ;
  vector < X_Branch_Base const * > _br ;

public :
  X_Passenger_Token
  ( X_Station_Base const * s_i , X_Station_Base const * s_o ,
    X_PlatformBranch * _pb , X_PassengerAppearance_Base const * _pa ,
    double _t_in , int _cd_in , double _np_in )
    : RS_Object () , _sta_orig ( s_i ) , _sta_dest ( s_o ) ,
      _plat_branch ( _pb ) , _pasapp ( _pa ) , _t_ ( _t_in ) ,
      _cd_ ( _cd_in ) , _num_passen ( _np_in ) {}

  X_PlatformBranch const * Branch () const { return _plat_branch ; }
  double PassengerNumber () const { return _num_passen ; }
  double Time () const { return _t_ ; }
  int CycleDelay () const { return _cd_ ; }
  X_Station_Base const * Origin () const { return _sta_orig ; }
  X_Station_Base const * Destination () const { return _sta_dest ; }

  void addBranch ( X_Branch_Base const * x ) { _br . push_back ( x ) ; }
} ;




class X_OD_Data_Base
  : public virtual RS_Object
{
public :
  explicit X_OD_Data_Base ( TiXmlNode const * const & x ) : RS_Object () {}
  explicit X_OD_Data_Base ( string const & _n_in ) : RS_Object ( _n_in ) {}
  explicit X_OD_Data_Base ( X_OD_Data_Base const & x )
    : RS_Object ( x ) {}
  X_OD_Data_Base & operator= ( X_OD_Data_Base const & x )
  { if ( & x == this ) return * this ;
    RS_Object :: operator= ( x ) ; return * this ; }

  virtual ~X_OD_Data_Base () {}

  virtual string const & getOrigin () const = 0 ;
  virtual string const & getDestination () const = 0 ;
  virtual double getDemand () const = 0 ;
} ;



typedef Expandable < X_OD_Data_Base , TiXmlNode const * > ODD_Expander ;


class X_OD_Data
  : public X_OD_Data_Base ,
    public ODD_Expander
{
  string _from_str ;
  string _to_str ;
  double _demand ;

  // These must be prohibited...
  explicit X_OD_Data ( X_OD_Data const & x ) ;
  X_OD_Data & operator= ( X_OD_Data const & x ) ;
  explicit X_OD_Data ( string const & ) ;

public :
  explicit X_OD_Data ( TiXmlNode const * const & ) ;
  ~X_OD_Data () {}

  string const & getOrigin () const { return _from_str ; }
  string const & getDestination () const { return _to_str ; }
  double getDemand () const { return _demand ; }
} ;


class X_OD_Data_Creator
  : public ODD_Expander :: ObjectCreator
{
public :
  X_OD_Data_Creator ()
  { add_to_creators_list ( "x_od_data" ) ; }
  X_OD_Data_Creator ( const char * x )
    : ODD_Expander :: ObjectCreator () { add_to_creators_list ( x ) ; }
  X_OD_Data_Base * create () const
  { cerr << "Error: X_OD_Data... default constructor prohibited" << endl ;
    exit ( 1 ) ; }
  X_OD_Data_Base * create ( TiXmlNode const * const & x ) const
  { return new X_OD_Data ( x ) ; }
} ;



class X_OD_Data_Container
  : public RS_ObjectContainer
{
public :
  bool create ( TiXmlNode const * & , vector < string > const & ,
                vector < RS_Object * > const & ) ;
  X_OD_Data_Base * locateODData ( string const & ) ;
  X_OD_Data_Base const * locateODData ( string const & ) const ;
  ~X_OD_Data_Container () {}
} ;




class X_PassengerAppearance_Base
  : public virtual RS_Object
{
public :
  explicit X_PassengerAppearance_Base ( TiXmlNode const * const & x )
    : RS_Object () {}
  explicit X_PassengerAppearance_Base ( string const & _n_in )
    : RS_Object ( _n_in ) {}
  explicit X_PassengerAppearance_Base ( X_PassengerAppearance_Base const & x )
    : RS_Object ( x ) {}
  X_PassengerAppearance_Base &
  operator= ( X_PassengerAppearance_Base const & x )
  { if ( & x == this ) return * this ;
    RS_Object :: operator= ( x ) ; return * this ; }
  virtual ~X_PassengerAppearance_Base () {}

  // Getting an OD data.
  virtual X_OD_Data_Base * getOD
  ( X_Station_Base const * , X_Station_Base const * ) = 0 ;
  virtual X_OD_Data_Base const *
  getOD ( X_Station_Base const * , X_Station_Base const * ) const = 0 ;
  virtual bool hasOD ( X_Station_Base const * ) const = 0 ;

  virtual void solveOriginal
  ( X_PlatformBranch * , X_Station_Container * , X_ScheduleSet_Base * ) = 0 ;
  
} ;



typedef Expandable < X_PassengerAppearance_Base , TiXmlNode const * >
PApp_Expander ;




class X_Dijkstra_Key
  : public virtual RS_Object
{
  X_SpaceTimeNode const * _n ;
  X_TrainSet_Base const * _t ;

public :

  X_Dijkstra_Key () : RS_Object () , _n ( 0 ) , _t ( 0 ) {}
  
  X_Dijkstra_Key ( X_SpaceTimeNode const * , X_TrainSet_Base const * ) ;

  X_Dijkstra_Key ( X_Dijkstra_Key const & x )
    : RS_Object ( x ) , _n ( x . _n ) , _t ( x . _t ) {}
  X_Dijkstra_Key & operator= ( X_Dijkstra_Key const & x )
  { if ( & x == this ) return * this ; RS_Object :: operator= ( x ) ;
    _n = x . _n ; _t = x . _t ; return * this ; }

  X_SpaceTimeNode const * getSTN () const { return _n ; }
  X_TrainSet_Base const * getTrainSet () const { return _t ; }

//   friend ostream & operator<< ( ostream & , X_Weight_Base const & ) ;
//   virtual ostream & writeOut ( ostream & ) const = 0 ;

  bool operator< ( X_Dijkstra_Key const & x ) const
  { return ( _n < x . _n )
      || ( ( _n == x . _n ) && ( _t < x . _t ) ) ; }
  bool operator== ( X_Dijkstra_Key const & x ) const
  { return _n == x . _n && _t == x . _t ; }

  bool operator!= ( X_Dijkstra_Key const & x ) const
  { return ! operator== ( x ) ; }

  friend ostream & operator<< ( ostream & , X_Dijkstra_Key const & ) ;
  
} ;




class X_PasApp_F1_Constant_Rate
  : public X_PassengerAppearance_Base ,
    public PApp_Expander
{
  // _odc: OD Data container.
  map < string , X_OD_Data_Container * > _odc ;

  // Nodes for passengers
  map < X_PlatformBranch const * , X_SpaceTimeNode * > _passenger_nodes ;

  // Solvers
  map < X_PlatformBranch const * ,
        RS_Dijkstra_MPS < Sujic_Weight , X_Dijkstra_Key > * > _dk_solvers ;

  // Temporary variables for "solveOriginal" function
  vector < double > dividing ;

  // These must be prohibited...
  explicit X_PasApp_F1_Constant_Rate ( X_PasApp_F1_Constant_Rate const & x ) ;
  X_PasApp_F1_Constant_Rate & operator=
  ( X_PasApp_F1_Constant_Rate const & x ) ;

public :
  explicit X_PasApp_F1_Constant_Rate ( TiXmlNode const * const & ) ;
  explicit X_PasApp_F1_Constant_Rate ( string const & ) ;

  RS_ObjectContainer * getContainer ( string const & ) ;
  RS_ObjectContainer const * getContainer ( string const & ) const ;

  bool acceptInitObject ( string const & _sx , RS_Object * _ox )
  { return false ; }

  // Getting an OD data.
  X_OD_Data_Base * getOD
  ( X_Station_Base const * , X_Station_Base const * ) ;
  X_OD_Data_Base const *
  getOD ( X_Station_Base const * , X_Station_Base const * ) const ;
  bool hasOD ( X_Station_Base const * ) const ;
  void solveOriginal
  ( X_PlatformBranch * , X_Station_Container * , X_ScheduleSet_Base * ) ;

private :
  // ----
  // Subfunction for solveOriginal, processing route diversion
  //  ix, ie: routes to process, 
  // ----
  void processRouteDiversion
  ( vector < vector < RS_Object * > > :: iterator ,
    vector < vector < RS_Object * > > :: size_type ,
    vector < vector < RS_Object * > > :: iterator ,
    vector < vector < RS_Object * > > :: size_type ,
    vector < RS_Object * > :: size_type = 0 ) ;

} ;


class X_PasApp_F1_Constant_Rate_Creator
  : public PApp_Expander :: ObjectCreator
{
public :
  X_PasApp_F1_Constant_Rate_Creator ()
  { add_to_creators_list ( "f1_constant_rate_inflow" ) ; }
  X_PasApp_F1_Constant_Rate_Creator ( char const * x )
    : PApp_Expander :: ObjectCreator () { add_to_creators_list ( x ) ; }
  X_PassengerAppearance_Base * create () const
  { cerr << "Error: X_PasApp_F1_Constant_Rate... default constructor "
         << "prohibited" << endl ; exit ( 1 ) ; }
  X_PassengerAppearance_Base * create ( TiXmlNode const * const & x ) const
  { return new X_PasApp_F1_Constant_Rate ( x ) ; }
} ;



class X_PassengerAppearance_Container
  : public RS_ObjectContainer
{
public :
  bool create ( TiXmlNode const * & , vector < string > const & ,
                vector < RS_Object * > const & ) ;
  X_PassengerAppearance_Base * locatePassengerAppearance ( string const & ) ;
  X_PassengerAppearance_Base const * locatePassengerAppearance
  ( string const & ) const ;
  ~X_PassengerAppearance_Container () {}
} ;




class X_PassengerActivity_Base
  : public virtual RS_Object
{
public :
  explicit X_PassengerActivity_Base ( TiXmlNode const * const & x )
    : RS_Object () {}
  explicit X_PassengerActivity_Base ( string const & _n_in )
    : RS_Object ( _n_in ) {}
  explicit X_PassengerActivity_Base ( X_PassengerActivity_Base const & x )
    : RS_Object ( x ) {}
  X_PassengerActivity_Base & operator= ( X_PassengerActivity_Base const & x )
  { if ( & x == this ) return * this ;
    RS_Object :: operator= ( x ) ; return * this ; }
  virtual ~X_PassengerActivity_Base () {}

  virtual X_PassengerAppearance_Container :: iterator
  firstAppearanceModel () = 0 ;
  virtual X_PassengerAppearance_Container :: iterator
  lastAppearanceModel () = 0 ;
  virtual X_PassengerAppearance_Container :: const_iterator
  firstAppearanceModel () const = 0 ;
  virtual X_PassengerAppearance_Container :: const_iterator
  lastAppearanceModel () const = 0 ;

  virtual void solveOriginal ( X_Station_Container * ,
                               X_ScheduleSet_Container * ) = 0 ;
} ;



typedef Expandable < X_PassengerActivity_Base , TiXmlNode const * >
PAct_Expander ;



class X_PasAct_F1
  : public X_PassengerActivity_Base ,
    public PAct_Expander
{
  // _pappc: Passenger Appearance Model container
  X_PassengerAppearance_Container * _pappc ;

  // These must be prohibited...
  explicit X_PasAct_F1 ( X_PasAct_F1 const & x ) ;
  X_PasAct_F1 & operator= ( X_PasAct_F1 const & x ) ;

public :
  explicit X_PasAct_F1 ( TiXmlNode const * const & ) ;
  explicit X_PasAct_F1 ( string const & ) ;

  RS_ObjectContainer * getContainer ( string const & ) ;
  RS_ObjectContainer const * getContainer ( string const & ) const ;

  bool acceptInitObject ( string const & _sx , RS_Object * _ox )
  { return false ; }

  X_PassengerAppearance_Container :: iterator firstAppearanceModel ()
  { return _pappc -> begin () ; }
  X_PassengerAppearance_Container :: iterator lastAppearanceModel ()
  { return _pappc -> end () ; }
  X_PassengerAppearance_Container :: const_iterator firstAppearanceModel ()
    const { return _pappc -> begin () ; }
  X_PassengerAppearance_Container :: const_iterator lastAppearanceModel ()
    const { return _pappc -> end () ; }

  void solveOriginal ( X_Station_Container * , X_ScheduleSet_Container * ) ;
} ;


class X_PasAct_F1_Creator
  : public PAct_Expander :: ObjectCreator
{
public :
  X_PasAct_F1_Creator ()
  { add_to_creators_list ( "x_passenger_activity_f1" ) ; }
  X_PasAct_F1_Creator ( char const * x )
    : PAct_Expander :: ObjectCreator () { add_to_creators_list ( x ) ; }
  X_PassengerActivity_Base * create () const
  { cerr << "Error: X_PasAct_F1... default constructor prohibited"
         << endl ; exit ( 1 ) ; }
  X_PassengerActivity_Base * create ( TiXmlNode const * const & x ) const
  { return new X_PasAct_F1 ( x ) ; }

} ;



class X_PassengerActivity_Container
  : public RS_ObjectContainer
{
public :
  bool create ( TiXmlNode const * & , vector < string > const & ,
                vector < RS_Object * > const & ) ;
  X_PassengerActivity_Base * locatePassengerActivity ( string const & ) ;
  X_PassengerActivity_Base const * locatePassengerActivity ( string const & )
    const ;
  ~X_PassengerActivity_Container () {}
} ;




#endif // X_PassengerActivity_HH
