// -*- C++ -*-

// ダイクストラ法のデモ

#include "X_Station.hh"
#include "X_SpaceTimeNode.hh"


X_Platform :: X_Platform
( string const & _n_in )
  : X_Platform_Base ( _n_in ) ,
    Plat_Expander () ,
    is_entrance ( false ) ,
    is_exit ( false ) ,
    my_station ( 0 ) ,
    _stn () ,
    _stn_a () ,
    _stn_b () ,
    _stn_d () ,
    _stn_o ()
{
}



X_Platform :: X_Platform
( TiXmlNode const * const & curnode )
  : X_Platform_Base ( curnode ) ,
    Plat_Expander () ,
    is_entrance ( false ) ,
    is_exit ( false ) ,
    my_station ( 0 ) ,
    _stn () ,
    _stn_a () ,
    _stn_b () ,
    _stn_d () ,
    _stn_o ()
{
  if ( ! curnode )
  {
    cerr << "Error: node not set in X_Platform constructor." << endl ;
    exit ( 1 ) ;
  }
  if ( curnode -> ValueStr () != "data"
       && curnode -> ValueStr () != "platform" )
  {
    cerr << "Error: <" << curnode -> ValueStr ()
         << "> tag not allowed in X_Platform constructor." << endl ;
    exit ( 1 ) ;
  }

  // -----
  // (1) Get all attributes of this "data" or "platform" tag.
  // -----
  map < string , string > _a_m ;
  getAttributeMap ( curnode , _a_m ) ;
  string _ein_msg = "in initialisation of an X_Platform instance" ;

  // -----
  // (2) Find attribute "class" in the map. Required (already checked ---
  // just discard it here).
  // -----
  map < string , string > :: iterator ii = findAttributeInMap
    ( _a_m , "class" , _ein_msg , true ) ;
  _a_m . erase ( ii ) ;

  // -----
  // (3) Find attribute "name" in the map. Required.
  // -----
  ii = findAttributeInMap ( _a_m , "name" , _ein_msg , true ) ;
  setName ( ii -> second ) ;
  _a_m . erase ( ii ) ;

  // -----
  // (4) Find attribute "entrance" in the map. Optional, true if this
  // is the entrance.
  // -----
  ii = findAttributeInMap ( _a_m , "entrance" ) ;
  if ( ii != _a_m . end () )
  {
    string _s_ax = ii -> second ;
    if ( _s_ax  == "true" || _s_ax == "True" || _s_ax == "t"
         || _s_ax == "T" || _s_ax == "TRUE" || _s_ax == "Y"
         || _s_ax == "yes" || _s_ax == "y" || _s_ax == "YES" )
    {
      is_entrance = true ;
    }
    else if ( _s_ax  == "false" || _s_ax == "False" || _s_ax == "f"
              || _s_ax == "F" || _s_ax == "FALSE" || _s_ax == "N"
              || _s_ax == "no" || _s_ax == "n" || _s_ax == "NO" )
    {
      is_entrance = false ;
    }
    else
    {
      cerr << "Error: " << _ein_msg << ": attribute \"entrance\" with "
           << "incorrect value " << _s_ax << " when initialising a "
           << "platform" << endl ;
      exit ( 1 ) ;
    }
    _a_m . erase ( ii ) ;
  }

  // -----
  // (5) Find attribute "exit" in the map. Optional, true if this
  // is the exit.
  // -----
  ii = findAttributeInMap ( _a_m , "exit" ) ;
  if ( ii != _a_m . end () )
  {
    string _s_ax = ii -> second ;
    if ( _s_ax  == "true" || _s_ax == "True" || _s_ax == "t"
         || _s_ax == "T" || _s_ax == "TRUE" || _s_ax == "Y"
         || _s_ax == "yes" || _s_ax == "y" || _s_ax == "YES" )
    {
      is_exit = true ;
    }
    else if ( _s_ax  == "false" || _s_ax == "False" || _s_ax == "f"
              || _s_ax == "F" || _s_ax == "FALSE" || _s_ax == "N"
              || _s_ax == "no" || _s_ax == "n" || _s_ax == "NO" )
    {
      is_exit = false ;
    }
    else
    {
      cerr << "Error: " << _ein_msg << ": attribute \"exit\" with "
           << "incorrect value " << _s_ax << " when initialising a "
           << "platform" << endl ;
      exit ( 1 ) ;
    }
    _a_m . erase ( ii ) ;
  }

  // -----
  // (6) Check if attribute map is exhausted.
  // -----
  if ( _a_m . size () )
  {
    ii = _a_m . begin () ;
    cerr << "Error: " << _ein_msg << ": incorrect attribute \""
         << ii -> first << "\" found" << endl ;
    exit ( 1 ) ;
  }
}



RS_ObjectContainer *
X_Platform :: getContainer
( string const & x )
{
  cerr << "Error: No class \"" << x << "\" found in X_Platform" << endl ;
  exit ( 1 ) ;
  return 0 ;
}




RS_ObjectContainer const *
X_Platform :: getContainer
( string const & x )
  const
{
  cerr << "Error: No class \"" << x << "\" found in X_Platform" << endl ;
  exit ( 1 ) ;
  return 0 ;
}




// ----
// Creator.
// ----
bool
X_Platform_Container :: create
( TiXmlNode const * & _n_in ,
  vector < string > const & _s_in ,
  vector < RS_Object * > const & _o_in )
{
  string xxx = getClassString ( _n_in ) ;
  string yyy = getNameString ( _n_in ) ;
  X_Platform_Base * x = Plat_Expander :: create ( xxx , _n_in ) ;
  ( * this ) [ yyy ] = x ;
  vector < string > :: const_iterator ii = _s_in . begin () ;
  vector < RS_Object * > :: const_iterator jj = _o_in . begin () ;
  while ( ii < _s_in . end () || jj < _o_in . end () )
  {
    if ( x -> acceptInitObject ( * ii , * jj ) )
    {
      ++ ii ;
      ++ jj ;
    }
    else
    {
      cerr << "Error: acceptInitObject failed, type = "
           << * ii << endl ;
      exit ( 1 ) ;
    }
  }
  return true ;
}




// ----
// "Combination name".
// ----
string const &
X_Platform :: getCombinationName
()
  const
{
  if ( ! my_station )
  {
    cerr << "Error: missing my_station in Platform \"" << getName () << "\""
         << endl ;
    exit ( 1 ) ;
  }
  _combi_name = my_station -> getName () ;
  _combi_name += "::" ;
  _combi_name += getName () ;
  return _combi_name ;
}

