// -*- C++ -*-

// ダイクストラ法のデモ

#include "X_TheWorld.hh"


X_ScheduleSet :: X_ScheduleSet
( string const & _n_in )
  : X_ScheduleSet_Base ( _n_in ) ,
    _trnc () ,
    is_cyclic ( false ) ,
    _c_time ( 0 ) ,
    _s_orig ( 0 ) ,
    _s_next ( 0 ) ,
    _m_pb () ,
    _m_pb_orig () ,
    _m_pb_dest () ,
    _m_tset () ,
    _m_tb () ,
    _m_trf () ,
    _m_trs () ,
    _m_misc ()
{
  _trnc = new X_Train_Container () ;
}



X_ScheduleSet :: X_ScheduleSet
( TiXmlNode const * const & curnode )
  : X_ScheduleSet_Base ( curnode ) ,
    _trnc () ,
    is_cyclic ( false ) ,
    _c_time ( 0 ) ,
    _s_orig ( 0 ) ,
    _s_next ( 0 ) ,
    _m_pb () ,
    _m_pb_orig () ,
    _m_pb_dest () ,
    _m_tset () ,
    _m_tb () ,
    _m_trf () ,
    _m_trs () ,
    _m_misc ()
{
  if ( ! curnode )
  {
    cerr << "Error: node not set in X_ScheduleSet constructor."
         << endl ;
    exit ( 1 ) ;
  }
  if ( curnode -> ValueStr () != "data" )
  {
    cerr << "Error: <" << curnode -> ValueStr () << "> tag not allowed in "
         << "X_ScheduleSet constructor." << endl ;
    exit ( 1 ) ;
  }

  // -----
  // (1) Get all attributes of this "data" tag.
  // -----
  map < string , string > _a_m ;
  getAttributeMap ( curnode , _a_m ) ;
  string _ein_msg = "in initialisation of an X_ScheduleSet instance" ;

  // -----
  // (2) Find attribute "class" in the map. Required (already checked ---
  // just discard it here).
  // -----
  map < string , string > :: iterator ii = findAttributeInMap
    ( _a_m , "class" , _ein_msg , true ) ;
  _a_m . erase ( ii ) ;

  // -----
  // (3) Find attribute "name" in the map. Required.
  // -----
  ii = findAttributeInMap ( _a_m , "name" , _ein_msg , true ) ;
  setName ( ii -> second ) ;
  _a_m . erase ( ii ) ;

  // -----
  // (4) Find attribute "cycletime" in the map. Optional. If set, it will
  // also mean that is_cyclic = true.
  // -----
  ii = findAttributeInMap ( _a_m , "cycletime" ) ;
  if ( ii != _a_m . end () )
  {
    sscanf ( ii -> second . c_str () , "%lf" , & _c_time ) ;
    is_cyclic = true ;
    _a_m . erase ( ii ) ;
  }

  // -----
  // (5) Check if attribute map is exhausted.
  // -----
  if ( _a_m . size () )
  {
    ii = _a_m . begin () ;
    cerr << "Error: " << _ein_msg << ": incorrect attribute \""
         << ii -> first << "\" found" << endl ;
    exit ( 1 ) ;
  }

  // ----
  // Setting a train container.
  // ----
  _trnc = new X_Train_Container () ;
}



RS_ObjectContainer *
X_ScheduleSet :: getContainer
( string const & x )
{
  if ( x != "sujic_train" )
  {
    cerr << "Error: No class \"" << x << "\" container found in "
         << "X_ScheduleSet" << endl ;
    exit ( 1 ) ;
  }
  return _trnc ;
}




RS_ObjectContainer const *
X_ScheduleSet :: getContainer
( string const & x )
  const
{
  if ( x != "sujic_train" )
  {
    cerr << "Error: No class \"" << x << "\" container found in "
         << "X_ScheduleSet" << endl ;
    exit ( 1 ) ;
  }
  return _trnc ;
}








// -----
// Set times of individual trains.
// -----
void
X_ScheduleSet :: setTimes
( X_Station_Container * x )
{
  X_Train_Container * y = dynamic_cast < X_Train_Container * > ( _trnc ) ;
  if ( ! y )
  {
    cerr << "Error: cannot convert to X_Train_Container in "
         << "X_ScheduleSet " << getName () << endl ;
    exit ( 1 ) ;
  }
  y -> setTimes ( x ) ;
}




// -----
// Accept init objects. Actually accepts nothing.
// -----
bool
X_ScheduleSet :: acceptInitObject
( string const & _t_in ,
  RS_Object * _o_in )
{
  cerr << "Error: <using> tag with incorrect type \"" << _t_in
       << "\", which will not be accepted by X_ScheduleSet object." << endl ;
  exit ( 1 ) ;
  return true ;
}




// ----
// Setting X_PlatformBranch objects in X_ScheduleSet object.
// ----
void
X_ScheduleSet :: setPlatformBranches
( X_Station_Container * xs_c )
{
  // ----
  // Check if this is the first schedule.
  // ----
  if ( ! Original () )
  {
    // ----
    // For each station...
    // ----
    for ( X_Station_Container :: iterator ii = xs_c -> begin () ;
          ii != xs_c -> end () ; ++ ii )
    {
      X_Station_Base *
        p_ii = dynamic_cast < X_Station_Base * > ( ii -> second ) ;
      if ( ! p_ii )
      {
        cerr << "Error: in X_ScheduleSet(\"" << getName() << "\")::setPlat"
             << "formBranches(): failed to retrieve X_Station_Base pointer"
             << endl ;
        exit ( 1 ) ;
      }

      // ----
      // For each platform in the station...
      // ----
      for ( X_Platform_Container :: iterator jj = p_ii -> firstPlatform () ;
            jj != p_ii -> lastPlatform () ; ++ jj )
      {
        X_Platform_Base *
          p_jj = dynamic_cast < X_Platform_Base * > ( jj -> second ) ;
        if ( ! p_jj )
        {
          cerr << "Error: in X_ScheduleSet(\"" << getName() << "\")::setPlat"
               << "formBranches(): failed to retrieve X_Platform_Base pointer"
               << endl ;
          exit ( 1 ) ;
        }

        // ----
        // Find adjacent space time nodes and link them with a platform
        // branch.
        // ----
        for ( vector < X_SpaceTimeNode * > :: const_iterator
                ix = p_jj -> firstNode () ;
              ix != p_jj -> lastNode () - 1 ; ++ ix )
        {
          X_SpaceTimeNode const * _stn_s = * ix ;
          X_SpaceTimeNode const * _stn_e = * ( ix + 1 ) ;
          double _tx = _stn_e -> getTime () - _stn_s -> getTime () ;
          if ( _tx < 0 )
          {
            _tx += X_SpaceTimeNode :: CYCLE_TIME ;
          }
          if ( _tx < 0 || ( X_SpaceTimeNode :: IS_CYCLIC
                            && _tx >= X_SpaceTimeNode :: CYCLE_TIME ) )
          {
            cerr << "Error: in X_ScheduleSet(\"" << getName() << "\")::"
                 << "setPlatformBranches(): adjacent SpateTimeNodes within "
                 << "platform " << p_jj -> getCombinationName ()
                 << " have inconsistent times" << endl ;
            exit ( 1 ) ;
          }
          cerr << "Branch: " << _tx << ", " << _stn_s -> getName () << " ["
               << _stn_s -> getTime () << "/" << _stn_s -> getCycleDelay ()
               << "] -> " << _stn_e -> getName () << " ["
               << _stn_e -> getTime () << "/" << _stn_e -> getCycleDelay ()
               << "]" << endl ;
          X_PlatformBranch *
            _sx = new X_PlatformBranch ( _stn_s , _stn_e , 0 , p_jj ) ;
          if ( _m_pb . find ( X_STB_Key ( _stn_s , _stn_e ) )
               != _m_pb . end () )
          {
            cerr << "Error: in X_ScheduleSet(\"" << getName() << "\")::"
                 << "setPlatformBranches(): X_PlatformBranch already "
                 << "present in _m_pb" << endl ;
            exit ( 1 ) ;
          }
          _m_pb [ X_STB_Key ( _stn_s , _stn_e ) ] = _sx ;
          _m_pb_orig [ _stn_s ] = _sx ;
          _m_pb_dest [ _stn_e ] = _sx ;
        }

        // ----
        // Cyclic simulation
        // ----
        if ( X_SpaceTimeNode :: IS_CYCLIC )
        {
          vector < X_SpaceTimeNode * > :: const_iterator
            i_last = p_jj -> lastNode () - 1 ;
          vector < X_SpaceTimeNode * > :: const_iterator
            i_first = p_jj -> firstNode () ;
          // ----
          // Creation of a cyclic SpaceTimeBranch object.
          // ----
          X_SpaceTimeNode const * _stn_s = * i_last ;
          X_SpaceTimeNode const * _stn_e = * i_first ;
          double _tx = _stn_e -> getTime () - _stn_s -> getTime () ;
          if ( _tx < 0 )
          {
            _tx += X_SpaceTimeNode :: CYCLE_TIME ;
          }
          if ( _tx < 0 || _tx >= X_SpaceTimeNode :: CYCLE_TIME )
          {
            cerr << "Error: in X_ScheduleSet(\"" << getName() << "\")::"
                 << "setPlatformBranches(): cyclic SpateTimeNodes within "
                 << "platform " << p_jj -> getCombinationName ()
                 << " have inconsistent times" << endl ;
            exit ( 1 ) ;
          }
          cerr << "Branch: " << _tx << ", " << _stn_s -> getName () << " ["
               << _stn_s -> getTime () << "/" << _stn_s -> getCycleDelay ()
               << "] -> " << _stn_e -> getName () << " ["
               << _stn_e -> getTime () << "/" << _stn_e -> getCycleDelay ()
               << "]" << endl ;
          X_PlatformBranch *
            _sx = new X_PlatformBranch ( _stn_s , _stn_e , 0 , p_jj ) ;
          if ( _m_pb . find ( X_STB_Key ( _stn_s , _stn_e ) )
               != _m_pb . end () )
          {
            cerr << "Error: in X_ScheduleSet(\"" << getName() << "\")::"
                 << "setPlatformBranches(): X_PlatformBranch already "
                 << "present in _m_pb" << endl ;
            exit ( 2 ) ;
          }
          _m_pb [ X_STB_Key ( _stn_s , _stn_e ) ] = _sx ;
          _m_pb_orig [ _stn_s ] = _sx ;
          _m_pb_dest [ _stn_e ] = _sx ;
        }
      }
    }
  }
  else
  {
    cerr << "Sorry, not yet implemented." << endl ;
  }
}




// ----
// Setting X_TrainBranch objects in X_ScheduleSet object.
// ----
void
X_ScheduleSet :: setTrainBranches
( X_Depot_Container const * _xdpt )
{
  // ----
  // Check if this is the first schedule.
  // ----
  if ( ! Original () )
  {
    // ----
    // For each train...
    // ----
    X_Train_Container * __p = dynamic_cast < X_Train_Container * > ( _trnc ) ;
    if ( ! __p )
    {
      cerr << "Error: in X_ScheduleSet(\"" << getName() << "\")::setTrain"
           << "Branches(): failed to retrieve X_Train_Container pointer"
           << endl ;
      exit ( 1 ) ;
    }
    for ( X_Train_Container :: iterator ii = __p -> begin () ;
          ii != __p -> end () ; ++ ii )
    {
      X_Train_Base *
        p_ii = dynamic_cast < X_Train_Base * > ( ii -> second ) ;
      if ( ! p_ii )
      {
        cerr << "Error: in X_ScheduleSet(\"" << getName() << "\")::setTrain"
             << "Branches(): failed to retrieve X_Train_Base pointer"
             << endl ;
        exit ( 1 ) ;
      }

      vector < X_Train_Time_Initialiser > :: iterator
        ttii = p_ii -> firstTimeInitialiser () ;
      vector < X_TrainSet_Base const * > v_tset ;
      v_tset . clear () ;
      map < X_TrainSet_Base const * , string > tset_action ;
      tset_action . clear () ;
      for ( vector < X_SpaceTimeNode * > :: const_iterator
              ix = p_ii -> firstNode () ; ix != p_ii -> lastNode () - 1 ;
            ++ ix )
      {
        ++ ttii ;
        vector < X_Train_Time_Initialiser > :: iterator
          ttix = ttii - 1 ;

        // ----
        // For each stop...
        // ----
        X_SpaceTimeNode const * _nns = * ix ;
        X_SpaceTimeNode const * _nne = * ( ix + 1 ) ;
        double _ts = ( * ix ) -> getTime () ;
        int _cds = ( * ix ) -> getCycleDelay () ;
        double _te = ( * ( ix + 1 ) ) -> getTime () ;
        int _cde = ( * ( ix + 1 ) ) -> getCycleDelay () ;
        double _tb = ( _te + _cde * X_SpaceTimeNode :: CYCLE_TIME )
          - ( _ts + _cds * X_SpaceTimeNode :: CYCLE_TIME ) ;

        // ----
        // Check trainset(s).
        // ----
        int _f_ins = 1 ;
//         bool _f_in_found = false ;
        bool _b_in_found = false ;
        unsigned int _f_outs = 0 ;
        unsigned int _b_outs = 0 ;
        vector < string > _out_actions ;
        _out_actions . clear () ;
        vector < string > _out_classes ;
        _out_classes . clear () ;
        vector < string > _out_targets ;
        _out_targets . clear () ;
        bool _out_front = false ;
        bool _out_back = false ;
        map < X_TrainSet_Base const * , X_TrainBranch * > _tbmc ;
        _tbmc . clear () ;
        X_STB_Key _stbk_c = X_STB_Key () ;
        for ( vector < X_TrainSet_Initialiser > :: const_iterator
                tsii = ttix -> v_tsi . begin () ;
              tsii != ttix -> v_tsi . end () ; ++ tsii )
        {
          cerr << "Trainset: " << tsii -> _name << ", action/class/target = "
               << tsii -> _action << "/" << tsii -> _class << "/"
               << tsii -> _target << endl ;
          bool _is_f_in = ( tsii -> _action == "f_in" ) ;
          bool _is_b_in = ( tsii -> _action == "b_in" ) ;
          bool _is_c_f_in = ( tsii -> _action == "c_f_in" ) ;
          bool _is_c_b_in = ( tsii -> _action == "c_b_in" ) ;
          bool _is_d_f_in = ( tsii -> _action == "d_f_in" ) ;
          bool _is_d_b_in = ( tsii -> _action == "d_b_in" ) ;
          if ( _is_f_in || _is_b_in )
          {
            if ( _is_f_in && ( ! _f_ins ) )
            {
              cerr << "Error: in X_ScheduleSet(\"" << getName() << "\")::"
                   << "setTrainBranches(): trainset data inconsistent, "
                   << "action f_in found after trainsets with other actions"
                   << endl ;
              exit ( 1 ) ;
            }
            if ( _b_in_found && ( ! _is_b_in ) )
            {
              cerr << "Error: in X_ScheduleSet(\"" << getName() << "\")::"
                   << "setTrainBranches(): trainset data inconsistent, "
                   << "action b_in found before trainsets with other actions"
                   << endl ;
              exit ( 1 ) ;
            }
            if ( _is_f_in )
            {
              // do nothing
            }
            else
            {
              _b_in_found = true ;
            }
            if ( tsii -> _class == "sujic_depot" )
            {
              X_Depot_Base const *
                __dx = _xdpt -> locateDepot ( tsii -> _target ) ;
              if ( ! __dx )
              {
                cerr << "Error: in X_ScheduleSet(\"" << getName() << "\")::"
                     << "setTrainBranches(): failed to locate depot \""
                     << tsii -> _target << "\"" << endl ;
                exit ( 1 ) ;
              }
              X_TrainSet_Base const *
                x = __dx -> locateTrainSet ( tsii -> _name ) ;
              if ( ! x )
              {
                cerr << "Error: in X_ScheduleSet(\"" << getName() << "\")::"
                     << "setTrainBranches(): failed to locate trainset \""
                     << tsii -> _name << "\" in depot \"" << tsii -> _target
                     << endl ;
                exit ( 1 ) ;
              }
              if ( _is_f_in )
              {
                v_tset . insert ( v_tset . begin () + ( _f_ins - 1 ) , x ) ;
                ++ _f_ins ;
              }
              else
              {
                v_tset . push_back ( x ) ;
                _f_ins = 0 ;
              }
            }
            else
            {
              cerr << "Error: in X_ScheduleSet(\"" << getName() << "\")::"
                   << "setTrainBranches(): trainset \"" << tsii -> _name
                   << "\" must start from a depot" << endl ;
              exit ( 1 ) ;
            }
          }
          else if ( _is_c_f_in || _is_c_b_in || _is_d_f_in || _is_d_b_in )
          {
            if ( ( _is_c_f_in || _is_d_f_in ) && ( ! _f_ins ) )
            {
              cerr << "Error: in X_ScheduleSet(\"" << getName() << "\")::"
                   << "setTrainBranches(): trainset data inconsistent, "
                   << "action f_in found after trainsets with other actions"
                   << endl ;
              exit ( 2 ) ;
            }
            if ( _b_in_found && ( ! ( _is_c_b_in || _is_d_b_in ) ) )
            {
              cerr << "Error: in X_ScheduleSet(\"" << getName() << "\")::"
                   << "setTrainBranches(): trainset data inconsistent, "
                   << "action b_in found before trainsets with other actions"
                   << endl ;
              exit ( 2 ) ;
            }
            if ( _is_c_f_in || _is_d_f_in )
            {
              // do nothing
            }
            else
            {
              _b_in_found = true ;
            }
            if ( tsii -> _class == "sujic_train" )
            {
              X_SpaceTimeNode const * xnorg
                = _nns -> getPlatform () -> getConnectedPlatformNode ( _nns ) ;
              if ( ! xnorg )
              {
                cerr << "Error: connected platform node could not be found "
                     << "for train node \"" << _nns -> getName () << "\" "
                     << "in initialisation of trainset for c_b_in or c_f_in"
                     << endl ;
                exit ( 1 ) ;
              }
              X_PlatformBranch * _ppb = getIncomingPlatformBranch ( xnorg ) ;
              X_TrainSet_Base const * xts = 0 ;
              if ( _ppb )
              {
                string scc ( "" ) ;
                X_Train_Base const * xtb = 0 ;
                X_SpaceTimeNode const * xn = 0 ;
                _ppb -> getRemainingTrainSet ( 0 , xts , scc , xtb , xn ) ;
                if ( xtb -> getName () != tsii -> _target )
                {
                  cerr << "Error: trainset remaining on the platform branch "
                       << "differs from what is specified in data" << endl ;
                  exit ( 1 ) ;
                }
                cerr << "Unset trainset " << xts -> getName () << ": "
                     << _ppb -> getEndSTN () -> getName () << " <-- "
                     << _ppb -> getStartSTN () -> getName () << endl ;
                _ppb -> removeTrainSet ( 0 ) ;
                while ( _ppb -> getStartSTN () -> getTrain () != xtb )
                {
                  if ( xnorg == _ppb -> getStartSTN () )
                  {
                    cerr << "Error: finding space time node with train "
                         << xtb -> getName () << " failed in processing "
                         << "originating trainset for train "
                         << p_ii -> getName () << endl ;
                    exit ( 1 ) ;
                  }
                  _ppb
                    = getIncomingPlatformBranch ( _ppb -> getStartSTN () ) ;
                  if ( ! _ppb )
                  {
                    cerr << "Error: finding branch failed in processing "
                         << "originating trainset for train "
                         << p_ii -> getName () << endl ;
                    exit ( 1 ) ;
                  }
                  X_TrainSet_Base const * xtz = 0 ;
                  X_Train_Base const * xty = 0 ;
                  string scz ( "" ) ;
                  X_SpaceTimeNode const * zn = 0 ;
                  _ppb -> getRemainingTrainSet ( 0 , xtz , scz , xty , zn ) ;
                  if ( xtz != xts || scz != scc || xty != xtb || zn != xn )
                  {
                    cerr << "Error: trainset remaining on the platform "
                         << "branch differs from what is specified in data"
                         << endl ;
                    exit ( 1 ) ;
                  }
                  cerr << "Unset trainset " << xtz -> getName () << ": "
                       << _ppb -> getEndSTN () -> getName () << " <-- "
                       << _ppb -> getStartSTN () -> getName () << endl ;
                  _ppb -> removeTrainSet ( 0 ) ;
                }
                if ( scc == "c_out" )
                {
                  // ----
                  // Action c_out -> c_b_in / c_f_in: create branch
                  // ----
                  if ( _is_d_b_in || _is_d_f_in )
                  {
                    cerr << "Error: action not inconsistent" << endl ;
                    exit ( 1 ) ;
                  }
                  if ( _tbmc . find ( xts ) != _tbmc . end () )
                  {
                    cerr << "Error: creating an element in m_tb: "
                         << "duplicate trainset?" << endl ;
                    exit ( 2 ) ;
                  }
                  cerr << "Branch (train c_out-c_in): " << _tb << ", "
                       << xn -> getName () << " [" << xn -> getTime ()
                       << "/" << xn -> getCycleDelay () << "] -> "
                       << _nns -> getName () << " [" << _ts << "/" << _cds
                       << "], trainset " << xts -> getName () << endl ;
                  X_TrainBranch * _br = new X_TrainBranch
                    ( xn , _nns , 0 , p_ii , xts ) ;
                  _tbmc [ xts ] = _br ;
                  setTrainSetForNode ( xn , xts ) ;
                  setTrainSetForNode ( _nns , xts ) ;
                  if ( _stbk_c . getStartSTN ()
                       && _stbk_c != X_STB_Key ( xn , _nns ) )
                  {
                    cerr << "Error: X_STB_Key differs in finding connecting "
                         << "trainsets" << endl ;
                    exit ( 1 ) ;
                  }
                  _stbk_c = X_STB_Key ( xn , _nns ) ;
                }
              }
              else
              {
                cerr << "Error: could not find connecting trainsets: "
                     << "branch not found" << endl ;
                exit ( 1 ) ;
              }
              if ( _is_c_f_in || _is_d_f_in )
              {
                v_tset . insert
                  ( v_tset . begin () + ( _f_ins - 1 ) , xts ) ;
                ++ _f_ins ;
              }
              else
              {
                v_tset . push_back ( xts ) ;
                _f_ins = 0 ;
              }
            }
            else
            {
              cerr << "Error: in X_ScheduleSet(\"" << getName() << "\")::"
                   << "setTrainBranches(): trainset \"" << tsii -> _name
                   << "\" must start using a trainset at platform" << endl ;
              exit ( 1 ) ;
            }
          }
          else if ( tsii -> _action == "out" || tsii -> _action == "c_out"
                    || tsii -> _action == "d_out" )
          {
            if ( tsii == ttix -> v_tsi . begin () )
            {
              _out_front = true ;
            }
            if ( _out_front )
            {
              ++ _f_outs ;
            }
            else if ( ! _out_back )
            {
              _b_outs = ttix -> v_tsi . end () - tsii ;
              _out_back = true ;
            }
            _out_actions . push_back ( tsii -> _action ) ;
            _out_classes . push_back ( tsii -> _class ) ;
            _out_targets . push_back ( tsii -> _target ) ;
          }
          else if ( tsii -> _action != "stay" && tsii -> _action != "cont" )
          {
            cerr << "Error: in X_ScheduleSet(\"" << getName() << "\")::"
                 << "setTrainBranches(): trainset \"" << tsii -> _name
                 << "\" with incorrect action " << tsii -> _action << endl ;
            exit ( 1 ) ;
          }
        }
        if ( _tbmc . size () )
        {
          if ( _m_tb . find ( _stbk_c ) != _m_tb . end () )
          {
            cerr << "Error: in X_ScheduleSet(\"" << getName() << "\")::"
                 << "setTrainBranches(): X_TrainBranch already "
                 << "present in _m_tb" << endl ;
            exit ( 2 ) ;
          }
          _m_tb [ _stbk_c ] = _tbmc ;
          
        }

        // ----
        // Remove trains, and if necessary, put trains to platform branch.
        // ----
        if ( _f_outs )
        {
          if ( _f_outs != _out_actions . size () )
          {
            cerr << "Error: out_actions.size() and _f_outs differ" << endl ;
            exit ( 1 ) ;
          }
          vector < string > :: const_iterator ifa = _out_actions . begin () ;
          vector < string > :: const_iterator ita = _out_targets . begin () ;
          for ( vector < X_TrainSet_Base const * > :: const_iterator
                  ii = v_tset . begin () ;
                ii != v_tset . begin () + _f_outs ; ++ ii , ++ ifa , ++ ita )
          {
            // ----
            // If *ifa is "out", just discard the trainset. Otherwise, put
            // this trainset onto the platform branch, if it exists.
            // ----
            if ( ( * ifa ) == "c_out" || ( * ifa ) == "d_out" )
            {
              X_SpaceTimeNode const * _nps
                = _nns -> getPlatform () -> getConnectedPlatformNode ( _nns ) ;
              if ( ! _nps )
              {
                cerr << "Error: connected platform node could not be found "
                     << "for node \"" << _nns -> getName () << "\"" << endl ;
                exit ( 1 ) ;
              }
              X_SpaceTimeNode const * _nps_orig = _nps ;
              do
              {
                X_PlatformBranch * _pbp = getOutgoingPlatformBranch ( _nps ) ;
                if ( _pbp )
                {
                  _pbp -> setTrainSet ( * ii , * ifa , p_ii , _nns ) ;
                  cerr << "Set trainset " << ( * ii ) -> getName () << ": "
                       << _pbp -> getStartSTN () -> getName () << " --> "
                       << _pbp -> getEndSTN () -> getName () << endl ;
                }
                else
                {
                  cerr << "Error: action " << ( * ifa ) << " specified, but "
                       << "branch could not be found" << endl ;
                  exit ( 1 ) ;
                }
                _nps = _pbp -> getEndSTN () ;
                if ( _nps == _nps_orig )
                {
                  cerr << "Error: finding train " << ( * ita ) << " failed"
                       << endl ;
                  exit ( 1 ) ;
                }
              }
              while ( _nps -> getTrain () -> getName () != ( * ita ) ) ;
            }
          }
          v_tset . erase ( v_tset . begin () , v_tset . begin () + _f_outs ) ;
        }
        else if ( _b_outs )
        {
          if ( _b_outs != _out_actions . size () )
          {
            cerr << "Error: out_actions.size() and _b_outs differ" << endl ;
            exit ( 1 ) ;
          }
          vector < string > :: const_iterator ifb = _out_actions . begin () ;
          vector < string > :: const_iterator itb = _out_targets . begin () ;
          for ( vector < X_TrainSet_Base const * > :: const_iterator
                  ii = v_tset . end () - _b_outs ;
                ii != v_tset . end () ; ++ ii , ++ ifb , ++ itb )
          {
            // ----
            // If *ifb is "out", just discard the trainset. Otherwise, put
            // this trainset onto the platform branch, if it exists.
            // ----
            if ( ( * ifb ) == "c_out" || ( * ifb ) == "d_out" )
            {
              X_SpaceTimeNode const * _nps
                = _nns -> getPlatform () -> getConnectedPlatformNode ( _nns ) ;
              if ( ! _nps )
              {
                cerr << "Error: connected platform node could not be found "
                     << "for node \"" << _nns -> getName () << "\"" << endl ;
                exit ( 2 ) ;
              }
              X_SpaceTimeNode const * _nps_orig = _nps ;
              do
              {
                X_PlatformBranch * _pbp
                  = getOutgoingPlatformBranch ( _nps ) ;
                if ( _pbp )
                {
                  _pbp -> setTrainSet ( * ii , * ifb , p_ii , _nns ) ;
                  cerr << "Set trainset " << ( * ii ) -> getName () << ": "
                       << _pbp -> getStartSTN () -> getName () << " --> "
                       << _pbp -> getEndSTN () -> getName () << endl ;
                }
                else
                {
                  cerr << "Error: action " << ( * ifb ) << " specified, but "
                       << "branch could not be found" << endl ;
                  exit ( 2 ) ;
                }
                _nps = _pbp -> getEndSTN () ;
                if ( _nps == _nps_orig )
                {
                  cerr << "Error: finding train " << ( * itb ) << " failed"
                       << endl ;
                  exit ( 2 ) ;
                }
              }
              while ( _nps -> getTrain () -> getName () != ( * itb ) ) ;
            }
          }
          v_tset . erase ( v_tset . end () - _b_outs , v_tset . end () ) ;
        }

        // ----
        // create branch for each trainset.
        // ----
        map < X_TrainSet_Base const * , X_TrainBranch * > _tbmx ;
        _tbmx . clear () ;
        for ( vector < X_TrainSet_Base const * > :: const_iterator
                ii = v_tset . begin () ; ii != v_tset . end () ; ++ ii )
        {
          if ( _tbmx . find ( * ii ) != _tbmx . end () )
          {
            cerr << "Error: creating an element in m_tb: duplicate trainset?"
                 << endl ;
            exit ( 1 ) ;
          }
          cerr << "Branch (train): " << _tb << ", " << _nns -> getName ()
               << " [" << _ts << "/" << _cds << "] -> " << _nne -> getName ()
               << " [" << _te << "/" << _cde << "], trainset "
               << ( * ii ) -> getName () << endl ;
          X_TrainBranch *
            _br = new X_TrainBranch ( _nns , _nne , 0 , p_ii , ( * ii ) ) ;
          _tbmx [ ( * ii ) ] = _br ;
          setTrainSetForNode ( _nns , * ii ) ;
          setTrainSetForNode ( _nne , * ii ) ;
        }
        if ( _m_tb . find ( X_STB_Key ( _nns , _nne ) ) != _m_tb . end () )
        {
          cerr << "Error: in X_ScheduleSet(\"" << getName() << "\")::"
               << "setTrainBranches(): X_TrainBranch already "
               << "present in _m_tb" << endl ;
          exit ( 1 ) ;
        }
        _m_tb [ X_STB_Key ( _nns , _nne ) ] = _tbmx ;
        _m_tset [ X_STB_Key ( _nns , _nne ) ] = v_tset ;
      }

      // ----
      // Processing the last node. Do not create a branch.
      // ----
      vector < X_SpaceTimeNode * > :: const_iterator
        ix = p_ii -> lastNode () - 1 ;
      vector < X_Train_Time_Initialiser > :: iterator ttix = ttii ;

      X_SpaceTimeNode const * _nns = * ix ;

      // ----
      // Check trainset(s).
      // ----
      int _f_ins = 1 ;
      bool _b_in_found = false ;
      unsigned int _f_outs = 0 ;
      unsigned int _b_outs = 0 ;
      vector < string > _out_actions ;
      _out_actions . clear () ;
      vector < string > _out_classes ;
      _out_classes . clear () ;
      vector < string > _out_targets ;
      _out_targets . clear () ;
      bool _out_front = false ;
      bool _out_back = false ;
      for ( vector < X_TrainSet_Initialiser > :: const_iterator
              tsii = ttix -> v_tsi . begin () ;
            tsii != ttix -> v_tsi . end () ; ++ tsii )
      {
        cerr << "Trainset: " << tsii -> _name << ", action/class/target = "
             << tsii -> _action << "/" << tsii -> _class << "/"
             << tsii -> _target << endl ;
        bool _is_f_in = ( tsii -> _action == "f_in" ) ;
        bool _is_b_in = ( tsii -> _action == "b_in" ) ;
        if ( _is_f_in || _is_b_in )
        {
          if ( _is_f_in && ( ! _f_ins ) )
          {
            cerr << "Error: in X_ScheduleSet(\"" << getName() << "\")::"
                 << "setTrainBranches(): trainset data inconsistent, "
                 << "action f_in found after trainsets with other actions"
                 << endl ;
            exit ( 2 ) ;
          }
          if ( _b_in_found && ( ! _is_b_in ) )
          {
            cerr << "Error: in X_ScheduleSet(\"" << getName() << "\")::"
                 << "setTrainBranches(): trainset data inconsistent, "
                 << "action b_in found before trainsets with other actions"
                 << endl ;
            exit ( 2 ) ;
          }
          if ( _is_f_in )
          {
            // do nothing
          }
          else
          {
            _b_in_found = true ;
          }
          if ( tsii -> _class == "sujic_depot" )
          {
            X_Depot_Base const *
              __dx = _xdpt -> locateDepot ( tsii -> _target ) ;
            if ( ! __dx )
            {
              cerr << "Error: in X_ScheduleSet(\"" << getName() << "\")::"
                   << "setTrainBranches(): failed to locate depot \""
                   << tsii -> _target << "\"" << endl ;
              exit ( 2 ) ;
            }
            X_TrainSet_Base const *
              x = __dx -> locateTrainSet ( tsii -> _name ) ;
            if ( ! x )
            {
              cerr << "Error: in X_ScheduleSet(\"" << getName() << "\")::"
                   << "setTrainBranches(): failed to locate trainset \""
                   << tsii -> _name << "\" in depot \"" << tsii -> _target
                   << endl ;
              exit ( 2 ) ;
            }
            if ( _is_f_in )
            {
              v_tset . insert ( v_tset . begin () + ( _f_ins - 1 ) , x ) ;
              ++ _f_ins ;
            }
            else
            {
              v_tset . push_back ( x ) ;
              _f_ins = 0 ;
            }
          }
          else
          {
            cerr << "Error: in X_ScheduleSet(\"" << getName() << "\")::"
                 << "setTrainBranches(): trainset \"" << tsii -> _name
                 << "\" must start from a depot" << endl ;
            exit ( 2 ) ;
          }
        }
        else if ( tsii -> _action == "out" || tsii -> _action == "c_out"
                  || tsii -> _action == "d_out" )
        {
          if ( tsii == ttix -> v_tsi . begin () )
          {
            _out_front = true ;
          }
          if ( _out_front )
          {
            ++ _f_outs ;
          }
          else if ( ! _out_back )
          {
            _b_outs = ttix -> v_tsi . end () - tsii ;
            _out_back = true ;
          }
          _out_actions . push_back ( tsii -> _action ) ;
        }
        else if ( tsii -> _action != "stay" && tsii -> _action != "cont" )
        {
          cerr << "Error: in X_ScheduleSet(\"" << getName() << "\")::"
               << "setTrainBranches(): trainset \"" << tsii -> _name
               << "\" with incorrect action " << tsii -> _action << endl ;
          exit ( 2 ) ;
        }
      }

      // ----
      // Remove trains, and if necessary, put trains to platform branch.
      // ----
      if ( _f_outs )
      {
        if ( _f_outs != _out_actions . size () )
        {
          cerr << "Error: out_actions.size() and _f_outs differ" << endl ;
          exit ( 2 ) ;
        }
        vector < string > :: const_iterator ifa = _out_actions . begin () ;
        vector < string > :: const_iterator ita = _out_targets . begin () ;
        for ( vector < X_TrainSet_Base const * > :: const_iterator
                ii = v_tset . begin () ;
              ii != v_tset . begin () + _f_outs ; ++ ii , ++ ifa , ++ ita )
        {
          // ----
          // If *ifa is "out", just discard the trainset. Otherwise, put
          // this trainset onto the platform branch, if it exists.
          // ----
          if ( ( * ifa ) == "c_out" || ( * ifa ) == "d_out" )
          {
            X_SpaceTimeNode const * _nps
              = _nns -> getPlatform () -> getConnectedPlatformNode ( _nns ) ;
            if ( ! _nps )
            {
              cerr << "Error: connected platform node could not be found "
                   << "for train node \"" << _nns -> getName () << "\" "
                   << "in initialisation of trainset for c_out or d_out"
                   << endl ;
              exit ( 1 ) ;
            }
            X_SpaceTimeNode const * _nps_orig = _nps ;
            do
            {
              X_PlatformBranch * _pbp
                = getOutgoingPlatformBranch ( _nps ) ;
              if ( _pbp )
              {
                _pbp -> setTrainSet ( * ii , * ifa , p_ii , _nns ) ;
                cerr << "Set trainset " << ( * ii ) -> getName () << ": "
                     << _pbp -> getStartSTN () -> getName () << " --> "
                     << _pbp -> getEndSTN () -> getName () << endl ;
              }
              else
              {
                cerr << "Error: action " << ( * ifa ) << " specified, but "
                     << "branch could not be found" << endl ;
                exit ( 3 ) ;
              }
              _nps = _pbp -> getEndSTN () ;
              if ( _nps == _nps_orig )
              {
                cerr << "Error: finding train " << ( * ita ) << " failed"
                     << endl ;
                exit ( 3 ) ;
              }
            }
            while ( _nps -> getTrain () -> getName () != ( * ita ) ) ;
          }
        }
        v_tset . erase ( v_tset . begin () , v_tset . begin () + _f_outs ) ;
      }
      else if ( _b_outs )
      {
        if ( _b_outs != _out_actions . size () )
        {
          cerr << "Error: out_actions.size() and _b_outs differ" << endl ;
          exit ( 2 ) ;
        }
        vector < string > :: const_iterator ifb = _out_actions . begin () ;
        vector < string > :: const_iterator itb = _out_targets . begin () ;
        for ( vector < X_TrainSet_Base const * > :: const_iterator
                ii = v_tset . end () - _b_outs ;
              ii != v_tset . end () ; ++ ii , ++ ifb , ++ itb )
        {
          // ----
          // If *ifb is "out", just discard the trainset. Otherwise, put
          // this trainset onto the platform branch, if it exists.
          // ----
          if ( ( * ifb ) == "c_out" || ( * ifb ) == "d_out" )
          {
            cerr << "XXXXX---005" << endl ;
            X_SpaceTimeNode const * _nps
              = _nns -> getPlatform () -> getConnectedTrainNode ( _nns ) ;
            cerr << "XXXXX---005 end" << endl ;
            X_SpaceTimeNode const * _nps_orig = _nps ;
            do
            {
              X_PlatformBranch * _pbp
                = getOutgoingPlatformBranch ( _nps ) ;
              if ( _pbp )
              {
                _pbp -> setTrainSet ( * ii , * ifb , p_ii , _nns ) ;
                cerr << "Set trainset " << ( * ii ) -> getName () << ": "
                     << _pbp -> getStartSTN () -> getName () << " --> "
                     << _pbp -> getEndSTN () -> getName () << endl ;
              }
              else
              {
                cerr << "Error: action " << ( * ifb ) << " specified, but "
                     << "branch could not be found" << endl ;
                exit ( 4 ) ;
              }
              _nps = _pbp -> getEndSTN () ;
              if ( _nps == _nps_orig )
              {
                cerr << "Error: finding train " << ( * itb ) << " failed"
                     << endl ;
                exit ( 1 ) ;
              }
            }
            while ( _nps -> getTrain () -> getName () != ( * itb ) ) ;
          }
        }
        v_tset . erase ( v_tset . end () - _b_outs , v_tset . end () ) ;
      }
    }
  }
  else
  {
    cerr << "Sorry, not yet implemented." << endl ;
  }
}




// ----
// Get a platform branch (non-const).
// ----
X_PlatformBranch *
X_ScheduleSet :: getPlatformBranch
( X_SpaceTimeNode const * x ,
  X_SpaceTimeNode const * y )
{
  if ( ! x )
  {
    cerr << "Error: null pointer for X_ScheduleSet::"
         << "getPlatformBranch()" << endl ;
    exit ( 1 ) ;
  }
  map < X_STB_Key , X_PlatformBranch * > :: iterator
    ii = _m_pb . find ( X_STB_Key ( x , y ) ) ;
  if ( ii == _m_pb . end () )
  {
    return 0 ;
  }
  return dynamic_cast < X_PlatformBranch * > ( ii -> second ) ;
}




// ----
// Get a platform branch (const).
// ----
X_PlatformBranch const *
X_ScheduleSet :: getPlatformBranch
( X_SpaceTimeNode const * x ,
  X_SpaceTimeNode const * y )
  const
{
  if ( ! x )
  {
    cerr << "Error: null pointer for X_ScheduleSet::"
         << "getPlatformBranch() const" << endl ;
    exit ( 1 ) ;
  }
  map < X_STB_Key , X_PlatformBranch * > :: const_iterator
    ii = _m_pb . find ( X_STB_Key ( x , y ) ) ;
  if ( ii == _m_pb . end () )
  {
    return 0 ;
  }
  return dynamic_cast < X_PlatformBranch const * > ( ii -> second ) ;
}




// ----
// Get a vector of trainset.
// ----
vector < X_TrainSet_Base const * > const *
X_ScheduleSet :: getTrainSets
( X_SpaceTimeNode const * x ,
  X_SpaceTimeNode const * y )
  const
{
  if ( ! x || ! y )
  {
    cerr << "Error: null pointer for X_ScheduleSet::"
         << "getTrainSets()" << endl ;
    exit ( 1 ) ;
  }
  map < X_STB_Key , vector < X_TrainSet_Base const * > > :: const_iterator
    ii = _m_tset . find ( X_STB_Key ( x , y ) ) ;
  if ( ii == _m_tset . end () )
  {
    return 0 ;
  }
  return & ( ii -> second ) ;
}




// ----
// Get a train branch (non-const).
// ----
X_TrainBranch *
X_ScheduleSet :: getTrainBranch
( X_SpaceTimeNode const * x ,
  X_SpaceTimeNode const * y ,
  X_TrainSet_Base const * ts )
{
  if ( ! x || ! y || ! ts )
  {
    cerr << "Error: null pointer for X_ScheduleSet::"
         << "getTrainBranch()" << endl ;
    exit ( 1 ) ;
  }
  map < X_STB_Key , map < X_TrainSet_Base const * , X_TrainBranch * > >
    :: iterator ii = _m_tb . find ( X_STB_Key ( x , y ) ) ;
  if ( ii == _m_tb . end () )
  {
    return 0 ;
  }
  map < X_TrainSet_Base const * , X_TrainBranch * > :: iterator
    jj = ii -> second . find ( ts ) ;
  if ( jj == ii -> second . end () )
  {
    return 0 ;
  }
  return jj -> second ;
}




// ----
// Get a train branch (const).
// ----
X_TrainBranch const *
X_ScheduleSet :: getTrainBranch
( X_SpaceTimeNode const * x ,
  X_SpaceTimeNode const * y ,
  X_TrainSet_Base const * ts )
  const
{
  if ( ! x || ! y || ! ts )
  {
    cerr << "Error: null pointer for X_ScheduleSet::"
         << "getTrainBranch() (const)" << endl ;
    exit ( 1 ) ;
  }
  map < X_STB_Key , map < X_TrainSet_Base const * , X_TrainBranch * > >
    :: const_iterator ii = _m_tb . find ( X_STB_Key ( x , y ) ) ;
  if ( ii == _m_tb . end () )
  {
    return 0 ;
  }
  map < X_TrainSet_Base const * , X_TrainBranch * > :: const_iterator
    jj = ii -> second . find ( ts ) ;
  if ( jj == ii -> second . end () )
  {
    return 0 ;
  }
  return jj -> second ;
}





// ----
// Get a "miscellaneous" branch (non-const).
// ----
X_Branch_Base *
X_ScheduleSet :: getMiscellaneousBranch
( X_SpaceTimeNode const * x ,
  X_SpaceTimeNode const * y )
{
  if ( ! x )
  {
    cerr << "Error: null pointer for X_ScheduleSet::"
         << "getMiscellaneousBranch()" << endl ;
    exit ( 1 ) ;
  }
  map < X_STB_Key , X_Branch_Base * > :: iterator
    ii = _m_misc . find ( X_STB_Key ( x , y ) ) ;
  if ( ii == _m_misc . end () )
  {
    return 0 ;
  }
  return dynamic_cast < X_Branch_Base * > ( ii -> second ) ;
}





// ----
// Get a "miscellaneous" branch (const).
// ----
X_Branch_Base const *
X_ScheduleSet :: getMiscellaneousBranch
( X_SpaceTimeNode const * x ,
  X_SpaceTimeNode const * y )
  const
{
  if ( ! x )
  {
    cerr << "Error: null pointer for X_ScheduleSet::"
         << "getMiscellaneousBranch() const" << endl ;
    exit ( 1 ) ;
  }
  map < X_STB_Key , X_Branch_Base * > :: const_iterator
    ii = _m_misc . find ( X_STB_Key ( x , y ) ) ;
  if ( ii == _m_misc . end () )
  {
    return 0 ;
  }
  return dynamic_cast < X_Branch_Base const * > ( ii -> second ) ;
}





// ----
// Get a platform branch (non-const).
// ----
X_PlatformBranch *
X_ScheduleSet :: getOutgoingPlatformBranch
( X_SpaceTimeNode const * x )
{
  if ( ! x )
  {
    cerr << "Error: null pointer for X_ScheduleSet::"
         << "getOutgoingPlatformBranch()" << endl ;
    exit ( 1 ) ;
  }
  map < X_SpaceTimeNode const * , X_PlatformBranch * > :: iterator
    ii = _m_pb_orig . find ( x ) ;
  if ( ii == _m_pb_orig . end () )
  {
    return 0 ;
  }
  return ii -> second ;
}


X_PlatformBranch *
X_ScheduleSet :: getIncomingPlatformBranch
( X_SpaceTimeNode const * x )
{
  if ( ! x )
  {
    cerr << "Error: null pointer for X_ScheduleSet::"
         << "getIncomingPlatformBranch()" << endl ;
    exit ( 1 ) ;
  }
  map < X_SpaceTimeNode const * , X_PlatformBranch * > :: iterator
    ii = _m_pb_dest . find ( x ) ;
  if ( ii == _m_pb_dest . end () )
  {
    return 0 ;
  }
  return ii -> second ;
}




// ----
// Get a platform branch (const).
// ----
X_PlatformBranch const *
X_ScheduleSet :: getOutgoingPlatformBranch
( X_SpaceTimeNode const * x )
  const
{
  if ( ! x )
  {
    cerr << "Error: null pointer for X_ScheduleSet::"
         << "getOutgoingPlatformBranch()" << endl ;
    exit ( 2 ) ;
  }
  map < X_SpaceTimeNode const * , X_PlatformBranch * > :: const_iterator
    ii = _m_pb_orig . find ( x ) ;
  if ( ii == _m_pb_orig . end () )
  {
    return 0 ;
  }
  return ii -> second ;
}



X_PlatformBranch const *
X_ScheduleSet :: getIncomingPlatformBranch
( X_SpaceTimeNode const * x )
  const
{
  if ( ! x )
  {
    cerr << "Error: null pointer for X_ScheduleSet::"
         << "getIncomingPlatformBranch()" << endl ;
    exit ( 2 ) ;
  }
  map < X_SpaceTimeNode const * , X_PlatformBranch * > :: const_iterator
    ii = _m_pb_orig . find ( x ) ;
  if ( ii == _m_pb_orig . end () )
  {
    return 0 ;
  }
  return ii -> second ;
}




// ----
// Create miscellaneous branches.
// ----
void
X_ScheduleSet :: setMiscBranches
( X_Station_Container * xs_c )
{
  // ----
  // Check if this is the first schedule.
  // ----
  if ( ! Original () )
  {
    // ----
    // For each station...
    // ----
    for ( X_Station_Container :: iterator ii = xs_c -> begin () ;
          ii != xs_c -> end () ; ++ ii )
    {
      X_Station_Base *
        p_ii = dynamic_cast < X_Station_Base * > ( ii -> second ) ;
      if ( ! p_ii )
      {
        cerr << "Error: in X_ScheduleSet(\"" << getName() << "\")::setPlat"
             << "formBranches(): failed to retrieve X_Station_Base pointer"
             << endl ;
        exit ( 1 ) ;
      }

      // ----
      // Entrance -> departure nodes at entrance platform
      // ----
      X_Platform_Base const * p_e = p_ii -> Entrance () ;
      X_SpaceTimeNode const * n_e = p_ii -> EntranceNode () ;
      for ( vector < X_SpaceTimeNode * > :: const_iterator
              ix = p_e -> firstNode () ; ix != p_e -> lastNode () ; ++ ix )
      {
        X_SpaceTimeNode const * s_e = * ix ;
        if ( s_e -> isDeparting () )
        {
          cerr << "Branch (entrance): " << n_e -> getName () << " [--/--] -> "
               << s_e -> getName () << " [" << s_e -> getTime () << "/"
               << s_e -> getCycleDelay () << "] " << endl ;
          X_Branch_Base * b_ent = new X_Branch_Base ( n_e , s_e , 0 ) ;
          _m_misc [ X_STB_Key ( n_e , s_e ) ] = b_ent ;
        }
      }

      // ----
      // Arrival nodes at exit platform -> exit
      // ----
      X_Platform_Base const * p_x = p_ii -> Exit () ;
      X_SpaceTimeNode const * n_x = p_ii -> ExitNode () ;
      for ( vector < X_SpaceTimeNode * > :: const_iterator
              ix = p_x -> firstNode () ; ix != p_x -> lastNode () ; ++ ix )
      {
        X_SpaceTimeNode const * e_e = * ix ;
        if ( ! e_e -> isDeparting () )
        {
          cerr << "Branch (exit): " << e_e -> getName () << " ["
               << e_e -> getTime () << "/" << e_e -> getCycleDelay () << "] "
               << "-> " << n_x -> getName () << " [--/--]" << endl ;
          X_Branch_Base * b_exit = new X_Branch_Base ( e_e , n_x , 0 ) ;
          _m_misc [ X_STB_Key ( e_e , n_x ) ] = b_exit ;
        }
      }
      
      // ----
      // Transfer branches and intra-station walking branches
      // ----
      for ( X_Platform_Container :: iterator jj = p_ii -> firstPlatform () ;
            jj != p_ii -> lastPlatform () ; ++ jj )
      {
        X_Platform_Base *
          p_jj = dynamic_cast < X_Platform_Base * > ( jj -> second ) ;
        if ( ! p_jj )
        {
          cerr << "Error: in X_ScheduleSet(\"" << getName() << "\")::setPlat"
               << "formBranches(): failed to retrieve X_Platform_Base pointer"
               << endl ;
          exit ( 1 ) ;
        }

        for ( vector < X_SpaceTimeNode * > :: const_iterator ix
                = p_jj -> firstNode () ; ix != p_jj -> lastNode () ; ++ ix )
        {
          // ----
          // Transfer branches
          // ----
          X_SpaceTimeNode const * _ctrn
            = p_jj -> getConnectedTrainNode ( * ix ) ;
          if ( _ctrn )
          {
            if ( ( * ix ) -> isDeparting () )
            {
              cerr << "Branch (board): " << ( * ix ) -> getName () << " ["
                   << ( * ix ) -> getTime () << "/"
                   << ( * ix ) -> getCycleDelay () << "] -> "
                   << _ctrn -> getName () << endl ;
              X_Branch_Base *
                _sx = new X_Branch_Base ( ( * ix ) , _ctrn , 0 ) ;
              if ( _m_trf . find ( X_STB_Key ( ( * ix ) , _ctrn ) )
                   != _m_trf . end () )
              {
                cerr << "Error: in X_ScheduleSet(\"" << getName() << "\")::"
                     << "setMiscBranches(): transfer branch already "
                     << "present in _m_trf" << endl ;
                exit ( 1 ) ;
              }
              _m_trf [ X_STB_Key ( ( * ix ) , _ctrn ) ] = _sx ;
            }
            else
            {
              cerr << "Branch (alight): " << _ctrn -> getName () << " -> "
                   << ( * ix ) -> getName () << " [" << ( * ix ) -> getTime ()
                   << "/" << ( * ix ) -> getCycleDelay () << "]" << endl ;
              X_Branch_Base *
                _sy = new X_Branch_Base ( _ctrn , ( * ix ) , 0 ) ;
              if ( _m_trf . find ( X_STB_Key ( _ctrn , ( * ix ) ) )
                   != _m_trf . end () )
              {
                cerr << "Error: in X_ScheduleSet(\"" << getName() << "\")::"
                     << "setMiscBranches(): transfer branch already "
                     << "present in _m_trf" << endl ;
                exit ( 2 ) ;
              }
              _m_trf [ X_STB_Key ( _ctrn , ( * ix ) ) ] = _sy ;
            }
          }

          // ----
          // Intra-station walking branches
          // ----
          X_SpaceTimeNode const * _opn
            = p_jj -> getOriginatingPlatformNode ( * ix ) ;
          if ( _opn )
          {
            cerr << "Branch (walk): " << _opn -> getName () << " ["
                 << _opn -> getTime () << "/" << _opn -> getCycleDelay ()
                 << "] -> " << ( * ix ) -> getName () << " ["
                 << ( * ix ) -> getTime () << "/"
                 << ( * ix ) -> getCycleDelay () << "]" << endl ;
            X_Branch_Base *
              _sx = new X_Branch_Base ( _opn , ( * ix ) , 0 ) ;
            if ( _m_misc . find ( X_STB_Key ( _opn , ( * ix ) ) )
                 != _m_misc . end () )
            {
              cerr << "Error: in X_ScheduleSet(\"" << getName() << "\")::"
                   << "setMiscBranches(): intra-station walk branch already "
                   << "present in _m_misc" << endl ;
              exit ( 1 ) ;
            }
            _m_misc [ X_STB_Key ( _opn , ( * ix ) ) ] = _sx ;
          }

          X_SpaceTimeNode const * _dpn
            = p_jj -> getDestinationPlatformNode ( * ix ) ;
          if ( _dpn )
          {
            cerr << "Branch (walk): " << ( * ix ) -> getName () << " ["
                 << ( * ix ) -> getTime () << "/"
                 << ( * ix ) -> getCycleDelay () << "] -> "
                 << _dpn -> getName () << " [" << _dpn -> getTime () << "/"
                 << _dpn -> getCycleDelay () << "]" << endl ;
            X_Branch_Base *
              _sy = new X_Branch_Base ( ( * ix ) , _dpn , 0 ) ;
            if ( _m_misc . find ( X_STB_Key ( ( * ix ) , _dpn ) )
                 != _m_misc . end () )
            {
              cerr << "Error: in X_ScheduleSet(\"" << getName() << "\")::"
                   << "setMiscBranches(): intra-station walk branch already "
                   << "present in _m_misc" << endl ;
              exit ( 2 ) ;
            }
            _m_misc [ X_STB_Key ( ( * ix ) , _dpn ) ] = _sy ;
          }
        }
      }
    }
  }
  else
  {
    cerr << "Sorry, not yet implemented." << endl ;
  }
}




void
X_ScheduleSet :: setTrainSetForNode
( X_SpaceTimeNode const * _n ,
  X_TrainSet_Base const * _s )
{
  cerr << "TS-NODE: " << _s -> getName () << " -- "
       << _n -> getName () << endl ;
  if ( _m_trs . find ( _n ) == _m_trs . end () )
  {
    _m_trs [ _n ] = set < X_TrainSet_Base const * > () ;
  }
  _m_trs [ _n ] . insert ( _s ) ;

  cerr << "in trainset set for node " << _n -> getName () << ":" ;
  for ( set < X_TrainSet_Base const * > :: iterator ii
          = _m_trs [ _n ] . begin () ; ii != _m_trs [ _n ] . end () ; ++ ii )
  {
    cerr << " " << ( * ii ) -> getName () ;
  }
  cerr << endl ;
} 



set < X_TrainSet_Base const * > const *
X_ScheduleSet :: getTrainSetsForNode
( X_SpaceTimeNode const * _n )
  const
{
  map < X_SpaceTimeNode const * , set < X_TrainSet_Base const * > >
    :: const_iterator ii = _m_trs . find ( _n ) ;
  if ( ii != _m_trs . end () )
  {
    return & ( ii -> second ) ;
  }
  else
  {
    return 0 ;
  }
}



void
X_ScheduleSet :: printResults
()
{
  cout << "RESULTS --- Platform Branches" << endl << endl ;
  for ( map < X_STB_Key , X_PlatformBranch * > :: iterator
          ii = firstPlatformBranch () ; ii != lastPlatformBranch () ; ++ ii )
  {
    double _pas_s = 0 ;
    double _pas_e = 0 ;
    for ( EmergingPassengersMap :: const_iterator
            jj = ii -> second -> firstEmergingPassengers () ;
          jj != ii -> second -> lastEmergingPassengers () ; ++ jj )
    {
      for ( EmergingPassengers :: const_iterator kk
              = jj -> second . begin () ; kk != jj -> second . end () ; ++ kk )
      {
        for ( EmergingPassengerSet :: const_iterator ix
                = kk -> second . begin () ;
              ix != kk -> second . end () ; ++ ix )
        {
          _pas_e += ( * ix ) -> PassengerNumber () ;
        }
      }
    }
    for ( vector < pair < X_Passenger_Token const * , double > >
            :: const_iterator jj = ii -> second -> firstPassenger () ;
          jj != ii -> second -> lastPassenger () ; ++ jj )
    {
      _pas_s += jj -> first -> PassengerNumber () * jj -> second ;
      _pas_e += jj -> first -> PassengerNumber () * jj -> second ;
    }
    cout << "Branch \"" << ii -> second -> getStartSTN () -> getName ()
         << "\" -> \"" << ii -> second -> getEndSTN () -> getName ()
         << "\": " << _pas_s << " -|- " << _pas_e << endl ;
    for ( EmergingPassengersMap :: const_iterator
            jj = ii -> second -> firstEmergingPassengers () ;
          jj != ii -> second -> lastEmergingPassengers () ; ++ jj )
    {
      for ( EmergingPassengers :: const_iterator kk
              = jj -> second . begin () ; kk != jj -> second . end () ; ++ kk )
      {
        for ( EmergingPassengerSet :: const_iterator ix
                = kk -> second . begin () ;
              ix != kk -> second . end () ; ++ ix )
        {
          cout << " [I] " << ( * ix ) -> Origin () -> getName () << ": ["
               << ( * ix ) -> Time () << "/" << ( * ix ) -> CycleDelay ()
               << "] -> " << ( * ix ) -> Destination () -> getName () << ": "
               << ( * ix ) -> PassengerNumber () << endl ;
        }
      }
    }
    for ( vector < pair < X_Passenger_Token const * , double > >
            :: const_iterator jj = ii -> second -> firstPassenger () ;
          jj != ii -> second -> lastPassenger () ; ++ jj )
    {
      cout << " [P] " << jj -> first -> Origin () -> getName () << ": ["
           << jj -> first -> Time () << "/" << jj -> first -> CycleDelay ()
           << "] -> " << jj -> first -> Destination () -> getName () << ": "
           << jj -> first -> PassengerNumber () << " * " << jj -> second
           << " = " << jj -> first -> PassengerNumber () * jj -> second
           << endl ;
    }
  }

  cout << endl << endl << "RESULTS --- Train Branches" << endl << endl ;
  for ( map < X_STB_Key , map
          < X_TrainSet_Base const * , X_TrainBranch * > > :: iterator
          ii = firstTrainBranches () ; ii != lastTrainBranches () ; ++ ii )
  {
    for ( map < X_TrainSet_Base const * , X_TrainBranch * > :: iterator
            jj = ii -> second . begin () ;
          jj != ii -> second . end () ; ++ jj )
    {
      double _pas_s = 0 ;
      for ( vector < pair < X_Passenger_Token const * , double > >
              :: const_iterator kk = jj -> second -> firstPassenger () ;
            kk != jj -> second -> lastPassenger () ; ++ kk )
      {
        _pas_s += kk -> first -> PassengerNumber () * kk -> second ;
      }
      cout << "TrainSet " << jj -> first -> getName () << " Branch \""
           << jj -> second -> getStartSTN () -> getName () << "\" -> \""
           << jj -> second -> getEndSTN () -> getName () << "\": "
           << _pas_s << " -|- " << _pas_s << endl ;
      for ( vector < pair < X_Passenger_Token const * , double > >
              :: const_iterator kk = jj -> second -> firstPassenger () ;
            kk != jj -> second -> lastPassenger () ; ++ kk )
      {
        cout << " [P] " << kk -> first -> Origin () -> getName () << ": ["
             << kk -> first -> Time () << "/" << kk -> first -> CycleDelay ()
             << "] -> " << kk -> first -> Destination () -> getName ()
             << ": " << kk -> first -> PassengerNumber () << " * "
             << kk -> second << " = "
             << kk -> first -> PassengerNumber () * kk -> second << endl ;
      }
    }
  }
}
