// -*- C++ -*-

// ダイクストラ法のデモ

#include "Expandable.hh"
#include "RS_XmlReader.hh"
#include "X_Train.hh"
#include "X_SpaceTimeBranch.hh"

#ifndef X_ScheduleSet_HH
#define X_ScheduleSet_HH


#include <algorithm>
#include <iostream>
#include <vector>
#include <map>
#include <set>


using std :: cerr ;
using std :: endl ;
using std :: vector ;
using std :: map ;
using std :: sort ;
using std :: set ;



class X_Station_Container ;
class X_Depot_Container ;



class X_ScheduleSet_Base
  : public virtual RS_Object
{
public :
  explicit X_ScheduleSet_Base ( TiXmlNode const * const & x )
    : RS_Object () {}
  explicit X_ScheduleSet_Base ( string const & _n_in )
    : RS_Object ( _n_in ) {}
  explicit X_ScheduleSet_Base ( X_ScheduleSet_Base const & x )
    : RS_Object ( x ) {}
  X_ScheduleSet_Base & operator= ( X_ScheduleSet_Base const & x )
  { if ( & x == this ) return * this ;
    RS_Object :: operator= ( x ) ; return * this ; }
  virtual ~X_ScheduleSet_Base () {}

  virtual X_Train_Container :: iterator firstTrain () = 0 ;
  virtual X_Train_Container :: iterator lastTrain () = 0 ;
  virtual X_Train_Container :: const_iterator firstTrain () const = 0 ;
  virtual X_Train_Container :: const_iterator lastTrain () const = 0 ;

  virtual void setTimes ( X_Station_Container * ) = 0 ;
  virtual bool isCyclic () const = 0 ;
  virtual double CycleTime () const = 0 ;
  virtual void setCyclicVars () = 0 ;
  virtual X_ScheduleSet_Base * Original () = 0 ;
  virtual void setOriginal ( X_ScheduleSet_Base * ) = 0 ;
  virtual X_ScheduleSet_Base * Superseding () = 0 ;
  virtual void setSuperseding ( X_ScheduleSet_Base * ) = 0 ;

  virtual X_PlatformBranch * getPlatformBranch
  ( X_SpaceTimeNode const * , X_SpaceTimeNode const * ) = 0 ;
  virtual X_PlatformBranch const * getPlatformBranch
  ( X_SpaceTimeNode const * , X_SpaceTimeNode const * ) const = 0 ;
  virtual vector < X_TrainSet_Base const * > const * getTrainSets
  ( X_SpaceTimeNode const * , X_SpaceTimeNode const * ) const = 0 ;
  virtual X_TrainBranch * getTrainBranch
  ( X_SpaceTimeNode const * , X_SpaceTimeNode const * ,
    X_TrainSet_Base const * ) = 0 ;
  virtual X_TrainBranch const * getTrainBranch
  ( X_SpaceTimeNode const * , X_SpaceTimeNode const * ,
    X_TrainSet_Base const * ) const = 0 ;
  virtual X_Branch_Base * getMiscellaneousBranch
  ( X_SpaceTimeNode const * , X_SpaceTimeNode const * ) = 0 ;
  virtual X_Branch_Base const * getMiscellaneousBranch
  ( X_SpaceTimeNode const * , X_SpaceTimeNode const * ) const = 0 ;

  virtual X_PlatformBranch * getOutgoingPlatformBranch
  ( X_SpaceTimeNode const * ) = 0 ;
  virtual X_PlatformBranch const * getOutgoingPlatformBranch
  ( X_SpaceTimeNode const * ) const = 0 ;
  virtual X_PlatformBranch * getIncomingPlatformBranch
  ( X_SpaceTimeNode const * ) = 0 ;
  virtual X_PlatformBranch const * getIncomingPlatformBranch
  ( X_SpaceTimeNode const * ) const = 0 ;

  virtual map < X_STB_Key , X_PlatformBranch * > :: iterator
  firstPlatformBranch () = 0 ;
  virtual map < X_STB_Key , X_PlatformBranch * > :: iterator
  lastPlatformBranch () = 0 ;
  virtual map < X_STB_Key , X_PlatformBranch * > :: const_iterator
  firstPlatformBranch () const = 0 ;
  virtual map < X_STB_Key , X_PlatformBranch * > :: const_iterator
  lastPlatformBranch () const = 0 ;

  virtual map < X_STB_Key , X_Branch_Base * > :: iterator
  firstTransferBranch () = 0 ;
  virtual map < X_STB_Key , X_Branch_Base * > :: iterator
  lastTransferBranch () = 0 ;
  virtual map < X_STB_Key , X_Branch_Base * > :: const_iterator
  firstTransferBranch () const = 0 ;
  virtual map < X_STB_Key , X_Branch_Base * > :: const_iterator
  lastTransferBranch () const = 0 ;

  virtual map < X_STB_Key , X_Branch_Base * > :: iterator
  firstMiscBranch () = 0 ;
  virtual map < X_STB_Key , X_Branch_Base * > :: iterator
  lastMiscBranch () = 0 ;
  virtual map < X_STB_Key , X_Branch_Base * > :: const_iterator
  firstMiscBranch () const = 0 ;
  virtual map < X_STB_Key , X_Branch_Base * > :: const_iterator
  lastMiscBranch () const = 0 ;

  virtual map < X_STB_Key ,
                map < X_TrainSet_Base const * , X_TrainBranch * > >
  :: iterator firstTrainBranches () = 0 ;
  virtual map < X_STB_Key ,
                map < X_TrainSet_Base const * , X_TrainBranch * > >
  :: iterator lastTrainBranches () = 0 ;
  virtual map < X_STB_Key ,
                map < X_TrainSet_Base const * , X_TrainBranch * > >
  :: const_iterator firstTrainBranches () const = 0 ;
  virtual map < X_STB_Key ,
                map < X_TrainSet_Base const * , X_TrainBranch * > >
  :: const_iterator lastTrainBranches () const = 0 ;

  virtual set < X_TrainSet_Base const * > const * getTrainSetsForNode
  ( X_SpaceTimeNode const * ) const = 0 ;

  virtual void setPlatformBranches ( X_Station_Container * ) = 0 ;
  virtual void setTrainBranches ( X_Depot_Container const * ) = 0 ;
  virtual void setMiscBranches ( X_Station_Container * ) = 0 ;

  virtual void setTrainSetForNode ( X_SpaceTimeNode const * ,
                                    X_TrainSet_Base const * ) = 0 ;

  virtual void printResults () = 0 ;
} ;



typedef Expandable < X_ScheduleSet_Base , TiXmlNode const * > SchS_Expander ;



class X_ScheduleSet
  : public X_ScheduleSet_Base ,
    public SchS_Expander
{
  // _trnc: Container of trains.
  RS_ObjectContainer * _trnc ;

  // is_cyclic: true if this is a "cyclic" schedule.
  // _c_time: Cycle time.
  bool is_cyclic ;
  double _c_time ;

  // _s_orig: original schedule to be superseded by this object. Null if
  // this is the original.
  X_ScheduleSet_Base * _s_orig ;

  // _s_next: schedule set that will supersede this object. Null if
  // none exists.
  X_ScheduleSet_Base * _s_next ;

  // -----
  // Containers for generated space time branches
  // -----
  map < X_STB_Key , X_PlatformBranch * > _m_pb ;
  map < X_SpaceTimeNode const * , X_PlatformBranch * > _m_pb_orig ;
  map < X_SpaceTimeNode const * , X_PlatformBranch * > _m_pb_dest ;
  map < X_STB_Key , vector < X_TrainSet_Base const * > > _m_tset ;
  map < X_STB_Key ,
        map < X_TrainSet_Base const * , X_TrainBranch * > > _m_tb ;
  map < X_STB_Key , X_Branch_Base * > _m_trf ;
  map < X_SpaceTimeNode const * , set < X_TrainSet_Base const * > > _m_trs ;
  map < X_STB_Key , X_Branch_Base * > _m_misc ;
//   vector < X_SpaceTimeBranch * > _stb_st ;

  // These must be prohibited...
  explicit X_ScheduleSet ( X_ScheduleSet const & x ) ;
  X_ScheduleSet & operator= ( X_ScheduleSet const & x ) ;

public :
  explicit X_ScheduleSet ( TiXmlNode const * const & ) ;
  explicit X_ScheduleSet ( string const & ) ;

  RS_ObjectContainer * getContainer ( string const & ) ;
  RS_ObjectContainer const * getContainer ( string const & ) const ;

  bool acceptInitObject ( string const & _sx , RS_Object * _ox ) ;

  // Virtual functions returning data.
  X_Train_Container :: iterator firstTrain ()
  { return _trnc -> begin () ; }
  X_Train_Container :: iterator lastTrain ()
  { return _trnc -> end () ; }
  X_Train_Container :: const_iterator firstTrain () const
  { return _trnc -> begin () ; }
  X_Train_Container :: const_iterator lastTrain () const
  { return _trnc -> end () ; }

  void setTimes ( X_Station_Container * ) ;
  bool isCyclic () const { return is_cyclic ; }
  double CycleTime () const { return _c_time ; }
  void setCyclicVars () { is_cyclic = RS_Object :: IS_CYCLIC ;
    _c_time = RS_Object :: CYCLE_TIME ; }
  X_ScheduleSet_Base * Original () { return _s_orig ; }
  void setOriginal ( X_ScheduleSet_Base * x ) { _s_orig = x ; }
  X_ScheduleSet_Base * Superseding () { return _s_next ; }
  void setSuperseding ( X_ScheduleSet_Base * x ) { _s_next = x ; }

  X_PlatformBranch * getPlatformBranch
  ( X_SpaceTimeNode const * , X_SpaceTimeNode const * ) ;
  X_PlatformBranch const * getPlatformBranch
  ( X_SpaceTimeNode const * , X_SpaceTimeNode const * ) const ;
  vector < X_TrainSet_Base const * > const * getTrainSets
  ( X_SpaceTimeNode const * , X_SpaceTimeNode const * ) const ;
  X_TrainBranch * getTrainBranch
  ( X_SpaceTimeNode const * , X_SpaceTimeNode const * ,
    X_TrainSet_Base const * ) ;
  X_TrainBranch const * getTrainBranch
  ( X_SpaceTimeNode const * , X_SpaceTimeNode const * ,
    X_TrainSet_Base const * ) const ;
  X_Branch_Base * getMiscellaneousBranch
  ( X_SpaceTimeNode const * , X_SpaceTimeNode const * ) ;
  X_Branch_Base const * getMiscellaneousBranch
  ( X_SpaceTimeNode const * , X_SpaceTimeNode const * ) const ;

  X_PlatformBranch * getOutgoingPlatformBranch
  ( X_SpaceTimeNode const * ) ;
  X_PlatformBranch const * getOutgoingPlatformBranch
  ( X_SpaceTimeNode const * ) const ;
  X_PlatformBranch * getIncomingPlatformBranch
  ( X_SpaceTimeNode const * ) ;
  X_PlatformBranch const * getIncomingPlatformBranch
  ( X_SpaceTimeNode const * ) const ;

  map < X_STB_Key , X_PlatformBranch * > :: iterator firstPlatformBranch ()
  { return _m_pb . begin () ; }
  map < X_STB_Key , X_PlatformBranch * > :: iterator lastPlatformBranch ()
  { return _m_pb . end () ; }
  map < X_STB_Key , X_PlatformBranch * > :: const_iterator firstPlatformBranch
  () const { return _m_pb . begin () ; }
  map < X_STB_Key , X_PlatformBranch * > :: const_iterator lastPlatformBranch
  () const { return _m_pb . end () ; }

  map < X_STB_Key , X_Branch_Base * > :: iterator firstTransferBranch ()
  { return _m_trf . begin () ; }
  map < X_STB_Key , X_Branch_Base * > :: iterator lastTransferBranch ()
  { return _m_trf . end () ; }
  map < X_STB_Key , X_Branch_Base * > :: const_iterator firstTransferBranch ()
    const { return _m_trf . begin () ; }
  map < X_STB_Key , X_Branch_Base * > :: const_iterator lastTransferBranch ()
    const { return _m_trf . end () ; }

  map < X_STB_Key , X_Branch_Base * > :: iterator firstMiscBranch ()
  { return _m_misc . begin () ; }
  map < X_STB_Key , X_Branch_Base * > :: iterator lastMiscBranch ()
  { return _m_misc . end () ; }
  map < X_STB_Key , X_Branch_Base * > :: const_iterator firstMiscBranch ()
    const { return _m_misc . begin () ; }
  map < X_STB_Key , X_Branch_Base * > :: const_iterator lastMiscBranch ()
    const { return _m_misc . end () ; }

  map < X_STB_Key ,
        map < X_TrainSet_Base const * , X_TrainBranch * > >
  :: iterator firstTrainBranches ()
  { return _m_tb . begin () ; }
  map < X_STB_Key ,
        map < X_TrainSet_Base const * , X_TrainBranch * > >
  :: iterator lastTrainBranches ()
  { return _m_tb . end () ; }
  map < X_STB_Key ,
        map < X_TrainSet_Base const * , X_TrainBranch * > >
  :: const_iterator firstTrainBranches () const
  { return _m_tb . begin () ; }
  map < X_STB_Key ,
        map < X_TrainSet_Base const * , X_TrainBranch * > >
  :: const_iterator lastTrainBranches () const
  { return _m_tb . end () ; }

  set < X_TrainSet_Base const * > const * getTrainSetsForNode
  ( X_SpaceTimeNode const * ) const ;

  void setPlatformBranches ( X_Station_Container * ) ;
  void setTrainBranches ( X_Depot_Container const * ) ;
  void setMiscBranches ( X_Station_Container * ) ;

  void setTrainSetForNode ( X_SpaceTimeNode const * ,
                            X_TrainSet_Base const * ) ;

  void printResults () ;
} ;


class X_ScheduleSet_Creator
  : public SchS_Expander :: ObjectCreator
{
public :
  X_ScheduleSet_Creator ()
  { add_to_creators_list ( "x_schedule_set" ) ; }
  X_ScheduleSet_Creator ( char const * x )
    : SchS_Expander :: ObjectCreator () { add_to_creators_list ( x ) ; }
  X_ScheduleSet_Base * create () const
  { cerr << "Error: X_ScheduleSet... default constructor prohibited"
         << endl ; exit ( 1 ) ; }
  X_ScheduleSet_Base * create ( TiXmlNode const * const & x ) const
  { return new X_ScheduleSet ( x ) ; }
} ;



class X_ScheduleSet_Container
  : public RS_ObjectContainer
{
  X_ScheduleSet_Base * _first ;

public :
  X_ScheduleSet_Container () : RS_ObjectContainer () , _first ( false ) {}
  bool create ( TiXmlNode const * & , vector < string > const & ,
                vector < RS_Object * > const & ) ;
  X_ScheduleSet_Base * firstScheduleSet () { return _first ; }
  X_ScheduleSet_Base const * firstScheduleSet () const { return _first ; }
  
  ~X_ScheduleSet_Container () {}
} ;




#endif // X_ScheduleSet_HH
