// -*- C++ -*-

// ダイクストラ法のデモ

#include <stdexcept>
#include "X_SpaceTimeBranch.hh"
#include "X_Station.hh"
#include "X_Train.hh"
#include "X_TrainSet.hh"
#include "X_Weight.hh"




X_Branch_Base :: X_Branch_Base
( X_SpaceTimeNode const * _bf_in ,
  X_SpaceTimeNode const * _af_in ,
  X_Weight_Base * _w_in )
  : X_STB_Key ( _bf_in , _af_in ) ,
    _weight ( _w_in ) ,
    _pas () ,
    _pas_v ()
{
  string _str = "BRANCH::" ;
  if ( ! _bf_in )
  {
    _str += "NULL--" ;
  }
  else
  {
    _str += _bf_in -> getName () ;
    _str += "--" ;
  }
  if ( ! _af_in )
  {
    _str += "NULL" ;
  }
  else
  {
    _str += _af_in -> getName () ;
  }
  RS_Object :: setName ( _str ) ;
}



X_Branch_Base :: X_Branch_Base
( X_Branch_Base const & x )
  : X_STB_Key ( x ) ,
    _weight ( 0 ) ,
    _pas ( x . _pas ) ,
    _pas_v ( x . _pas_v )
{
  if ( x . _weight )
  {
    _weight = x . _weight -> MyClone () ;
  }
}


  
X_Branch_Base &
X_Branch_Base :: operator=
( X_Branch_Base const & x )
{
  if ( this == & x )
  {
    return * this ;
  }
  X_STB_Key :: operator= ( x ) ;
  delete _weight ;
  if ( x . _weight )
  {
    _weight = x . _weight -> MyClone () ;
  }
  else
  {
    _weight = 0 ;
  }
  return * this ;
}



// -----
// Manipulating "remaining trainsets" on X_PlatformBranch
// -----
void
X_PlatformBranch :: getRemainingTrainSet
( unsigned int i ,
  X_TrainSet_Base const * & x ,
  string & y ,
  X_Train_Base const * & z ,
  X_SpaceTimeNode const * & w )
  const
{
  try
  {
    x = _trainsets . at ( i ) ;
    y = _actions . at ( i ) ;
    z = _trains . at ( i ) ;
    w = _tr_nodes . at ( i ) ;
  }
  catch ( std :: out_of_range )
  {
    cerr << "Error: out of range at X_PlatformBranch::getRemainingTrainSet()"
         << endl ;
    exit ( 1 ) ;
  }
}



void
X_PlatformBranch :: removeTrainSet
( unsigned int i )
{
  if ( i < 0 || i >= getNumberOfRemainingTrainSets () )
  {
    cerr << "Error: out of range at X_PlatformBranch::removeTrainSet()"
         << endl ;
    exit ( 1 ) ;
  }
  vector < X_TrainSet_Base const * > :: iterator
    ii = _trainsets . begin () + i ;
  _trainsets . erase ( ii ) ;
  vector < string > :: iterator jj = _actions . begin () + i ;
  _actions . erase ( jj ) ;
  vector < X_Train_Base const * > :: iterator kk = _trains . begin () + i ;
  _trains . erase ( kk ) ;
  vector < X_SpaceTimeNode const * > :: iterator
    ll = _tr_nodes . begin () + i ;
  _tr_nodes . erase ( ll ) ;
}




X_TrainBranch :: X_TrainBranch
( X_SpaceTimeNode const * _bf_in ,
  X_SpaceTimeNode const * _af_in ,
  X_Weight_Base * _w_in ,
  X_Train_Base const * _trn_in ,
  X_TrainSet_Base const * _trs_in )
  : X_Branch_Base ( _bf_in , _af_in , _w_in ) ,
    _trn ( _trn_in ) ,
    _trs ( _trs_in )
{
  string _str = getName () ;
  _str += "::" ;
  if ( _trs_in )
  {
    _str += _trs_in -> getName () ;
  }
  else
  {
    _str += "TRS_NULL" ;
  }
  RS_Object :: setName ( _str ) ;
}
