// -*- C++ -*-

// ダイクストラ法のデモ

#ifndef X_SpaceTimeBranch_HH
#define X_SpaceTimeBranch_HH

#include "X_SpaceTimeNode.hh"

#include <iostream>
#include <set>
#include <vector>
#include <utility>

using std :: ostream ;
using std :: cerr ;
using std :: endl ;
using std :: pair ;
using std :: make_pair ;
using std :: vector ;
using std :: set ;



class X_Weight_Base ;
class X_Passenger_Token ;
class X_PassengerAppearance_Base ;
class X_Train_Base ;
class X_TrainSet_Base ;
class X_Station_Base ;




class X_STB_Key
  : public virtual RS_Object
{
  X_SpaceTimeNode const * _f ;
  X_SpaceTimeNode const * _t ;

public :

  X_STB_Key () : RS_Object () , _f ( 0 ) , _t ( 0 ) {}
  
  X_STB_Key ( X_SpaceTimeNode const * x_f , X_SpaceTimeNode const * x_t )
    : RS_Object () , _f ( x_f ) , _t ( x_t )
  { if ( ! _f || ! _t ) {
      cerr << "Error: null pointer used to initialise X_SpaceTimeBranch"
           << endl ; exit ( 1 ) ; } }
  X_STB_Key ( X_STB_Key const & x )
    : RS_Object ( x ) , _f ( x . _f ) , _t ( x . _t ) {}
  X_STB_Key & operator= ( X_STB_Key const & x )
  { if ( & x == this ) return * this ; RS_Object :: operator= ( x ) ;
    _f = x . _f ; _t = x . _t ; return * this ; }

  X_SpaceTimeNode const * getStartSTN () const { return _f ; }
  X_SpaceTimeNode const * getEndSTN () const { return _t ; }

//   friend ostream & operator<< ( ostream & , X_Weight_Base const & ) ;
//   virtual ostream & writeOut ( ostream & ) const = 0 ;

  bool operator< ( X_STB_Key const & x ) const
  { return ( _f < x . _f )
      || ( ( _f == x . _f ) && ( _t < x . _t ) ) ; }
  bool operator== ( X_STB_Key const & x ) const
  { return _f == x . _f && _t == x . _t ; }

  bool operator!= ( X_STB_Key const & x ) const
  { return ! operator== ( x ) ; }
} ;
  


typedef set < X_Passenger_Token * > EmergingPassengerSet ;
typedef map < X_Station_Base const * , EmergingPassengerSet >
EmergingPassengers ;
typedef map < X_PassengerAppearance_Base * , EmergingPassengers >
EmergingPassengersMap ;



class X_Branch_Base
  : public X_STB_Key
{

protected :

  X_Weight_Base * _weight ;
  vector < pair < X_Passenger_Token const * , double > > _pas ;
  EmergingPassengersMap _pas_v ;

public :
  explicit X_Branch_Base
  ( X_SpaceTimeNode const * , X_SpaceTimeNode const * , X_Weight_Base * ) ;
  X_Branch_Base ( X_Branch_Base const & ) ;
  X_Branch_Base & operator= ( X_Branch_Base const & ) ;

//   friend ostream & operator<< ( ostream & , X_Weight_Base const & ) ;
//   virtual ostream & writeOut ( ostream & ) const = 0 ;

  X_Weight_Base const * Weight () const { return _weight ; }
  X_Weight_Base * Weight () { return _weight ; }

  void setWeight ( X_Weight_Base * x ) { _weight = x ; }

  void addPassenger ( X_Passenger_Token const * x , double y = 1.0 ) 
  { _pas . push_back ( make_pair ( x , y ) ) ; }

  vector < pair < X_Passenger_Token const * , double > > :: const_iterator
  firstPassenger () const { return _pas . begin () ; }
  vector < pair < X_Passenger_Token const * , double > > :: const_iterator
  lastPassenger () const { return _pas . end () ; }

  EmergingPassengersMap :: const_iterator
  firstEmergingPassengers () const { return _pas_v . begin () ; }
  EmergingPassengersMap :: const_iterator
  lastEmergingPassengers () const { return _pas_v . end () ; }

  void setEmergingPassengers ( X_PassengerAppearance_Base * x ,
                              EmergingPassengers const & y )
  { if ( _pas_v . find ( x ) != _pas_v . end () )
    { cerr << "Error: EmergingPassengers already set" << endl ; exit ( 1 ) ; }
    _pas_v [ x ] = y ; }

  EmergingPassengerSet const * getEmergingPassengers
  ( X_PassengerAppearance_Base * x ,
    X_Station_Base const * y )
    const
  {
    EmergingPassengersMap :: const_iterator ii = _pas_v . find ( x ) ;
    if ( ii == _pas_v . end () )
    {
      return 0 ;
    }
    EmergingPassengers :: const_iterator jj = ii -> second . find ( y ) ;
    if ( jj == ii -> second . end () )
    {
      return 0 ;
    }
    return & ( jj -> second ) ;
  }

  virtual X_TrainSet_Base const * MyTrainSet () const { return 0 ; }
  virtual X_Train_Base const * MyTrain () const { return 0 ; }

} ;
  


class X_PlatformBranch
  : public X_Branch_Base
{
  X_Platform_Base const * _plat ;
  vector < X_TrainSet_Base const * > _trainsets ;
  vector < string > _actions ;
  vector < X_Train_Base const * > _trains ;
  vector < X_SpaceTimeNode const * > _tr_nodes ;

public :
  explicit X_PlatformBranch
  ( X_SpaceTimeNode const * _bf_in , X_SpaceTimeNode const * _af_in ,
    X_Weight_Base * _w_in , X_Platform_Base const * _p_in )
    : X_Branch_Base ( _bf_in , _af_in , _w_in ) , _plat ( _p_in ) ,
      _trainsets () , _actions () , _trains () , _tr_nodes () {}
  X_PlatformBranch ( X_PlatformBranch const & x )
    : X_Branch_Base ( x ) , _plat ( x . _plat ) ,
      _trainsets () , _actions () , _trains () , _tr_nodes () {}
  X_PlatformBranch & operator= ( X_PlatformBranch const & x )
  { if ( this == & x ) return * this ; X_Branch_Base :: operator= ( x ) ;
    _plat = x . _plat ; _trainsets = x . _trainsets ;
    _actions = x . _actions ; _trains = x . _trains ;
    _tr_nodes = x . _tr_nodes ; return * this ; }

  void setTrainSet ( X_TrainSet_Base const * x , string const & y ,
                     X_Train_Base const * z , X_SpaceTimeNode const * w )
  { _trainsets . push_back ( x ) ; _actions . push_back ( y ) ;
    _trains . push_back ( z ) ; _tr_nodes . push_back ( w ) ; }

  void getRemainingTrainSet
  ( unsigned int , X_TrainSet_Base const * & , string & ,
    X_Train_Base const * & , X_SpaceTimeNode const * & ) const ;

  unsigned int getNumberOfRemainingTrainSets () const
  { return _trainsets . size () ; }
  
  void removeTrainSet ( unsigned int ) ;

} ;




class X_TrainBranch
  : public X_Branch_Base
{
  X_Train_Base const * _trn ;
  X_TrainSet_Base const * _trs ;

public :
  explicit X_TrainBranch
  ( X_SpaceTimeNode const * , X_SpaceTimeNode const * , X_Weight_Base * ,
    X_Train_Base const * , X_TrainSet_Base const * ) ;
  X_TrainBranch ( X_TrainBranch const & x )
    : X_Branch_Base ( x ) , _trn ( x . _trn ) , _trs ( x . _trs ) {}
  X_TrainBranch & operator= ( X_TrainBranch const & x )
  { if ( this == & x ) return * this ;
    X_Branch_Base :: operator= ( x ) ; _trn = x . _trn ;
    _trs = x . _trs ; return * this ; }

  X_Train_Base const * MyTrain () const { return _trn ; }
  X_TrainSet_Base const * MyTrainSet () const { return _trs ; }
} ;



#endif   // X_SpaceTimeBranch_HH
