// -*- C++ -*-

// ダイクストラ法のデモ

#include "RS_Object.hh"


#ifndef X_SpaceTimeNode_HH
#define X_SpaceTimeNode_HH


const bool DEP = true ;
const bool ARR = false ;



class X_Train_Base ;
class X_Platform_Base ;
class X_Weight_Base ;




class X_SpaceTimeNode
  : public virtual RS_Object
{
public :

protected :
  bool _dep ;                         // true if departing
  double _t_ ;                        // Time (of arrival or departure)
  int _cd_ ;                          // Cycle delay number
  bool _is_trn ;                      // True if on the train side
  X_Train_Base const * _trn ;         // Train
  X_Platform_Base const * _plat ;     // Platform
  X_Weight_Base * _weight ;           // Weight

public :
  explicit X_SpaceTimeNode
  ( bool _dep_in , double _t_in , X_Train_Base * _trn_in , bool _sw = true )
    : RS_Object () , _dep ( _dep_in ) , _t_ ( _t_in ) , _cd_ ( 0 ) ,
      _is_trn ( _sw ) , _trn ( _trn_in ) , _plat ( 0 ) , _weight ( 0 )
  { if ( IS_CYCLIC ) 
    { while ( _t_ < 0 ) { _t_ += CYCLE_TIME ; -- _cd_ ; }
      while ( _t_ >= CYCLE_TIME ) { _t_ -= CYCLE_TIME ; ++ _cd_ ; } }
    setName () ; }

  explicit X_SpaceTimeNode ( X_Platform_Base const * p , bool d )
    : RS_Object () , _dep ( d ) , _t_ ( 0 ) , _cd_ ( 0 ) , _is_trn ( false ) ,
      _trn ( 0 ) , _plat ( p ) , _weight ( 0 ) { setName () ; }

  ~X_SpaceTimeNode () ;

  X_SpaceTimeNode ( X_SpaceTimeNode const & , bool ) ;
  X_SpaceTimeNode ( X_SpaceTimeNode const & ) ;
  X_SpaceTimeNode & operator= ( X_SpaceTimeNode const & ) ;

  bool isDeparting () const { return _dep ; }
  double getTime () const { return _t_ ; }
  int getCycleDelay () const { return _cd_ ; }
  X_Weight_Base const * Weight () const { return _weight ; }

  void setPlatform ( X_Platform_Base const * x ) { _plat = x ; setName () ; }
  X_Platform_Base const * getPlatform () const { return _plat ; }
  X_Train_Base const * getTrain () const { return _trn ; }
  bool isTrain () const { return _is_trn ; }

  void setName () ;
  
} ;




class LessSTN
{
public :
  bool operator() ( X_SpaceTimeNode const * _l ,
                    X_SpaceTimeNode const * _r ) const
  { return _l && _r && ( _l -> getTime () < _r -> getTime () ) ; }
} ;



#endif  // X_SpaceTimeNode_HH
