// -*- C++ -*-

// ダイクストラ法のデモ

#include "X_Station.hh"


X_Station :: X_Station
( string const & _n_in )
  : X_Station_Base ( _n_in ) ,
    _demand () ,
    _platc ( 0 ) ,
    _itimec ( 0 ) ,
    _plat_in ( 0 ) ,
    _node_in ( 0 ) ,
    _plat_out ( 0 ) ,
    _node_out ( 0 ) ,
    _plat_processed ( false ) ,
    _ipt_processed ( false )
{
  _platc = new X_Platform_Container () ;
  _itimec = new X_IntraTime_Container () ;
}



X_Station :: X_Station
( TiXmlNode const * const & curnode )
  : X_Station_Base ( curnode ) ,
    _demand () ,
    _platc ( 0 ) ,
    _itimec ( 0 ) ,
    _plat_in ( 0 ) ,
    _node_in ( 0 ) ,
    _plat_out ( 0 ) ,
    _node_out ( 0 ) ,
    _plat_processed ( false ) ,
    _ipt_processed ( false )
{
  if ( ! curnode )
  {
    cerr << "Error: node not set in X_Station constructor." << endl ;
    exit ( 1 ) ;
  }
  if ( curnode -> ValueStr () != "data" )
  {
    cerr << "Error: <" << curnode -> ValueStr ()
         << "> tag not allowed in X_Station constructor." << endl ;
    exit ( 1 ) ;
  }

  const TiXmlAttribute * attr = curnode -> ToElement () -> FirstAttribute () ;
  bool sw_name = false ;
  while ( attr )
  {
    if ( string ( attr -> Name () ) == "name" )
    {
      // Attribute "name"
      if ( sw_name )
      {
        // Strange ... two attributes of same name?
        // 同じ属性が2つある？... エラー
        cerr << "Error: two \"name\" attributes?" << endl ;
        exit ( 2 ) ;
      }
      setName ( attr -> ValueStr () ) ;
      sw_name = true ;
    }
    attr = attr -> Next () ;
  }
  if ( ! sw_name )
  {
    cerr << "Error: <data> tag without attribute \"name\" in initialising "
         << "an X_Station object." << endl ;
    exit ( 1 ) ;
  }

  // ----
  // Setting a platform data container and an intratime data container.
  // ----
  _platc = new X_Platform_Container () ;
  _itimec = new X_IntraTime_Container () ;

  // ----
  // Reading subelements.
  // ----
  for ( TiXmlNode const * _c_in = curnode -> FirstChild () ;
        _c_in ; _c_in = _c_in -> NextSibling () )
  {
    // -----
    // Ignore non-Element node.
    // エレメント以外は無視。
    // -----
    if ( _c_in -> Type () != TiXmlNode :: TINYXML_ELEMENT )
    {
      continue ;
    }

    // ----
    // If _c_in is of tag <ss_soc_feedforward_data>, process it here.
    // ----
    string _v_cin = _c_in -> ValueStr () ;
    if ( _v_cin == "platforms" )
    {
      processPlatformsTag ( _c_in ) ;
    }
    else if ( _v_cin == "intratimes" )
    {
      processIntraTimesTag ( _c_in ) ;
    }
    else
    {
      cerr << "Error: tag \"" << _v_cin << "\" not allowed for the "
           << "initialisation of an X_Station object" << endl ;
      exit ( 1 ) ;
    }
  }
}




// -----
// Processing tag <platforms>.
// -----
void
X_Station :: processPlatformsTag
( TiXmlNode const * curnode )
{
  if ( _plat_processed )
  {
    cerr << "Error: duplicate <platforms> tag found in the initialisation of "
         << "station " << getName () << endl ;
    exit ( 1 ) ;
  }
  _plat_processed = true ;
  
  // ----
  // Reading subelements.
  // ----
  for ( TiXmlNode const * _c_in = curnode -> FirstChild () ;
        _c_in ; _c_in = _c_in -> NextSibling () )
  {
    // -----
    // Ignore non-Element node.
    // エレメント以外は無視。
    // -----
    if ( _c_in -> Type () != TiXmlNode :: TINYXML_ELEMENT )
    {
      continue ;
    }

    // ----
    // Abort if _c_in is not of tag <platform>.
    // ----
    string _v_cin = _c_in -> ValueStr () ;
    if ( _v_cin != "platform" )
    {
      cerr << "Error: tag <" << _v_cin << "> not allowed under tag "
           << "<platforms> in initialising station " << getName () << endl ;
      exit ( 1 ) ;
    }

    // ----
    // Create a platform object.
    // ----
    vector < string > _x_a ;
    _x_a . clear () ;
    vector < RS_Object * > _x_b ;
    _x_b . clear () ;
    _platc -> create ( _c_in , _x_a , _x_b ) ;
  }

  // ----
  // Find entrance and exit.
  // ----
  for ( X_Platform_Container :: const_iterator ii = _platc -> begin () ;
        ii != _platc -> end () ; ++ ii )
  {
    X_Platform_Base *
      _pxx = dynamic_cast < X_Platform_Base * > ( ii -> second ) ;
    if ( ! _pxx )
    {
      cerr << "Error: in initialisation of station object \""
           << getName () << "\": failed to get pointer to Platform" << endl ;
      exit ( 1 ) ;
    }
    _pxx -> setMyStation ( this ) ;
    if ( _pxx -> isEntrance () )
    {
      _plat_in = _pxx ;
    }
    if ( _pxx -> isExit () )
    {
      _plat_out = _pxx ;
    }
  }
  if ( ( ! _plat_in ) || ( ! _plat_out ) )
  {
    cerr << "Error: in initialisation of station object \""
         << getName () << "\": entrance or exit not set" << endl ;
    exit ( 1 ) ;
  }

  // ----
  // Set entrance and exit nodes.
  // ----
  string _name_io ;
  _name_io = getName () ;
  _name_io += "::Entrance" ;
  _node_in = new X_SpaceTimeNode ( _plat_in , false ) ;
  _node_in -> RS_Object :: setName ( _name_io ) ;
  _name_io = getName () ;
  _name_io += "::Exit" ;
  _node_out = new X_SpaceTimeNode ( _plat_out , false ) ;
  _node_out -> RS_Object :: setName ( _name_io ) ;
}




// -----
// Processing tag <intratimes>.
// -----
void
X_Station :: processIntraTimesTag
( TiXmlNode const * curnode )
{
  if ( _ipt_processed )
  {
    cerr << "Error: duplicate <intratimes> tag found in the "
         << "initialisation of station " << getName () << endl ;
    exit ( 1 ) ;
  }
  if ( ! _plat_processed )
  {
    cerr << "Error: <intratimes> tag must follow <platforms> tag in the "
         << "initialisation of station " << getName () << endl ;
    exit ( 1 ) ;
  }
  _ipt_processed = true ;
  
  // ----
  // Reading subelements.
  // ----
  for ( TiXmlNode const * _c_in = curnode -> FirstChild () ;
        _c_in ; _c_in = _c_in -> NextSibling () )
  {
    // -----
    // Ignore non-Element node.
    // エレメント以外は無視。
    // -----
    if ( _c_in -> Type () != TiXmlNode :: TINYXML_ELEMENT )
    {
      continue ;
    }

    // ----
    // Abort if _c_in is not of tag <intratime>.
    // ----
    string _v_cin = _c_in -> ValueStr () ;
    if ( _v_cin != "intratime" )
    {
      cerr << "Error: tag <" << _v_cin << "> not allowed under tag "
           << "<intratimes> in initialising station " << getName () << endl ;
      exit ( 1 ) ;
    }

    // -----
    // (1) Get all attributes of this "intratime" tag.
    // -----
    map < string , string > _a_m ;
    getAttributeMap ( _c_in , _a_m ) ;
    string _ein_msg = "in initialisation of an X_IntraTime instance" ;

    // -----
    // (2) Find attribute "class" in the map.
    // -----
    map < string , string > :: iterator ii = findAttributeInMap
      ( _a_m , "class" , _ein_msg , true ) ;
    string _class_str = ii -> second ;
    _a_m . erase ( ii ) ;

    // -----
    // (3) Find attribute "from" in the map. Required.
    // -----
    ii = findAttributeInMap ( _a_m , "from" , _ein_msg , true ) ;
    string _from_str = ii -> second ;
    _a_m . erase ( ii ) ;

    // -----
    // (4) Find attribute "to" in the map. Required.
    // -----
    ii = findAttributeInMap ( _a_m , "to" , _ein_msg , true ) ;
    string _to_str = ii -> second ;
    _a_m . erase ( ii ) ;

    // -----
    // (5) Find attribute "time" in the map. Required.
    // -----
    ii = findAttributeInMap ( _a_m , "time" , _ein_msg , true ) ;
    string _time_str = ii -> second ;
    _a_m . erase ( ii ) ;

    // -----
    // (6) Check if attribute map is exhausted.
    // -----
    if ( _a_m . size () )
    {
      ii = _a_m . begin () ;
      cerr << "Error: " << _ein_msg << ": incorrect attribute \""
           << ii -> first << "\" found when initialising an "
           << "intratime instance" << endl ;
      exit ( 1 ) ;
    }

    // ----
    // Create an intratime object.
    // ----
    vector < string > _x_a ;
    _x_a . clear () ;
    vector < RS_Object * > _x_b ;
    _x_b . clear () ;
    RS_Object * _from_obj = _platc -> locate ( _from_str ) ;
    _x_a . push_back ( "from" ) ;
    _x_b . push_back ( _from_obj ) ;
    RS_Object * _to_obj = _platc -> locate ( _to_str ) ;
    _x_a . push_back ( "to" ) ;
    _x_b . push_back ( _to_obj ) ;
    string _name_str = dynamic_cast < X_Platform_Base * > ( _from_obj )
      -> getCombinationName () ;
    _name_str += "--" ;
    _name_str += _to_obj -> getName () ;
    TiXmlElement * _data_elem = new TiXmlElement ( "data" ) ;
    _data_elem -> SetAttribute ( "class" , _class_str ) ;
    _data_elem -> SetAttribute ( "name" , _name_str ) ;
    _data_elem -> SetAttribute ( "time" , _time_str ) ;
    TiXmlNode const * _n_data_elem = _data_elem ;
    _itimec -> create ( _n_data_elem , _x_a , _x_b ) ;
    delete _data_elem ;
  }
}




RS_ObjectContainer *
X_Station :: getContainer
( string const & x )
{
  cerr << "Error: No class \"" << x << "\" found in X_Station" << endl ;
  exit ( 1 ) ;
  return 0 ;
}




RS_ObjectContainer const *
X_Station :: getContainer
( string const & x )
  const
{
  cerr << "Error: No class \"" << x << "\" found in X_Station" << endl ;
  exit ( 1 ) ;
  return 0 ;
}




bool
X_Station_Container :: create
( TiXmlNode const * & _n_in ,
  vector < string > const & _s_in ,
  vector < RS_Object * > const & _o_in )
{
  string xxx = getClassString ( _n_in ) ;
  string yyy = getNameString ( _n_in ) ;
  X_Station_Base * x = STA_Expander :: create ( xxx , _n_in ) ;
  ( * this ) [ yyy ] = x ;
  vector < string > :: const_iterator ii = _s_in . begin () ;
  vector < RS_Object * > :: const_iterator jj = _o_in . begin () ;
  while ( ii < _s_in . end () || jj < _o_in . end () )
  {
    if ( x -> acceptInitObject ( * ii , * jj ) )
    {
      ++ ii ;
      ++ jj ;
    }
    else
    {
      cerr << "Error: acceptInitObject failed, type = "
           << * ii << endl ;
      exit ( 1 ) ;
    }
  }
  return true ;
}




X_Station_Base const *
X_Station_Container :: locateStation
( string const & x )
  const
{
  RS_Object const * retv = locate ( x ) ;
  return dynamic_cast < X_Station_Base const * > ( retv ) ;
}




X_Station_Base *
X_Station_Container :: locateStation
( string const & x )
{
  RS_Object * retv = locate ( x ) ;
  return dynamic_cast < X_Station_Base * > ( retv ) ;
}




X_Platform_Base const *
X_Station :: locatePlatform
( string const & x )
  const
{
  RS_Object const * retv = _platc -> locate ( x ) ;
  return dynamic_cast < X_Platform_Base const * > ( retv ) ;
}




X_Platform_Base *
X_Station :: locatePlatform
( string const & x )
{
  RS_Object * retv = _platc -> locate ( x ) ;
  return dynamic_cast < X_Platform_Base * > ( retv ) ;
}
