// -*- C++ -*-

// ダイクストラ法のデモ

#include "X_SpaceTimeNode.hh"
#include "Expandable.hh"
#include "RS_XmlReader.hh"

#ifndef X_Station_HH
#define X_Station_HH


#include <algorithm>
#include <iostream>
#include <vector>
#include <map>
#include <utility>


using std :: cerr ;
using std :: endl ;
using std :: vector ;
using std :: multimap ;
using std :: sort ;
using std :: pair ;




class X_Station_Base ;




class X_Platform_Base
  : public virtual RS_Object
{
public :
  explicit X_Platform_Base ( TiXmlNode const * const & x ) : RS_Object () {}
  explicit X_Platform_Base ( string const & _n_in ) : RS_Object ( _n_in ) {}
  explicit X_Platform_Base ( X_Platform_Base const & x ) : RS_Object ( x ) {}
  X_Platform_Base & operator= ( X_Platform_Base const & x )
  { if ( & x == this ) return * this ;
    RS_Object :: operator= ( x ) ; return * this ; }
  virtual ~X_Platform_Base () {}

  virtual void setMyStation ( X_Station_Base * ) = 0 ;

  virtual X_SpaceTimeNode * setTime ( X_SpaceTimeNode * ) = 0 ;
  virtual X_SpaceTimeNode * setTime ( X_SpaceTimeNode ) = 0 ;
  virtual X_SpaceTimeNode * setTime
  ( X_SpaceTimeNode , X_SpaceTimeNode const * , bool ) = 0 ;

  virtual X_SpaceTimeNode const *
  getConnectedTrainNode ( X_SpaceTimeNode const * x ) const = 0 ;
  virtual X_SpaceTimeNode const *
  getConnectedPlatformNode ( X_SpaceTimeNode const * x ) const = 0 ;
  virtual X_SpaceTimeNode const *
  getOriginatingPlatformNode ( X_SpaceTimeNode const * x ) const = 0 ;
  virtual X_SpaceTimeNode const *
  getDestinationPlatformNode ( X_SpaceTimeNode const * x ) const = 0 ;

  virtual void sortTimes () = 0 ;

  virtual bool isEntrance () const = 0 ;
  virtual bool isExit () const = 0 ;
  virtual string const & getCombinationName () const = 0 ;
  virtual X_Station_Base * MyStation () = 0 ;
  virtual X_Station_Base const * MyStation () const = 0 ;

  virtual vector < X_SpaceTimeNode * > :: iterator firstNode () = 0 ;
  virtual vector < X_SpaceTimeNode * > :: iterator lastNode () = 0 ;
  virtual vector < X_SpaceTimeNode * > :: const_iterator firstNode ()
    const = 0 ;
  virtual vector < X_SpaceTimeNode * > :: const_iterator lastNode ()
    const = 0 ;

} ;



typedef Expandable < X_Platform_Base , TiXmlNode const * > Plat_Expander ;


class X_Platform
  : public X_Platform_Base ,
    public Plat_Expander
{
  // is_entrance: true if this actually is an entrance place of this station.
  // is_exit: true if this actually is an exit place of this station.
  bool is_entrance ;
  bool is_exit ;

  // my_station: station to which this object belongs.
  X_Station_Base * my_station ;

  // Temporary variable for spooling combination name.
  mutable string _combi_name ;

  // Space time nodes of departure and arrival of trains, and the
  // corresponding STNs of trains.
  vector < X_SpaceTimeNode * > _stn ;
  map < X_SpaceTimeNode const * , X_SpaceTimeNode const * > _stn_a ;
  map < X_SpaceTimeNode const * , X_SpaceTimeNode const * > _stn_b ;

  // Space time nodes of departure and arrival of trains at other platforms
  map < X_SpaceTimeNode const * , X_SpaceTimeNode const * > _stn_d ;
  map < X_SpaceTimeNode const * , X_SpaceTimeNode const * > _stn_o ;

  // These must be prohibited...
  explicit X_Platform ( X_Platform const & x ) ;
  X_Platform & operator= ( X_Platform const & x ) ;

public :
  explicit X_Platform ( TiXmlNode const * const & ) ;
  explicit X_Platform ( string const & ) ;
  ~X_Platform () {}

  void sortTimes ()
  { sort ( _stn . begin () , _stn . end () , LessSTN () ) ; }

  bool isEntrance () const { return is_entrance ; }
  bool isExit () const { return is_exit ; }

  string const & getCombinationName () const ;

  // Prohibited
  RS_ObjectContainer * getContainer ( string const & ) ;
  RS_ObjectContainer const * getContainer ( string const & ) const ;

  // bool acceptInitObject ( string const & , RS_Object * ) { return false ; }

  void setMyStation ( X_Station_Base * x ) { my_station = x ; }

  X_SpaceTimeNode * setTime ( X_SpaceTimeNode * x )
  { x -> setPlatform ( this ) ; X_SpaceTimeNode * y = setTime ( * x ) ;
    _stn_a [ x ] = y ; _stn_b [ y ] = x ; return y ; }

  X_SpaceTimeNode * setTime ( X_SpaceTimeNode x )
  { X_SpaceTimeNode * y = new X_SpaceTimeNode ( x , false ) ;
    y -> setPlatform ( this ) ; _stn . push_back ( y ) ; return y ; }

  X_SpaceTimeNode * setTime
  ( X_SpaceTimeNode x , X_SpaceTimeNode const * _nt , bool _dp )
  { X_SpaceTimeNode * y = new X_SpaceTimeNode ( x , false ) ;
    y -> setPlatform ( this ) ; _stn . push_back ( y ) ;
    if ( _dp ) _stn_d [ y ] = _nt ; else _stn_o [ y ] = _nt ; return y ; }

  X_SpaceTimeNode const *
  getConnectedTrainNode ( X_SpaceTimeNode const * x ) const
  { map < X_SpaceTimeNode const * , X_SpaceTimeNode const * >
      :: const_iterator ii = _stn_b . find ( x ) ;
    if ( ii == _stn_b . end () ) { return 0 ; } return ii -> second ; }

  X_SpaceTimeNode const *
  getConnectedPlatformNode ( X_SpaceTimeNode const * x ) const
  { map < X_SpaceTimeNode const * , X_SpaceTimeNode const * >
      :: const_iterator ii = _stn_a . find ( x ) ;
    if ( ii == _stn_a . end () ) { return 0 ; } return ii -> second ; }

  X_SpaceTimeNode const *
  getOriginatingPlatformNode ( X_SpaceTimeNode const * x ) const
  { map < X_SpaceTimeNode const * , X_SpaceTimeNode const * >
      :: const_iterator ii = _stn_o . find ( x ) ;
    if ( ii == _stn_o . end () ) { return 0 ; } return ii -> second ; }

  X_SpaceTimeNode const *
  getDestinationPlatformNode ( X_SpaceTimeNode const * x ) const
  { map < X_SpaceTimeNode const * , X_SpaceTimeNode const * >
      :: const_iterator ii = _stn_d . find ( x ) ;
    if ( ii == _stn_d . end () ) { return 0 ; } return ii -> second ; }

  X_Station_Base * MyStation () { return my_station ; }
  X_Station_Base const * MyStation () const { return my_station ; }

  vector < X_SpaceTimeNode * > :: iterator firstNode ()
  { return _stn . begin () ; }
  vector < X_SpaceTimeNode * > :: iterator lastNode ()
  { return _stn . end () ; }
  vector < X_SpaceTimeNode * > :: const_iterator firstNode () const
  { return _stn . begin () ; }
  vector < X_SpaceTimeNode * > :: const_iterator lastNode () const
  { return _stn . end () ; }

} ;


class X_Platform_Creator
  : public Plat_Expander :: ObjectCreator
{
public :
  X_Platform_Creator ()
  { add_to_creators_list ( "x_platform" ) ; }
  X_Platform_Creator ( const char * x )
    : Plat_Expander :: ObjectCreator () { add_to_creators_list ( x ) ; }
  X_Platform_Base * create () const
  { cerr << "Error: X_Platform... default constructor prohibited" << endl ;
    exit ( 1 ) ; }
  X_Platform_Base * create ( TiXmlNode const * const & x ) const
  { return new X_Platform ( x ) ; }

} ;



class X_Platform_Container
  : public RS_ObjectContainer
{
public :
  bool create ( TiXmlNode const * & , vector < string > const & ,
                vector < RS_Object * > const & ) ;
  ~X_Platform_Container () {}
} ;




class X_IntraTime_Base
  : public virtual RS_Object
{
public :
  explicit X_IntraTime_Base ( TiXmlNode const * const & x ) : RS_Object () {}
  explicit X_IntraTime_Base ( string const & _n_in ) : RS_Object ( _n_in ) {}
  explicit X_IntraTime_Base ( X_IntraTime_Base const & x )
    : RS_Object ( x ) {}
  virtual X_IntraTime_Base & operator= ( X_IntraTime_Base const & x )
  { if ( & x == this ) return * this ;
    RS_Object :: operator= ( x ) ; return * this ; }

  virtual ~X_IntraTime_Base () {}

  virtual X_Platform_Base * From () = 0 ;
  virtual X_Platform_Base const * From () const = 0 ;
  virtual X_Platform_Base * To () = 0 ;
  virtual X_Platform_Base const * To () const = 0 ;
  virtual double Time () const = 0 ;

} ;



typedef Expandable < X_IntraTime_Base , TiXmlNode const * > ITime_Expander ;


class X_IntraTime
  : public X_IntraTime_Base ,
    public ITime_Expander
{
  // _from: origin platform.
  // _to: destination platform. to start.
  X_Platform_Base * _from ;
  X_Platform_Base * _to ;

  // _walk_time: walking time between platforms (places in a station).
  double _walk_time ;

  // These must be prohibited...
  explicit X_IntraTime ( X_IntraTime const & x ) ;
  X_IntraTime & operator= ( X_IntraTime const & x ) ;
  explicit X_IntraTime ( string const & ) ;

public :
  explicit X_IntraTime ( TiXmlNode const * const & ) ;
  ~X_IntraTime () {}

  X_Platform_Base const * From () const { return _from ; }
  X_Platform_Base * From () { return _from ; }
  X_Platform_Base const * To () const { return _to ; }
  X_Platform_Base * To () { return _to ; }
  double Time () const { return _walk_time ; }

  // Prohibited
  RS_ObjectContainer * getContainer ( string const & ) ;
  RS_ObjectContainer const * getContainer ( string const & ) const ;

  bool acceptInitObject ( string const & , RS_Object * ) ;

} ;


class X_IntraTime_Creator
  : public ITime_Expander :: ObjectCreator
{
public :
  X_IntraTime_Creator ()
  { add_to_creators_list ( "x_intra_sta" ) ; }
  X_IntraTime_Creator ( const char * x )
    : ITime_Expander :: ObjectCreator () { add_to_creators_list ( x ) ; }
  X_IntraTime_Base * create () const
  { cerr << "Error: X_IntraTime... default constructor prohibited" << endl ;
    exit ( 1 ) ; }
  X_IntraTime_Base * create ( TiXmlNode const * const & x ) const
  { return new X_IntraTime ( x ) ; }
} ;




typedef multimap < X_Platform_Base const * , X_IntraTime_Base * >
IntraTimeMap ;
typedef map < pair < X_Platform_Base const * , X_Platform_Base const * > ,
              X_IntraTime_Base * > IntraTimePMap ;
typedef pair < IntraTimeMap :: iterator ,
               IntraTimeMap :: iterator > IntraTimeRange ;
typedef pair < IntraTimeMap :: const_iterator ,
               IntraTimeMap :: const_iterator > const_IntraTimeRange ;



class X_IntraTime_Container
  : public RS_ObjectContainer
{
  IntraTimeMap from_map ;
  IntraTimeMap to_map ;
  IntraTimePMap p_map ;

public :
  X_IntraTime_Container () : RS_ObjectContainer () , from_map () ,
                             to_map () , p_map () {}
  bool create ( TiXmlNode const * & , vector < string > const & ,
                vector < RS_Object * > const & ) ;
  IntraTimeRange getIntraTimesFrom ( X_Platform_Base const * ) ;
  const_IntraTimeRange getIntraTimesFrom ( X_Platform_Base const * ) const ;
  IntraTimeRange getIntraTimesTo ( X_Platform_Base const * ) ;
  const_IntraTimeRange getIntraTimesTo ( X_Platform_Base const * ) const ;
  ~X_IntraTime_Container () {}
} ;




class X_Station_Base
  : public virtual RS_Object
{
public :
  explicit X_Station_Base ( TiXmlNode const * const & x ) : RS_Object () {}
  explicit X_Station_Base ( string const & _n_in ) : RS_Object ( _n_in ) {}
  explicit X_Station_Base ( X_Station_Base const & x ) : RS_Object ( x ) {}
  X_Station_Base & operator= ( X_Station_Base const & x )
  { if ( & x == this ) return * this ;
    RS_Object :: operator= ( x ) ; return * this ; }
  virtual ~X_Station_Base () {}

  virtual X_Platform_Base const * Entrance () const = 0 ;
  virtual X_Platform_Base * Entrance () = 0 ;
  virtual X_SpaceTimeNode const * EntranceNode () const = 0 ;
  virtual X_Platform_Base const * Exit () const = 0 ;
  virtual X_Platform_Base * Exit () = 0 ;
  virtual X_SpaceTimeNode const * ExitNode () const = 0 ;
  virtual X_Platform_Base * locatePlatform ( string const & ) = 0 ;
  virtual X_Platform_Base const * locatePlatform ( string const & )
    const = 0 ;

  virtual X_Platform_Container :: iterator firstPlatform () = 0 ;
  virtual X_Platform_Container :: iterator lastPlatform () = 0 ;
  virtual X_Platform_Container :: const_iterator firstPlatform ()
    const = 0 ;
  virtual X_Platform_Container :: const_iterator lastPlatform ()
    const = 0 ;

  virtual IntraTimeRange getIntraTimesFrom ( X_Platform_Base const * ) = 0 ;
  virtual const_IntraTimeRange getIntraTimesFrom ( X_Platform_Base const * )
    const = 0 ;
  virtual IntraTimeRange getIntraTimesTo ( X_Platform_Base const * ) = 0 ;
  virtual const_IntraTimeRange getIntraTimesTo ( X_Platform_Base const * )
    const = 0 ;

} ;



typedef Expandable < X_Station_Base , TiXmlNode const * > STA_Expander ;



class X_Station
  : public X_Station_Base ,
    public STA_Expander
{
  map < X_Station_Base const * , double > _demand ;
  X_Platform_Container * _platc ;
  X_IntraTime_Container * _itimec ;

  // "Platform" objects marked as entrance and exit.
  X_Platform_Base * _plat_in ;
  X_SpaceTimeNode * _node_in ;
  X_Platform_Base * _plat_out ;
  X_SpaceTimeNode * _node_out ;

  // Prohibiting duplicate calls of processPlatforms and processInterPlatTimes
  // subfunctions.
  bool _plat_processed ;
  bool _ipt_processed ;

  // These must be prohibited...
  explicit X_Station ( X_Station const & x ) ;
  X_Station & operator= ( X_Station const & x ) ;

public :
  explicit X_Station ( TiXmlNode const * const & ) ;
  explicit X_Station ( string const & ) ;
  ~X_Station ()
  { delete _platc ; delete _itimec ;
    delete _node_in ; delete _node_out ; }

  RS_ObjectContainer * getContainer ( string const & ) ;
  RS_ObjectContainer const * getContainer ( string const & ) const ;

  bool acceptInitObject ( string const & _sx , RS_Object * _ox )
  { return false ; }

  X_Platform_Base const * Entrance () const { return _plat_in ; }
  X_Platform_Base * Entrance () { return _plat_in ; }
  X_SpaceTimeNode const * EntranceNode () const { return _node_in ; }
  X_Platform_Base const * Exit () const { return _plat_out ; }
  X_Platform_Base * Exit () { return _plat_out ; }
  X_SpaceTimeNode const * ExitNode () const { return _node_out ; }

  X_Platform_Base * locatePlatform ( string const & ) ;
  X_Platform_Base const * locatePlatform ( string const & ) const ;

  X_Platform_Container :: iterator firstPlatform ()
  { return _platc -> begin () ; }
  X_Platform_Container :: iterator lastPlatform ()
  { return _platc -> end () ; }
  X_Platform_Container :: const_iterator firstPlatform () const
  { return _platc -> begin () ; }
  X_Platform_Container :: const_iterator lastPlatform () const
  { return _platc -> end () ; }

  IntraTimeRange getIntraTimesFrom ( X_Platform_Base const * x )
  { return _itimec -> getIntraTimesFrom ( x ) ; }
  virtual const_IntraTimeRange getIntraTimesFrom ( X_Platform_Base const * x )
    const { return _itimec -> getIntraTimesFrom ( x ) ; }
  virtual IntraTimeRange getIntraTimesTo ( X_Platform_Base const * x )
  { return _itimec -> getIntraTimesTo ( x ) ; }
  virtual const_IntraTimeRange getIntraTimesTo ( X_Platform_Base const * x )
    const { return _itimec -> getIntraTimesTo ( x ) ; }

private :

  void processPlatformsTag ( TiXmlNode const * ) ;
  void processIntraTimesTag ( TiXmlNode const * ) ;

} ;


class X_Station_Creator
  : public STA_Expander :: ObjectCreator
{
public :
  X_Station_Creator ()
  { add_to_creators_list ( "x_station" ) ; }
  X_Station_Creator ( const char * x )
    : STA_Expander :: ObjectCreator () { add_to_creators_list ( x ) ; }
  X_Station_Base * create () const
  { cerr << "Error: X_Station... default constructor prohibited" << endl ;
    exit ( 1 ) ; }
  X_Station_Base * create ( TiXmlNode const * const & x ) const
  { return new X_Station ( x ) ; }
} ;



class X_Station_Container
  : public RS_ObjectContainer
{
public :
  bool create ( TiXmlNode const * & , vector < string > const & ,
                vector < RS_Object * > const & ) ;
  X_Station_Base * locateStation ( string const & ) ;
  X_Station_Base const * locateStation ( string const & ) const ;
  ~X_Station_Container () {}
} ;




#endif // X_Station_HH
