// -*- C++ -*-

// ダイクストラ法による評価のデモ

#include "X_TheWorld.hh"




int
main
( int argc ,
  char * * argv )
{
  // -----
  // Get the "current working directory" for this program.
  // 現在のワーキングディレクトリを取得。
  // -----
  char * curpath = getcwd ( NULL , 0 ) ;
  if ( curpath == NULL )
  {
    cerr << "Error: Getting present working directory failed" << endl ;
    exit ( 1 ) ;
  }
  else
  {
    cerr << "CWD: " << curpath << endl ;
  }

  X_TheWorld my_world ( argc , argv , curpath ) ;

  my_world . prepareSpaceTimeNodes () ;
  my_world . prepareSpaceTimeBranches () ;

  //////////////////////////////////////////////////////////////////////

  /***********************************************************************

  // ----
  // Create and populate a vector of SpaceTimeBranch for starting the path
  // search, and a vector of passenger tokens.
  // ----
  vector < X_Passenger_Token * > _pas_f1 ;
  for ( X_Station_Container :: iterator ii = xs_c . begin () ;
        ii != xs_c . end () ; ++ ii )
  {
// ----
// ....
//  sent to X_ScheduleSet::setPlatformBranches(), until...
// ....
// ----

      // ----
      // Creation of X_Passenger_Token objects. If there is a demand data
      // from this particular station, then a X_Passenger_Token object per
      // destination station will be created. Each object will represent
      // ( D / C * T ) passengers, where D is the demand (number of
      // passengers) per cycle, C is the cycle time in [sec] and T is the
      // length in [sec] of the SpaceTimeBranch _stb_x.
      // ----
      X_SpaceTimeNode const * _n_s = _sx -> getStartSTN () ;
      X_SpaceTimeNode const * _n_e = _sx -> getEndSTN () ;
      int _t_s = _n_s -> getTime () ;
      int _cd_s = _n_s -> getCycleDelay () ;
      int _t_e = _n_e -> getTime () ;
      int _t_x = _t_e - _t_s ;
      _t_s += _t_x / 2 ;
      while ( _t_s >= X_SpaceTimeNode :: CYCLE_TIME )
      {
        _t_s -= X_SpaceTimeNode :: CYCLE_TIME ;
        ++ _cd_s ;
      }
      for ( X_OD_Data_Container :: const_iterator iy = p_ii -> firstOD () ;
            iy != p_ii -> lastOD () ; ++ iy )
      {
        X_OD_Data_Base const *
          x_od = dynamic_cast < X_OD_Data_Base const * > ( iy -> second ) ;
        if ( ! x_od )
        {
          cerr << "Error: dynamic cast failed in call for X_OD_Data."
               << endl ;
          exit ( 1 ) ;
        }
        double _dem_x = x_od -> getDemand () ;
        X_Station_Base const * _sta_dest = x_od -> getDestination () ;
        _dem_x *= _t_x ;
        _dem_x /= X_SpaceTimeNode :: CYCLE_TIME ;
        cerr << "  Passenger: \"" << _n_s -> getName () << "\"-|-\""
             << _n_e -> getName () << "\" -> \"" << _sta_dest -> getName ()
             << "\"" << endl << "    [" << _t_s << "/" << _cd_s << "]: "
             << _dem_x << " = " << _t_x << " / "
             << X_SpaceTimeNode :: CYCLE_TIME << " * " << x_od -> getDemand ()
             << endl ;
        X_Passenger_Token *
          _x_pfi = new X_Passenger_Token
          ( _sx , _sta_dest , _t_s , _cd_s , _dem_x ) ;
        _pas_f1 . push_back ( _x_pfi ) ;
        _sx -> addInitiatingPassenger ( _x_pfi ) ;
      }
    }

      //...
      // ----
      // Creation of a "cyclic" X_Passenger_Token object.
      // ----
      X_SpaceTimeNode const * _n_s = _sx -> getStartSTN () ;
      X_SpaceTimeNode const * _n_e = _sx -> getEndSTN () ;
      int _t_s = _n_s -> getTime () ;
      int _cd_s = _n_s -> getCycleDelay () ;
      int _t_e = _n_e -> getTime () ;
      int _t_x = _t_e - _t_s + X_SpaceTimeNode :: CYCLE_TIME ;
      _t_s += _t_x / 2 ;
      while ( _t_s >= X_SpaceTimeNode :: CYCLE_TIME )
      {
        _t_s -= X_SpaceTimeNode :: CYCLE_TIME ;
        ++ _cd_s ;
      }
      for ( X_OD_Data_Container :: const_iterator iy = p_ii -> firstOD () ;
            iy != p_ii -> lastOD () ; ++ iy )
      {
        X_OD_Data_Base const *
          x_od = dynamic_cast < X_OD_Data_Base const * > ( iy -> second ) ;
        if ( ! x_od )
        {
          cerr << "Error: dynamic cast failed in call for X_OD_Data."
               << endl ;
          exit ( 1 ) ;
        }
        double _dem_x = x_od -> getDemand () ;
        X_Station_Base const * _sta_dest = x_od -> getDestination () ;
        _dem_x *= _t_x ;
        _dem_x /= X_SpaceTimeNode :: CYCLE_TIME ;
        cerr << "  Passenger: \"" << _n_s -> getName () << "\"-|-\""
             << _n_e -> getName () << "\" -> \"" << _sta_dest -> getName ()
             << "\"" << endl << "    [" << _t_s << "/" << _cd_s << "]: "
             << _dem_x << " = " << _t_x << " / "
             << X_SpaceTimeNode :: CYCLE_TIME << " * " << x_od -> getDemand ()
             << endl ;
        X_Passenger_Token *
          _x_pfi = new X_Passenger_Token
          ( _sx , _sta_dest , _t_s , _cd_s , _dem_x ) ;
        _pas_f1 . push_back ( _x_pfi ) ;
        _sx -> addInitiatingPassenger ( _x_pfi ) ;
      }
    }
  }

  // ----
  // For each SpaceTimeBranch in vector _stb_st, perform path search
  // using a Dijkstra's algorithm solver. Solvers must be kept on the memory
  // after the algorithm has been run.
  // ----
  map < X_SpaceTimeBranch const * ,
    RS_Dijkstra < Sujic_Weight , X_SpaceTimeNode const * > * > _dk_solvers ;

  for ( vector < X_SpaceTimeBranch * > :: iterator
          istb = _stb_st . begin () ; istb != _stb_st . end () ; ++ istb )
  {
    // ----
    // Setting names.
    // ----
    string _stb_name = ( * istb ) -> getMiddleName () ;
    X_SpaceTimeNode const *
      _stb_ptr = ( * istb ) -> getPassengerArrivalNode () ;
    X_SpaceTimeNode const * _own_exit
      = ( * istb ) -> getStartSTN () -> getStation () -> getExit () ;

    // ----
    // Creating a Dijkstra's Algorithm solver.
    // ----
    RS_Dijkstra < Sujic_Weight , X_SpaceTimeNode const * > *
      _x_slv = new RS_Dijkstra < Sujic_Weight , X_SpaceTimeNode const * >
      ( Sujic_Weight ( LARGENUM , 0 , 0 , 0 ) ) ;
    _dk_solvers [ * istb ] = _x_slv ;

    // ----
    // Creating nodes in the Dijkstra's algorithm solver.
    // ----
    for ( X_Station_Container :: iterator ii = xs_c . begin () ;
          ii != xs_c . end () ; ++ ii )
    {
      X_Station_Base *
        p_ii = dynamic_cast < X_Station_Base * > ( ii -> second ) ;
      for ( vector < X_SpaceTimeNode * > :: const_iterator ix
              = p_ii -> firstNode () ;
            ix != p_ii -> lastNode () ; ++ ix )
      {
        _x_slv -> setNode ( ( * ix ) , ( * ix ) -> getName () ) ;
      }
      X_SpaceTimeNode const * _exit = p_ii -> getExit () ;
      _x_slv -> setNode ( _exit , _exit -> getName () ) ;
    }

    // ----
    // Creating the node representing passengers' arrival at the originating
    // station.
    // ----
    _x_slv -> setNode ( _stb_ptr , _stb_name ) ;

    // ----
    // Trains must have their own nodes at stations.
    // ----
    for ( X_ScheduleSet_Container :: iterator idc = xschs_c . begin () ;
          idc != xschs_c . end () ; ++ idc )
    {
      X_ScheduleSet_Base * p_xssb
        = dynamic_cast < X_ScheduleSet_Base * > ( idc -> second ) ;
      if ( p_xssb )
      {
        cerr << "Error: pointer to X_ScheduleSet_Base could not be "
             << "extracted out of X_ScheduleSet_Container" << endl ;
        exit ( 2 ) ;
      }
      for ( X_Train_Container :: iterator ii = p_xssb -> firstTrain () ;
            ii != p_xssb -> lastTrain () ; ++ ii )
      {
        X_Train_Base const *
          _trn = dynamic_cast < X_Train_Base const * > ( ii -> second ) ;
        if ( ! _trn )
        {
          cerr << "Error: dynamic cast to X_Train failed" << endl ;
          exit ( 102 ) ;
        }
        for ( vector < X_SpaceTimeNode * > :: const_iterator
                ix = _trn -> firstNode () ; ix != _trn -> lastNode () ;
              ++ ix )
        {
          _x_slv -> setNode ( ( * ix ) , ( * ix ) -> getName () ) ;
        }
      }
    }

    // ----
    // Creating branches at stations.
    // Note::: at stations, branch time length does not need consideration
    // of cycle delays.
    // ----
    for ( vector < X_SpaceTimeBranch * > :: iterator ii = _stb_st . begin () ;
          ii != _stb_st . end () ; ++ ii )
    {
      X_SpaceTimeNode const * _nns = ( * ii ) -> getStartSTN () ;
      X_SpaceTimeNode const * _nne = ( * ii ) -> getEndSTN () ;
      if ( ( * ii ) == ( * istb ) )
      {
        Sujic_Weight _wt_half = ( * ii ) -> getWeight () . getHalf () ;
        X_SpaceTimeNode const * _nnm = _stb_ptr ;
        _x_slv -> setBranch ( _nns , _nnm , _wt_half ) ;
        _x_slv -> setBranch
          ( _nnm , _nne , ( * ii ) -> getWeight () - _wt_half ) ;
        _x_slv -> setBranch
          ( _stb_ptr , _own_exit , Sujic_Weight ( 0 , 0 , 0 , 0 ) ) ;
      }
      else
      {
        _x_slv -> setBranch ( _nns , _nne , ( * ii ) -> getWeight () ) ;
      }
    }

    // ----
    // Creating other branches.
    // ----
    for ( vector < X_SpaceTimeBranch * > :: iterator ii = _stb_n . begin () ;
          ii != _stb_n . end () ; ++ ii )
    {
      X_SpaceTimeNode const * _nns = ( * ii ) -> getStartSTN () ;
      X_SpaceTimeNode const * _nne = ( * ii ) -> getEndSTN () ;
      _x_slv -> setBranch ( _nns , _nne , ( * ii ) -> getWeight () ) ;
    }

    // ----
    // Solver complete. Run the algorithm.
    // ----
    cerr << endl << "Solver complete: " << _stb_name << endl ;
    _x_slv -> run ( _stb_ptr , Sujic_Weight ( 0 , 0 , 0 , 0 ) ) ;

    // ----
    // Printing paths.
    // ----
    for ( X_Station_Container :: iterator ii = xs_c . begin () ;
          ii != xs_c . end () ; ++ ii )
    {
      X_Station_Base *
        p_ii = dynamic_cast < X_Station_Base * > ( ii -> second ) ;
      _x_slv -> printRoute ( p_ii -> getExit () ) ;
    }

    // ----
    // Putting passengers in containers.
    // ----
    for ( vector < X_Passenger_Token const * > :: const_iterator
            ix = ( * istb ) -> firstInitialPassenger () ;
          ix != ( * istb ) -> lastInitialPassenger () ; ++ ix )
    {
      deque < X_SpaceTimeNode const * > _pth = _x_slv -> getRoute
        ( ( * ix ) -> getDestination () -> getExit () ) ;
      for ( deque < X_SpaceTimeNode const * > :: const_iterator
              ipth = _pth . begin () ; ipth != _pth . end () - 1 ; ++ ipth )
      {
        map < X_STB_Key , X_SpaceTimeBranch * > :: iterator
          ibr = _stb_map . find ( X_STB_Key ( * ipth , * ( ipth + 1 ) ) ) ;
        if ( ibr != _stb_map . end () )
        {
          ibr -> second -> addPassenger ( * ix ) ;
        }
      }
    }
  }

  // ----
  // Checking passengers in the containers.
  // ----
  cerr << endl ;
  for ( vector < X_SpaceTimeBranch * > :: iterator
          ii = _stb_n . begin () ; ii != _stb_n . end () ; ++ ii )
  {
    double _pas_s = 0 ;
    double _pas_e = 0 ;
    for ( vector < X_Passenger_Token const * > :: const_iterator
            ix = ( * ii ) -> firstInitialPassenger () ;
          ix != ( * ii ) -> lastInitialPassenger () ; ++ ix )
    {
      _pas_e += ( * ix ) -> getPassengerNumber () ;
    }
    for ( vector < X_Passenger_Token const * > :: const_iterator
            ix = ( * ii ) -> firstPassenger () ;
          ix != ( * ii ) -> lastPassenger () ; ++ ix )
    {
      _pas_s += ( * ix ) -> getPassengerNumber () ;
      _pas_e += ( * ix ) -> getPassengerNumber () ;
    }
    cerr << "Branch \"" << ( * ii ) -> getStartSTN () -> getName ()
         << "\" -> \"" << ( * ii ) -> getEndSTN () -> getName ()
         << "\": " << _pas_s << " -|- " << _pas_e << endl ;
    for ( vector < X_Passenger_Token const * > :: const_iterator
            ix = ( * ii ) -> firstInitialPassenger () ;
          ix != ( * ii ) -> lastInitialPassenger () ; ++ ix )
    {
      cerr << "  [I] "
           << ( * ix ) -> getStartSTB () -> getStartStationName ()
           << ": [" << ( * ix ) -> getTime () << "/"
           << ( * ix ) -> getCycleDelay () << "] -> "
           << ( * ix ) -> getDestination () -> getName ()
           << ": " << ( * ix ) -> getPassengerNumber () << endl ;
    }
    for ( vector < X_Passenger_Token const * > :: const_iterator
            ix = ( * ii ) -> firstPassenger () ;
          ix != ( * ii ) -> lastPassenger () ; ++ ix )
    {
      cerr << "  [P] "
           << ( * ix ) -> getStartSTB () -> getStartStationName ()
           << ": [" << ( * ix ) -> getTime () << "/"
           << ( * ix ) -> getCycleDelay () << "] -> "
           << ( * ix ) -> getDestination () -> getName ()
           << ": " << ( * ix ) -> getPassengerNumber () << endl ;
    }
  }
  for ( vector < X_SpaceTimeBranch * > :: iterator
          ii = _stb_st . begin () ; ii != _stb_st . end () ; ++ ii )
  {
    double _pas_s = 0 ;
    double _pas_e = 0 ;
    for ( vector < X_Passenger_Token const * > :: const_iterator
            ix = ( * ii ) -> firstInitialPassenger () ;
          ix != ( * ii ) -> lastInitialPassenger () ; ++ ix )
    {
      _pas_e += ( * ix ) -> getPassengerNumber () ;
    }
    for ( vector < X_Passenger_Token const * > :: const_iterator
            ix = ( * ii ) -> firstPassenger () ;
          ix != ( * ii ) -> lastPassenger () ; ++ ix )
    {
      _pas_s += ( * ix ) -> getPassengerNumber () ;
      _pas_e += ( * ix ) -> getPassengerNumber () ;
    }
    cerr << "Branch \"" << ( * ii ) -> getStartSTN () -> getName ()
         << "\" -> \"" << ( * ii ) -> getEndSTN () -> getName ()
         << "\": " << _pas_s << " -|- " << _pas_e << endl ;
    for ( vector < X_Passenger_Token const * > :: const_iterator
            ix = ( * ii ) -> firstInitialPassenger () ;
          ix != ( * ii ) -> lastInitialPassenger () ; ++ ix )
    {
      cerr << "  [I] "
           << ( * ix ) -> getStartSTB () -> getStartStationName ()
           << ": [" << ( * ix ) -> getTime () << "/"
           << ( * ix ) -> getCycleDelay () << "] -> "
           << ( * ix ) -> getDestination () -> getName ()
           << ": " << ( * ix ) -> getPassengerNumber () << endl ;
    }
    for ( vector < X_Passenger_Token const * > :: const_iterator
            ix = ( * ii ) -> firstPassenger () ;
          ix != ( * ii ) -> lastPassenger () ; ++ ix )
    {
      cerr << "  [P] "
           << ( * ix ) -> getStartSTB () -> getStartStationName ()
           << ": [" << ( * ix ) -> getTime () << "/"
           << ( * ix ) -> getCycleDelay () << "] -> "
           << ( * ix ) -> getDestination () -> getName ()
           << ": " << ( * ix ) -> getPassengerNumber () << endl ;
    }
  }

  ***********************************************************************/
}
