// This may look like C code, but it is really -*- C++ -*-



#include "X_TheWorld.hh"
#include <iostream>
#include <cstdlib>
#include <pcre.h>
#include <unistd.h>
#include <vector>

using std :: cerr ;
using std :: endl ;
using std :: vector ;



// -----
// World constructor.
// -----
X_TheWorld :: X_TheWorld
( int argc ,
  char * * argv ,
  char * curpath )
  : RS_Object () ,
    xs_c () ,
    xttyp_c () ,
    xdpt_c () ,
    xschs_c () ,
    xpact_c () ,
//     _stb_map () ,
//     _stb_st () ,
    current_directory ( "" ) ,
    current_drive ( "" ) ,
    is_backslash ( false )
{
  // ----
  // Prepare XML reader.
  // ----
  RS_XmlReader xread ;
  xread [ "sujic_station" ] = & xs_c ;
  xread [ "sujic_traintype" ] = & xttyp_c ;
  xread [ "sujic_depot" ] = & xdpt_c ;
  xread [ "sujic_schedule_set" ] = & xschs_c ;
  xread [ "sujic_passenger_activity" ] = & xpact_c ;
  
  current_directory = curpath ;

  // -----
  // Check if the directory is "Windows" style. If windows, then separate the
  // drive name and the file path.
  // -----
  RTSS_Path_String check_win_path ( curpath ) ;
  is_backslash = check_win_path . isWindowsStyleFullPath ( current_drive ) ;
  if ( is_backslash )
  {
    current_directory = check_win_path ;
    cerr << "Windows path, dir = " << current_directory << ", drive = "
          << current_drive << "!" << endl ;
  }
  else
  {
    cerr << "Unix path, dir = " << current_directory << ", drive = "
          << current_drive << "!" << endl ;
  }

  // ----
  // There must be at least one command-line argument.
  // コマンド行引数は少なくともひとつ必要。
  // ----
  if ( argc == 1 )
  {
    printUsage () ;
    exit ( 1 ) ;
  }

  // -----
  // Assume the command-line arguments are file names, and process them.
  // コマンド行引数はすべてファイル名であると仮定し，処理する。
  // -----
  vector < string > _files ;
  for ( -- argc ; argc ; -- argc )
  {
    RTSS_Path_String control_file ( * ++ argv ) ;
    // -----
    // Filename check (2 and thereafter) ... all processed here.
    // ファイル名チェック(2 以降) ... すべてここで.
    // -----
    string controlfiledrive = "" ;
    string controlfiledir = "" ;
    string controlfilename = "" ;
    if ( ! control_file . processPath
         ( is_backslash , current_drive , current_directory ,
           controlfiledrive , controlfiledir , controlfilename ) )
    {
      cerr << "ERROR: unknown error in processPath (control file)" << endl ;
      exit ( 1 ) ;
    }
    current_drive = controlfiledrive ;
    current_directory = controlfiledir ;
    string _fname_in = controlfiledrive ;
    _fname_in += controlfiledir ;
    if ( is_backslash )
    {
      _fname_in += "\\" ;
    }
    else
    {
      _fname_in += "/" ;
    }
    _fname_in += controlfilename ;
    _files . push_back ( _fname_in ) ;
    cerr << "Control file: " << _fname_in << endl << " dir = "
         << current_directory << ", drive = " << current_drive << endl ;
    xread . openXmlDocument ( _fname_in , current_drive , current_directory ) ;
    do
    {
      xread . processElement () ;
    } while ( xread . hasNextElement () ) ;
  }
  cerr << endl << endl << "XML file Read Finished." << endl << endl ;
}




// ----
// Prepare SpaceTimeNode before evaluation.
// ----
void
X_TheWorld :: prepareSpaceTimeNodes
()
{
  // Set SpaceTimeNode's
  X_ScheduleSet_Base * x = xschs_c . firstScheduleSet () ;
  x -> setTimes ( & xs_c ) ;
  while ( x -> Superseding () )
  {
    x = x -> Superseding () ;
    x -> setTimes ( & xs_c ) ;
  }

  // ----
  // Sorting Space Time Nodes in X_Platform objects.
  // ----
  for ( X_Station_Container :: iterator ii = xs_c . begin () ;
        ii != xs_c . end () ; ++ ii )
  {
    X_Station_Base * x = dynamic_cast < X_Station_Base * > ( ii -> second ) ;
    if ( ! x )
    {
      cerr << "Error: X_Station pointer could not be retrieved from "
           << "X_Station_Container" << endl ;
      exit ( 1 ) ;
    }
    for ( X_Platform_Container :: iterator jj = x -> firstPlatform () ;
          jj != x -> lastPlatform () ; ++ jj )
    {
      X_Platform_Base *
        y = dynamic_cast < X_Platform_Base * > ( jj -> second ) ;
      if ( ! y )
      {
        cerr << "Error: X_Platform pointer could not be retrieved from the "
             << "X_Station object" << endl ;
        exit ( 1 ) ;
      }
      y -> sortTimes () ;
    }
  }
}





// ----
// Prepare SpaceTimeNode before evaluation.
// ----
void
X_TheWorld :: prepareSpaceTimeBranches
()
{
  X_ScheduleSet_Base * x = xschs_c . firstScheduleSet () ;
  x -> setPlatformBranches ( & xs_c ) ;
  x -> setTrainBranches ( & xdpt_c ) ;
  x -> setMiscBranches ( & xs_c ) ;

  for ( X_PassengerActivity_Container :: iterator ii = xpact_c . begin () ;
        ii != xpact_c . end () ; ++ ii )
  {
    X_PassengerActivity_Base * x_pact
      = dynamic_cast < X_PassengerActivity_Base * > ( ii -> second ) ;
    x_pact -> solveOriginal ( & xs_c , & xschs_c ) ;
  }

  x -> printResults () ;
}





// ----
// Subfunction --- prints usage before abort.
// ----
void
X_TheWorld :: printUsage
()
{
  cerr << "RTSS Sujic, version " << RTSS_VERSION << " (Build ID "
       << RTSS_BUILD_ID << ")" << endl << endl
       << "There must be at least one command-line argument "
       << "(path to input file(s)) to run the program." << endl ;
}
