// -*- C++ -*-

// ダイクストラ法のデモ

#include "X_Train.hh"
#include "X_Station.hh"


X_Train :: X_Train
( TiXmlNode const * const & curnode )
  : X_Train_Base ( curnode )
{
  if ( ! curnode )
  {
    cerr << "Error: node not set in X_Train constructor." << endl ;
    exit ( 1 ) ;
  }
  if ( curnode -> ValueStr () != "data" )
  {
    cerr << "Error: <" << curnode -> ValueStr ()
         << "> tag not allowed in X_Train constructor." << endl ;
    exit ( 1 ) ;
  }
  const TiXmlAttribute * attr = curnode -> ToElement () -> FirstAttribute () ;
  bool sw_name = false ;
  while ( attr )
  {
    if ( string ( attr -> Name () ) == "name" )
    {
      // Attribute "name"
      if ( sw_name )
      {
        // Strange ... two attributes of same name?
        // 同じ属性が2つある？... エラー
        cerr << "Error: two \"name\" attributes?" << endl ;
        exit ( 2 ) ;
      }
      setName ( attr -> ValueStr () ) ;
      sw_name = true ;
    }
    attr = attr -> Next () ;
  }
  if ( ! sw_name )
  {
    cerr << "Error: <data> tag without attribute \"name\" in initialising "
         << "an X_Train object." << endl ;
    exit ( 1 ) ;
  }

  // ----
  // Reading timetable data.
  // ----
  vector < X_TrainSet_Initialiser > _vx_tset ;
  _vx_tset . clear () ;
  bool _b_ssd = false ;
  bool _b_ssd_next = false ;
  string _str_stn ;
  for ( TiXmlNode const * curch = curnode -> FirstChild () ;
        curch ; curch = curch -> NextSibling () )
  {
    // ----
    // flags for <trainset> and <superseding>
    // ----
    while ( _vx_tset . size ()
            && ( ( _vx_tset . end () - 1 ) -> _action == "out"
                 || ( _vx_tset . end () - 1 ) -> _action == "c_out"
                 || ( _vx_tset . end () - 1 ) -> _action == "d_out" ) )
    {
      _vx_tset . pop_back () ;
    }
    while ( _vx_tset . size ()
            && ( _vx_tset . begin () -> _action == "out"
                 || _vx_tset . begin () -> _action == "c_out"
                 || _vx_tset . begin () -> _action == "d_out" ) )
    {
      _vx_tset . erase ( _vx_tset . begin () ) ;
    }
    for ( vector < X_TrainSet_Initialiser > :: iterator
            iv = _vx_tset . begin () ; iv != _vx_tset . end () ; ++ iv )
    {
      iv -> _action = "stay" ;
    }
    _b_ssd = _b_ssd_next ;

    // ----
    // Element only.
    // ----
    if ( curch -> Type () != TiXmlNode :: TINYXML_ELEMENT )
    {
      continue ;
    }

    // ----
    // <train_time> only.
    // ----
    if ( curch -> ValueStr () != "train_time" )
    {
      cerr << "Error: tag <" << curch -> ValueStr ()
           << "> not allowed in initialisation of X_Train object." << endl ;
      exit ( 1 ) ;
    }
    string _s_in ;
    string _p_in ;
    bool _d_in = false ;
    double _t_in ;

    // -----
    // (1) Get all attributes of this "train_time" tag.
    // -----
    map < string , string > _a_m ;
    getAttributeMap ( curch , _a_m ) ;

    // -----
    // (2) Find attribute "station" in the map. Required.
    // -----
    map < string , string > :: iterator ii = findAttributeInMap
      ( _a_m , "station" ,
        "in <train_time> in initialisation of x_train" , true ) ;
    _s_in = ii -> second ;
    _a_m . erase ( ii ) ;

    // -----
    // (3) Find attribute "platform" in the map. Required.
    // -----
    ii = findAttributeInMap
      ( _a_m , "platform" ,
        "in <train_time> in initialisation of x_train" , true ) ;
    _p_in = ii -> second ;
    _a_m . erase ( ii ) ;

    // -----
    // (4) Find attribute "status" in the map. Required.
    // -----
    ii = findAttributeInMap
      ( _a_m , "status" ,
        "in <train_time> in initialisation of x_train" , true ) ;
    if ( ii -> second == "DEP" )
    {
      _d_in = true ;
    }
    else if ( ii -> second == "ARR" )
    {
      _d_in = false ;
    }
    else
    {
      cerr << "Error: in <train_time> in initialisation of x_train: "
           << "value \"" << ii -> second
           << "\" not allowed in \"status\" attribute" << endl ;
      exit ( 1 ) ;
    }
    _a_m . erase ( ii ) ;

    // -----
    // (5) Find attribute "time" in the map. Required.
    // -----
    ii = findAttributeInMap
      ( _a_m , "time" ,
        "in <train_time> in initialisation of x_train" , true ) ;
    sscanf ( ii -> second . c_str () , "%lf" , & _t_in ) ;
    _a_m . erase ( ii ) ;

    // -----
    // (6) Check if attribute map is exhausted.
    // -----
    if ( _a_m . size () )
    {
      ii = _a_m . begin () ;
      cerr << "Error: in <train_time> in initialisation of x_train: "
           << "incorrect attribute \"" << ii -> first << "\" found" << endl ;
      exit ( 1 ) ;
    }

    // -----
    // (7) Reading subelements.
    // -----
    string _action_prev = "" ;
    for ( TiXmlNode const * _c_in = curch -> FirstChild () ;
          _c_in ; _c_in = _c_in -> NextSibling () )
    {
      // -----
      // (7-1) Ignore non-Element node.
      // -----
      if ( _c_in -> Type () != TiXmlNode :: TINYXML_ELEMENT )
      {
        continue ;
      }

      // ----
      // (7-2) If _c_in is of tag <trainset> or of tag <superseding>...
      // ----
      string _v_cin = _c_in -> ValueStr () ;
      if ( _v_cin == "trainset" )
      {
        // ----
        // (a1) Get all attributes of this tag.
        // ----
        map < string , string > _tsam ;
        getAttributeMap ( _c_in , _tsam ) ;
        string _tsammsg = "in reading trainset for train " ;
        _tsammsg += getName () ;

        // -----
        // (a2) Find attribute "class" in the map. Required.
        // -----
        map < string , string > :: iterator ij = findAttributeInMap
          ( _tsam , "class" , _tsammsg , true ) ;
        string _tset_class = ij -> second ;
        _tsam . erase ( ij ) ;

        // -----
        // (a3) Find attribute "name" in the map. Required.
        // -----
        ij = findAttributeInMap ( _tsam , "name" , _tsammsg , true ) ;
        string _tset_name = ij -> second ;
        _tsam . erase ( ij ) ;

        // -----
        // (a4) Find attribute "action" in the map. Required.
        // -----
        ij = findAttributeInMap ( _tsam , "action" , _tsammsg , true ) ;
        string _tset_action = ij -> second ;
        _tsam . erase ( ij ) ;

        // -----
        // (a5) Find attribute "target" in the map. Required.
        // -----
        ij = findAttributeInMap ( _tsam , "target" , _tsammsg , true ) ;
        string _tset_target = ij -> second ;
        _tsam . erase ( ij ) ;

        // -----
        // (a6) Check if attribute map is exhausted.
        // -----
        if ( _tsam . size () )
        {
          ij = _tsam . begin () ;
          cerr << "Error: " << _tsammsg << ": incorrect attribute \""
               << ij -> first << "\" found" << endl ;
          exit ( 1 ) ;
        }

        // -----
        // (a7) Create.
        // -----
        if ( _action_prev != "" && _tset_action != _action_prev )
        {
          cerr << "Error: different trainset action specified at the same "
               << "train time" << endl ;
          exit ( 1 ) ;
        }
        _action_prev = _tset_action ;
        bool _sw_caught = false ;
        int _f_ins = 1 ;
        for ( vector < X_TrainSet_Initialiser > :: iterator
                iv = _vx_tset . begin () ; iv != _vx_tset . end () ; ++ iv )
        {
          if ( iv -> _name == _tset_name )
          {
            if ( _tset_action == "out" || _tset_action == "c_out"
                 || _tset_action == "d_out" )
            {
              iv -> _action = _tset_action ;
              iv -> _class = _tset_class ;
              iv -> _target = _tset_target ;
            }
            else
            {
              cerr << "Error: trainset already exists, name = " << _tset_name
                   << endl ;
              exit ( 1 ) ;
            }
            _sw_caught = true ;
          }
        }
        if ( _tset_action == "b_in" || _tset_action == "c_b_in"
             || _tset_action == "d_b_in" || _tset_action == "cont" )
        {
          _vx_tset . push_back
            ( X_TrainSet_Initialiser ( _tset_name , _tset_action ,
                                       _tset_class , _tset_target ) ) ;
          _f_ins = 0 ;
        }
        else if ( _tset_action == "f_in" || _tset_action == "c_f_in"
                  || _tset_action == "d_f_in" )
        {
          if ( ! _f_ins )
          {
            cerr << "Error: in reading <trainset> tags: trainset "
                 << _tset_name << " not allowed at the front because other "
                 << "train(s) are already appended to the back of this train" ;
            exit ( 1 ) ;
          }
          _vx_tset . insert
            ( _vx_tset . begin () + ( _f_ins - 1 ) ,
              X_TrainSet_Initialiser ( _tset_name , _tset_action ,
                                       _tset_class , _tset_target ) ) ;
          ++ _f_ins ;
        }
        else if ( ! _sw_caught )
        {
          cerr << "Error: incorrect trainset specifier, tn/N/A/C/T = "
               << getName () << "/" << _tset_name << "/" << _tset_action
               << "/" << _tset_class << "/" << _tset_target << endl ;
          exit ( 1 ) ;
        }
      }
      else if ( _v_cin == "superseding" )
      {
        // ----
        // (b1) Get all attributes of this tag.
        // ----
        map < string , string > _tssd ;
        getAttributeMap ( _c_in , _tssd ) ;
        string _tssdmsg = "in reading tag <superseding> for train " ;
        _tssdmsg += getName () ;

        // -----
        // (b2) Find attribute "action" in the map. Required.
        // -----
        map < string , string > :: iterator ik = findAttributeInMap
          ( _tssd , "action" , _tssdmsg , true ) ;
        string _tssd_action = ik -> second ;
        _tssd . erase ( ik ) ;

        // -----
        // (b3) Find attribute "name" in the map. Required.
        // -----
        ik = findAttributeInMap ( _tssd , "name" , _tssdmsg , true ) ;
        string _tssd_name = ik -> second ;
        _tssd . erase ( ik ) ;

        // -----
        // (b4) Check if attribute map is exhausted.
        // -----
        if ( _tssd . size () )
        {
          ik = _tssd . begin () ;
          cerr << "Error: " << _tssdmsg << ": incorrect attribute \""
               << ik -> first << "\" found" << endl ;
          exit ( 1 ) ;
        }

        // -----
        // (b5) Create.
        // -----
        if ( _tssd_action == "start" )
        {
          _b_ssd = true ;
          _b_ssd_next = true ;
          _str_stn = _tssd_name ;
        }
        else if ( _tssd_action == "end" )
        {
          _b_ssd_next = false ;
          if ( _str_stn != _tssd_name )
          {
            cerr << "Error: <superseding action=\"end\"> tag with incorrect "
                 << "name" << endl ;
            exit ( 1 ) ;
          }
        }
        else
        {
          cerr << "Error: <superseding> tag with incorrect action "
               << _tssd_action << endl ;
          exit ( 1 ) ;
        }
      }
      else
      {
        cerr << "Error: tag <" << _v_cin << "> not allowed" << endl ;
        exit ( 1 ) ;
      }
    }

    // -----
    // (7a) Check if "out" trainsets are at either end of the train.
    // -----
    if ( _action_prev == "out" || _action_prev == "c_out"
         || _action_prev == "d_out" )
    {
      if ( _vx_tset . begin () -> _action == "out"
           || _vx_tset . begin () -> _action == "c_out"
           || _vx_tset . begin () -> _action == "d_out" )
      {
        bool _sw_f = false ;
        for ( vector < X_TrainSet_Initialiser > :: iterator
                iv = _vx_tset . begin () ; iv != _vx_tset . end () ; ++ iv )
        {
          if ( _sw_f && ( iv -> _action == "out" || iv -> _action == "c_out"
                          || iv -> _action == "d_out" ) )
          {
            cerr << "Error: action out at non-end trainset" << endl ;
            exit ( 1 ) ;
          }
          if ( iv -> _action != "out" && iv -> _action != "c_out"
               && iv -> _action != "d_out" )
          {
            _sw_f = true ;
          }
        }
      }
      else if ( _vx_tset . rbegin () -> _action == "out"
                || _vx_tset . rbegin () -> _action == "c_out"
                || _vx_tset . rbegin () -> _action == "d_out" )
      {
        bool _sw_b = false ;
        for ( vector < X_TrainSet_Initialiser > :: reverse_iterator
                iv = _vx_tset . rbegin () ; iv != _vx_tset . rend () ; ++ iv )
        {
          if ( _sw_b && ( iv -> _action == "out" || iv -> _action == "c_out"
                          || iv -> _action == "d_out" ) )
          {
            cerr << "Error: action out at non-end trainset" << endl ;
            exit ( 2 ) ;
          }
          if ( iv -> _action != "out" && iv -> _action != "c_out"
               && iv -> _action != "d_out" )
          {
            _sw_b = true ;
          }
        }
      }
      else
      {
        cerr << "Error: action out at non-end trainset" << endl ;
        exit ( 3 ) ;
      }
    }

    // -----
    // (8) Create a train time initialiser.
    // -----
    _si . push_back
      ( X_Train_Time_Initialiser ( _s_in , _p_in , _d_in , _t_in ,
                                   _vx_tset , _b_ssd , _str_stn ) ) ;
  }
}



bool
X_Train_Container :: create
( TiXmlNode const * & _n_in ,
  vector < string > const & _s_in ,
  vector < RS_Object * > const & _o_in )
{
  string xxx = getClassString ( _n_in ) ;
  string yyy = getNameString ( _n_in ) ;
  X_Train_Base * x = TRN_Expander :: create ( xxx , _n_in ) ;
  ( * this ) [ yyy ] = x ;
  vector < string > :: const_iterator ii = _s_in . begin () ;
  vector < RS_Object * > :: const_iterator jj = _o_in . begin () ;
  while ( ii < _s_in . end () || jj < _o_in . end () )
  {
    if ( x -> acceptInitObject ( * ii , * jj ) )
    {
      ++ ii ;
      ++ jj ;
    }
    else
    {
      cerr << "Error: acceptInitObject failed, type = "
           << * ii << endl ;
      exit ( 1 ) ;
    }
  }
  return true ;
}




X_Train_Base const *
X_Train_Container :: locateTrain
( string const & x )
  const
{
  RS_Object const * retv = locate ( x ) ;
  return dynamic_cast < X_Train_Base const * > ( retv ) ;
}




X_Train_Base *
X_Train_Container :: locateTrain
( string const & x )
{
  RS_Object * retv = locate ( x ) ;
  return dynamic_cast < X_Train_Base * > ( retv ) ;
}



bool
X_Train_Container :: setTimes
( X_Station_Container * _xsc )
{
  for ( map < string , RS_Object * > :: iterator _itrn = begin () ;
        _itrn != end () ; ++ _itrn )
  {
    X_Train_Base *
      _xtb = dynamic_cast < X_Train_Base * > ( _itrn -> second ) ;
    if ( ! _xtb )
    {
      cerr << "Error: failed to dynamic_cast X_Train_Base in setTimes"
           << endl ;
      exit ( 1 ) ;
    }
    for ( vector < X_Train_Time_Initialiser > :: iterator
            _iti = _xtb -> firstTimeInitialiser () ;
          _iti != _xtb -> lastTimeInitialiser () ; ++ _iti )
    {
      // ----
      // Find corresponding X_Station and X_Platform objects.
      // ----
      string _sta_name = _iti -> _station ;
      string _plat_name = _iti -> _platform ;
      bool _dep_in = _iti -> _dep ;
      double _time_in = _iti -> _t_ ;

      X_Station_Base * _xsb = _xsc -> locateStation ( _sta_name ) ;
      X_Platform_Base * _xpb = _xsb -> locatePlatform ( _plat_name ) ;

      // ----
      // Create SpaceTimeNode's at Train dwelling at Platform.
      // ----
      X_SpaceTimeNode * _trnd
        = _xpb -> setTime ( _xtb -> setTime ( _dep_in , _time_in ) ) ;

      // ----
      // Create SpaceTimeNode's at other platforms in the same station.
      // ----
      if ( _dep_in )
      {
        // ----
        // Departing node. Time to be subtracted.
        // ----
        IntraTimeRange _xitr = _xsb -> getIntraTimesTo ( _xpb ) ;
        for ( IntraTimeMap :: iterator ii = _xitr . first ;
              ii != _xitr . second ; ++ ii )
        {
          double _tdpo = _time_in - ii -> second -> Time () ;
          ii -> second -> From () -> setTime
            ( X_SpaceTimeNode ( _dep_in , _tdpo , _xtb ) , _trnd , _dep_in ) ;
        }
      }
      else
      {
        // ----
        // Arriving node. Time to be added.
        // ----
        IntraTimeRange _xitr = _xsb -> getIntraTimesFrom ( _xpb ) ;
        for ( IntraTimeMap :: iterator ii = _xitr . first ;
              ii != _xitr . second ; ++ ii )
        {
          double _taro = _time_in + ii -> second -> Time () ;
          ii -> second -> To () -> setTime
            ( X_SpaceTimeNode ( _dep_in , _taro , _xtb ) , _trnd , _dep_in ) ;
          // _dep_in : false
        }
      }
    }
  }
  return true ;
}



