// -*- C++ -*-

// ダイクストラ法のデモ

#include "X_TrainSet.hh"


X_TrainSet :: X_TrainSet
( string const & _n_in )
  : X_TrainSet_Base ( _n_in ) ,
    _trn_type ( 0 )
{
}



X_TrainSet :: X_TrainSet
( TiXmlNode const * const & curnode )
  : X_TrainSet_Base ( curnode ) ,
    _trn_type ( 0 )
{
  if ( ! curnode )
  {
    cerr << "Error: node not set in X_TrainSet constructor." << endl ;
    exit ( 1 ) ;
  }
  if ( curnode -> ValueStr () != "data" )
  {
    cerr << "Error: <" << curnode -> ValueStr ()
         << "> tag not allowed in X_TrainSet constructor." << endl ;
    exit ( 1 ) ;
  }

  // -----
  // (1) Get all attributes of this "data" tag.
  // -----
  map < string , string > _a_m ;
  getAttributeMap ( curnode , _a_m ) ;
  string _ein_msg = "in initialisation of an X_TrainSet instance" ;

  // -----
  // (2) Find attribute "class" in the map. Required (already checked ---
  // just discard it here).
  // -----
  map < string , string > :: iterator ii = findAttributeInMap
    ( _a_m , "class" , _ein_msg , true ) ;
  _a_m . erase ( ii ) ;

  // -----
  // (3) Find attribute "name" in the map. Required.
  // -----
  ii = findAttributeInMap ( _a_m , "name" , _ein_msg , true ) ;
  setName ( ii -> second ) ;
  _a_m . erase ( ii ) ;

  // -----
  // (4) Check if attribute map is exhausted.
  // -----
  if ( _a_m . size () )
  {
    ii = _a_m . begin () ;
    cerr << "Error: " << _ein_msg << ": incorrect attribute \""
         << ii -> first << "\" found" << endl ;
    exit ( 1 ) ;
  }
}



RS_ObjectContainer *
X_TrainSet :: getContainer
( string const & x )
{
  cerr << "Error: No class \"" << x << "\" found in X_TrainSet" << endl ;
  exit ( 1 ) ;
}




RS_ObjectContainer const *
X_TrainSet :: getContainer
( string const & x )
  const
{
  cerr << "Error: No class \"" << x << "\" found in X_TrainSet" << endl ;
  exit ( 1 ) ;
}




bool
X_TrainSet_Container :: create
( TiXmlNode const * & _n_in ,
  vector < string > const & _s_in ,
  vector < RS_Object * > const & _o_in )
{
  string xxx = getClassString ( _n_in ) ;
  string yyy = getNameString ( _n_in ) ;
  if ( find ( yyy ) != end () )
  {
    cerr << "Error: TrainSet container already has a TrainSet instance "
         << "of name \"" << xxx << "\"" << endl ;
    exit ( 1 ) ;
  }
  X_TrainSet_Base * x = TSet_Expander :: create ( xxx , _n_in ) ;
  ( * this ) [ yyy ] = x ;
  vector < string > :: const_iterator ii = _s_in . begin () ;
  vector < RS_Object * > :: const_iterator jj = _o_in . begin () ;
  while ( ii < _s_in . end () || jj < _o_in . end () )
  {
    if ( x -> acceptInitObject ( * ii , * jj ) )
    {
      ++ ii ;
      ++ jj ;
    }
    else
    {
      cerr << "Error: acceptInitObject failed, type = "
           << * ii << endl ;
      exit ( 1 ) ;
    }
  }
  return true ;
}




X_TrainSet_Base const *
X_TrainSet_Container :: locateTrainSet
( string const & x )
  const
{
  RS_Object const * retv = locate ( x ) ;
  return dynamic_cast < X_TrainSet_Base const * > ( retv ) ;
}




X_TrainSet_Base *
X_TrainSet_Container :: locateTrainSet
( string const & x )
{
  RS_Object * retv = locate ( x ) ;
  return dynamic_cast < X_TrainSet_Base * > ( retv ) ;
}




bool
X_TrainSet :: acceptInitObject
( string const & _t_in ,
  RS_Object * _o_in )
{
  if ( _t_in != "type" )
  {
    cerr << "Error: <using> tag with incorrect type \"" << _t_in
         << "\", which will not be accepted by X_TrainSet object." << endl ;
    exit ( 1 ) ;
  }
  if ( _trn_type )
  {
    cerr << "Error: _trn_type seems to have already been set. "
         << "Duplicate <using> tag?" << endl ;
    exit ( 1 ) ;
  }
  _trn_type = dynamic_cast < X_TrainType_Base * > ( _o_in ) ;
  if ( ! _trn_type )
  {
    cerr << "Error: dynamic cast failed in acceptInitObject, type = \""
         << _t_in << "\"." << endl ;
    exit ( 1 ) ;
  }
  return true ;
}
