// -*- C++ -*-

// ダイクストラ法のデモ

#include "Expandable.hh"
#include "RS_XmlReader.hh"
#include "X_TrainType.hh"

#ifndef X_TrainSet_HH
#define X_TrainSet_HH


#include <algorithm>
#include <iostream>
#include <vector>
#include <map>


using std :: cerr ;
using std :: endl ;
using std :: vector ;
using std :: map ;
using std :: sort ;




class X_TrainSet_Base
  : public virtual RS_Object
{
public :
  explicit X_TrainSet_Base ( TiXmlNode const * const & x ) : RS_Object () {}
  explicit X_TrainSet_Base ( string const & _n_in ) : RS_Object ( _n_in ) {}
  explicit X_TrainSet_Base ( X_TrainSet_Base const & x ) : RS_Object ( x ) {}
  X_TrainSet_Base & operator= ( X_TrainSet_Base const & x )
  { if ( & x == this ) return * this ;
    RS_Object :: operator= ( x ) ; return * this ; }
  virtual ~X_TrainSet_Base () {}

  virtual X_TrainType_Base const * Type () const = 0 ;

} ;



typedef Expandable < X_TrainSet_Base , TiXmlNode const * > TSet_Expander ;



class X_TrainSet
  : public X_TrainSet_Base ,
    public TSet_Expander
{
  // Data to be initialised by XML --- pointer to train type.
  X_TrainType_Base const * _trn_type ;

  // These must be prohibited...
  explicit X_TrainSet ( X_TrainSet const & x ) ;
  X_TrainSet & operator= ( X_TrainSet const & x ) ;

public :
  explicit X_TrainSet ( TiXmlNode const * const & ) ;
  explicit X_TrainSet ( string const & ) ;

  RS_ObjectContainer * getContainer ( string const & ) ;
  RS_ObjectContainer const * getContainer ( string const & ) const ;

  bool acceptInitObject ( string const & , RS_Object * ) ;

  X_TrainType_Base const * Type () const { return _trn_type ; }

} ;


class X_TrainSet_Creator
  : public TSet_Expander :: ObjectCreator
{
public :
  X_TrainSet_Creator () { add_to_creators_list ( "x_trainset" ) ; }
  X_TrainSet_Creator ( char const * x )
    : TSet_Expander :: ObjectCreator () { add_to_creators_list ( x ) ; }
  X_TrainSet_Base * create () const
  { cerr << "Error: X_TrainSet... default constructor prohibited" << endl ;
    exit ( 1 ) ; }
  X_TrainSet_Base * create ( TiXmlNode const * const & x ) const
  { return new X_TrainSet ( x ) ; }
} ;



class X_TrainSet_Container
  : public RS_ObjectContainer
{
public :
  bool create ( TiXmlNode const * & , vector < string > const & ,
                vector < RS_Object * > const & ) ;
  X_TrainSet_Base * locateTrainSet ( string const & ) ;
  X_TrainSet_Base const * locateTrainSet ( string const & ) const ;
  ~X_TrainSet_Container () {}
} ;




#endif // X_TrainSet_HH
