// -*- C++ -*-

// ダイクストラ法のデモ

#include "X_TrainType.hh"


X_TrainType :: X_TrainType
( string const & _n_in )
  : X_TrainType_Base ( _n_in ) ,
    _length ( 0 ) ,
    _coaches ( 0 ) ,
    _capacity ( 0 ) ,
    _seatings ( 0 )
{
}



X_TrainType :: X_TrainType
( TiXmlNode const * const & curnode )
  : X_TrainType_Base ( curnode ) ,
    _length ( 0 ) ,
    _coaches ( 0 ) ,
    _capacity ( 0 ) ,
    _seatings ( 0 )
{
  if ( ! curnode )
  {
    cerr << "Error: node not set in X_TrainType constructor." << endl ;
    exit ( 1 ) ;
  }
  if ( curnode -> ValueStr () != "data" )
  {
    cerr << "Error: <" << curnode -> ValueStr ()
         << "> tag not allowed in X_TrainType constructor." << endl ;
    exit ( 1 ) ;
  }

  // -----
  // (1) Get all attributes of this "data" tag.
  // -----
  map < string , string > _a_m ;
  getAttributeMap ( curnode , _a_m ) ;
  string _ein_msg = "in initialisation of an X_TrainType instance" ;

  // -----
  // (2) Find attribute "class" in the map. Required (already checked ---
  // just discard it here).
  // -----
  map < string , string > :: iterator ii = findAttributeInMap
    ( _a_m , "class" , _ein_msg , true ) ;
  _a_m . erase ( ii ) ;

  // -----
  // (3) Find attribute "name" in the map. Required.
  // -----
  ii = findAttributeInMap ( _a_m , "name" , _ein_msg , true ) ;
  setName ( ii -> second ) ;
  _a_m . erase ( ii ) ;

  // -----
  // (4) Find attribute "length" in the map. Required.
  // -----
  ii = findAttributeInMap ( _a_m , "length" , _ein_msg , true ) ;
  sscanf ( ii -> second . c_str () , "%lf" , & _length ) ;
  _a_m . erase ( ii ) ;

  // -----
  // (5) Find attribute "coaches" in the map. Required.
  // -----
  ii = findAttributeInMap ( _a_m , "coaches" , _ein_msg , true ) ;
  sscanf ( ii -> second . c_str () , "%d" , & _coaches ) ;
  _a_m . erase ( ii ) ;

  // -----
  // (6) Find attribute "capacity" in the map. Required.
  // -----
  ii = findAttributeInMap ( _a_m , "capacity" , _ein_msg , true ) ;
  sscanf ( ii -> second . c_str () , "%d" , & _capacity ) ;
  _a_m . erase ( ii ) ;

  // -----
  // (7) Find attribute "seatings" in the map. Required.
  // -----
  ii = findAttributeInMap ( _a_m , "seatings" , _ein_msg , true ) ;
  sscanf ( ii -> second . c_str () , "%d" , & _seatings ) ;
  _a_m . erase ( ii ) ;

  // -----
  // (8) Check if attribute map is exhausted.
  // -----
  if ( _a_m . size () )
  {
    ii = _a_m . begin () ;
    cerr << "Error: " << _ein_msg << ": incorrect attribute \""
         << ii -> first << "\" found" << endl ;
    exit ( 1 ) ;
  }
}



RS_ObjectContainer *
X_TrainType :: getContainer
( string const & x )
{
  cerr << "Error: No class \"" << x << "\" found in X_TrainType" << endl ;
  exit ( 1 ) ;
}




RS_ObjectContainer const *
X_TrainType :: getContainer
( string const & x )
  const
{
  cerr << "Error: No class \"" << x << "\" found in X_TrainType" << endl ;
  exit ( 1 ) ;
}




bool
X_TrainType_Container :: create
( TiXmlNode const * & _n_in ,
  vector < string > const & _s_in ,
  vector < RS_Object * > const & _o_in )
{
  string xxx = getClassString ( _n_in ) ;
  string yyy = getNameString ( _n_in ) ;
  if ( find ( yyy ) != end () )
  {
    cerr << "Error: TrainType container already has a TrainType instance "
         << "of name \"" << xxx << "\"" << endl ;
    exit ( 1 ) ;
  }
  X_TrainType_Base * x = TTP_Expander :: create ( xxx , _n_in ) ;
  ( * this ) [ yyy ] = x ;
  vector < string > :: const_iterator ii = _s_in . begin () ;
  vector < RS_Object * > :: const_iterator jj = _o_in . begin () ;
  while ( ii < _s_in . end () || jj < _o_in . end () )
  {
    if ( x -> acceptInitObject ( * ii , * jj ) )
    {
      ++ ii ;
      ++ jj ;
    }
    else
    {
      cerr << "Error: acceptInitObject failed, type = "
           << * ii << endl ;
      exit ( 1 ) ;
    }
  }
  return true ;
}




X_TrainType_Base const *
X_TrainType_Container :: locateTrainType
( string const & x )
  const
{
  RS_Object const * retv = locate ( x ) ;
  return dynamic_cast < X_TrainType_Base const * > ( retv ) ;
}




X_TrainType_Base *
X_TrainType_Container :: locateTrainType
( string const & x )
{
  RS_Object * retv = locate ( x ) ;
  return dynamic_cast < X_TrainType_Base * > ( retv ) ;
}
