// -*- C++ -*-

// ダイクストラ法のデモ

#include "Expandable.hh"
#include "RS_XmlReader.hh"

#ifndef X_TrainType_HH
#define X_TrainType_HH


#include <algorithm>
#include <iostream>
#include <vector>
#include <map>


using std :: cerr ;
using std :: endl ;
using std :: vector ;
using std :: map ;
using std :: sort ;




class X_TrainType_Base
  : public virtual RS_Object
{
public :
  explicit X_TrainType_Base ( TiXmlNode const * const & x ) : RS_Object () {}
  explicit X_TrainType_Base ( string const & _n_in ) : RS_Object ( _n_in ) {}
  explicit X_TrainType_Base ( X_TrainType_Base const & x ) : RS_Object ( x ) {}
  X_TrainType_Base & operator= ( X_TrainType_Base const & x )
  { if ( & x == this ) return * this ;
    RS_Object :: operator= ( x ) ; return * this ; }
  virtual ~X_TrainType_Base () {}

  virtual int Capacity () const = 0 ;
  virtual int SeatingCapacity () const = 0 ;
  virtual double Length () const = 0 ;
  virtual int NumberOfCoaches () const = 0 ;
  
} ;



typedef Expandable < X_TrainType_Base , TiXmlNode const * > TTP_Expander ;



class X_TrainType
  : public X_TrainType_Base ,
    public TTP_Expander
{
  // Data to be initialised by XML --- length in [m], number of coach(es),
  // capacity including standing passengers and seating capacity.
  double _length ;
  int _coaches ;
  int _capacity ;
  int _seatings ;

  // These must be prohibited...
  explicit X_TrainType ( X_TrainType const & x ) ;
  X_TrainType & operator= ( X_TrainType const & x ) ;

public :
  explicit X_TrainType ( TiXmlNode const * const & ) ;
  explicit X_TrainType ( string const & ) ;

  RS_ObjectContainer * getContainer ( string const & ) ;
  RS_ObjectContainer const * getContainer ( string const & ) const ;

  bool acceptInitObject ( string const & _sx , RS_Object * _ox )
  { return false ; }

  // Virtual functions returning data.
  int Capacity () const { return _capacity ; }
  int SeatingCapacity () const { return _seatings ; }
  double Length () const { return _length ; }
  int NumberOfCoaches () const { return _coaches ; }
} ;


class X_TrainType_Creator
  : public TTP_Expander :: ObjectCreator
{
public :
  X_TrainType_Creator () { add_to_creators_list ( "x_traintype" ) ; }
  X_TrainType_Creator ( char const * x )
    : TTP_Expander :: ObjectCreator () { add_to_creators_list ( x ) ; }
  X_TrainType_Base * create () const
  { cerr << "Error: X_TrainType... default constructor prohibited" << endl ;
    exit ( 1 ) ; }
  X_TrainType_Base * create ( TiXmlNode const * const & x ) const
  { return new X_TrainType ( x ) ; }
} ;



class X_TrainType_Container
  : public RS_ObjectContainer
{
public :
  bool create ( TiXmlNode const * & , vector < string > const & ,
                vector < RS_Object * > const & ) ;
  X_TrainType_Base * locateTrainType ( string const & ) ;
  X_TrainType_Base const * locateTrainType ( string const & ) const ;
  ~X_TrainType_Container () {}
} ;




#endif // X_TrainType_HH
