// -*- C++ -*-

#ifndef X_Weight_HH
#define X_Weight_HH

#include <iostream>
#include "RS_Object.hh"

using std :: ostream ;


const int LARGENUM = 10000 ;


class X_Weight_Base
  : public virtual RS_Object
{
public :
  X_Weight_Base () : RS_Object () {}
  X_Weight_Base ( string const & x ) : RS_Object ( x ) {}
  X_Weight_Base ( X_Weight_Base const & x ) : RS_Object ( x ) {}

  virtual ~X_Weight_Base () {}

  virtual X_Weight_Base * MyClone () const = 0 ;

  virtual X_Weight_Base * createHalf () const = 0 ;

  friend ostream & operator<< ( ostream & , X_Weight_Base const & ) ;
  virtual ostream & writeOut ( ostream & ) const = 0 ;

  friend bool operator< ( X_Weight_Base const & , X_Weight_Base const & ) ;
  virtual bool isLessThan ( X_Weight_Base const & ) const = 0 ;

  friend bool operator== ( X_Weight_Base const & , X_Weight_Base const & ) ;
  virtual bool isEqualTo ( X_Weight_Base const & ) const = 0 ;

  X_Weight_Base & operator+= ( X_Weight_Base const & ) ;
  virtual void addWeight ( X_Weight_Base const & ) = 0 ;
  
  X_Weight_Base & operator-= ( X_Weight_Base const & ) ;
  virtual void subtractWeight ( X_Weight_Base const & ) = 0 ;

} ;



class Sujic_Weight
  : public X_Weight_Base
{
  double _time ;
  int _transfers ;
  double _time_walk ;
  double _time_wait ;

public :

  Sujic_Weight ()
    : X_Weight_Base () , _time ( 0 ) , _transfers ( 0 ) , _time_walk ( 0 ) ,
      _time_wait ( 0 ) {}

  Sujic_Weight ( double _t, int _tf , double _twk , double _twt )
    : X_Weight_Base () , _time ( _t ) , _transfers ( _tf ) ,
      _time_walk ( _twk ) , _time_wait ( _twt ) {}
  
  Sujic_Weight ( Sujic_Weight const & x )
    : X_Weight_Base ( x ) , _time ( x . _time ) ,
      _transfers ( x . _transfers ) , _time_walk ( x . _time_walk ) ,
      _time_wait ( x . _time_wait ) {}

  virtual ~Sujic_Weight () {}
  X_Weight_Base * MyClone () const { return new Sujic_Weight ( * this ) ; }

  X_Weight_Base * createHalf () const
  { return new Sujic_Weight
      ( _time / 2 , _transfers / 2 , _time_walk / 2 , _time_wait / 2 ) ; }

  ostream & writeOut ( ostream & out ) const
  { out << _time << "(" << _transfers << "/" << _time_walk << "/"
        << _time_wait << ")" ; return out ; }

  bool isLessThan ( X_Weight_Base const & ) const ;
  bool isEqualTo ( X_Weight_Base const & ) const ;
  void addWeight ( X_Weight_Base const & ) ;
  void subtractWeight ( X_Weight_Base const & ) ;

  double Time () const { return _time ; }
  int Transfers () const { return _transfers ; }
  double WalkingTime () const { return _time_walk ; }
  double WaitTime () const { return _time_wait ; }

} ;

#endif // X_Weight_HH
